<?php
/** Zend_Controller_Action */
//require_once 'Zend/Controller/Action.php';

class ApiController extends Zend_Controller_Action
{
		protected $_response = "";

		private function _initResponse()
		{
			$this->_response = new SimpleXMLElement("<response />");
		}

		private function _returnResponse()
		{
			header("Content-type: application/xml");
			echo $this->_response->asXML();
		}

		public function init()
		{
			$this->config = Zend_Registry::get("config");
			$this->_initResponse();
			$this->_helper->viewRenderer->setNoRender(true);
		}

    public function indexAction()
    {
	 		$function = $this->_getParam("function");
			$args = $this->_getParam("args", array());
			$events = $this->_getParam("events", array());
			$extent = $this->_getParam("extent", false);
			
	 		try
			{
				MapChat_Session::startSession();
				
				// If an extent coordinates array is included in the POST parameters, set it in the
				// registry, so that any subsequent operations can use it.
				if ($extent) Zend_Registry::set("client_extent",$extent);
				
				// Store the XML Response in the Registry so our API functions can add to it.
				Zend_Registry::set("XMLResponse", $this->_response);
				
				// If any events were included in the parameters, log them to the current discussion.
				$reg = Zend_Registry::getInstance();
				if (count($events)>0 && isset($reg["discussion"]) && $reg["discussion"]->exists())
				{
					$reg["discussion"]->Event_logQueue($events);
				}
				
				// If there is a current discussion in the registry, update the user's last online time.
				if ($reg["discussion"] && $reg["discussion"]->exists()) $reg["discussion"]->Event_userOnline();
				
				// Load the Reflection Class, which will load our API Classes.
				$l = new MapChat_Loader();

				// Get the response from the Class (ie, actually load/reflect/execute the class)
				$this->_response = $l->LoadClass($function, $args);
				
				// If a function returns false, do nothing on the response (e.g., maybe 
				// the function returns images to the browser instead of XML).
				if ($this->_response !== false)
				{
					
					// If there are any events currently stored in the session, add them to the response, then
					// reset the session variable ClientEvents to an empty array:
					if (count(MapChat_Session::ns()->ClientEvents)>0)
					{
						$this->_response->events = Zend_Json::encode(MapChat_Session::ns()->ClientEvents);
						MapChat_Session::ns()->ClientEvents = array();
					}
					
					// Return our Response
					$this->_returnResponse();
				}
			}
			catch(MapChat_Exception $e)
			{
				$this->_initResponse();

				$this->_response->addChild("error");
				$this->_response->error->addChild("msg", $e->getMessage());
				if($this->config->debug == true)
				{
					$this->_response->error->addChild("code", $e->getError());
					$this->_response->error->addChild("line", $e->getLineNumber());
					$this->_response->error->addChild("file", $e->getFileName());
					$this->_response->error->addChild("backtrace",@json_encode(debug_backtrace()));
				}
				// Return our Response
				$this->_returnResponse();
			}
			catch(Exception $e)
			{
				$this->_initResponse();

				$this->_response->addChild("error");
				if($this->config->debug == true)
				{
					$this->_response->error->addChild("msg", $e->getMessage());
					$this->_response->error->addChild("code", $e->getCode());
					$this->_response->error->addChild("line", $e->getLine());
					$this->_response->error->addChild("file", $e->getFile());
					$this->_response->error->addChild("backtrace",@json_encode(debug_backtrace()));
				}
				else
				{
					$this->_response->error->addChild("msg", "Please contact the website administrator");
				}
				
				// Return our Response
				$this->_returnResponse();
			}
    }
}

?>