<?php
/** Zend_Controller_Action */
//require_once 'Zend/Controller/Action.php';

class IndexController extends Zend_Controller_Action
{
	protected $_redirector = null;
	
	public function init()
	{
		$this->_redirector = $this->_helper->getHelper('Redirector');
	}
	
	public function indexAction()
	{
		
		// Redirect to https if we're not already in HTTPS mode and useHTTPS is set to true in the config:
		if (Zend_Registry::get("config")->useHTTPS && 
			!(isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"]) &&
			!isset(MapChat_Session::ns()->load_discussion))
		{
			$this->_redirector->gotoUrl("https://".$_SERVER["HTTP_HOST"].Zend_Registry::get("request")->getRequestUri());
		}
				
		// Only start the session if using cookies, or if an SID was given in the request (i.e., sessionExists).
		try
		{
			MapChat_Session::startSession();
		}
		catch (MapChat_Exception $e)
		{
			// This would happen on startup, where no 'sid' is supplied in the URL, and cookies are disabled.
			if ($e->getError()==1)
			{
				Zend_Session::start(); // Start a session, so we will have an ID that we can return...
				$this->_redirector->gotoUrl("?sid=".Zend_Session::getId()); // redirect the browser so it has the 'sid' in the URL.
			}
			else
			{
				echo "Error (".$e->getError()."): ".$e->getMessage();
				exit();
			}
		}
		catch (Exception $e)
		{
				echo "Error (".$e->getCode()."): ".$e->getMessage();
				exit();
		}
		
		// Next, make sure the database can be connected:
		try
		{
			Zend_Registry::get("db")->getConnection();
		}
		catch (Exception $e)
		{
			// It is possible that the db hasn't been setup yet - if the client is
			// 127.0.0.1, and the dbname is in the error message (a likely inidicator
			// that the server is running, but the db is not accessible), then redirect
			// to the setup controller:
			if (in_array($_SERVER["REMOTE_ADDR"],array("127.0.0.1","::1")))
			{
				$redirector = $this->_helper->getHelper('Redirector');
				$redirector->setGoto("index", "setup");
			}
			else
			{
				exit("Error (".$e->getCode()."):".$e->getMessage());
			}
		}
		
		// All good...lets get the session info:
		$this->view->sessionInfo = MapChat_Session::getSessionInfo();
		
		// And the debug setting:
		$this->view->debug = Zend_Registry::get("config")->debug;
		
		// Determine whether to include all scripts in the document:
		$this->view->scripts = Zend_Registry::get("config")->debug && !Zend_Registry::get("config")->minify;
	}
}

?>