<?php
/** Zend_Controller_Action */
//require_once 'Zend/Controller/Action.php';

class RegistrationController extends Zend_Controller_Action
{
	public function init()
	{
		$this->db = Zend_Registry::get("db");
		try
		{
			$this->db->getConnection();
		}
		catch (Exception $e)
		{
			exit("Error: The MapChat database is not available.");
		}
		$this->_redirector = $this->_helper->getHelper('Redirector');
	}
	
	public function indexAction()
	{
		
		// Redirect to https if we're not already in HTTPS mode and useHTTPS is set to true in the config:
		if (Zend_Registry::get("config")->useHTTPS && 
			!(isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"]) &&
			!isset(MapChat_Session::ns()->load_discussion))
		{
			$this->_redirector->gotoUrl("https://".$_SERVER["HTTP_HOST"].Zend_Registry::get("request")->getRequestUri());
		}
				
		// Only start the session if using cookies, or if an SID was given in the request (i.e., sessionExists).
		try
		{
			MapChat_Session::startSession();
		}
		catch (MapChat_Exception $e)
		{
			// This would happen on startup, where no 'sid' is supplied in the URL, and cookies are disabled.
			if ($e->getError()==1)
			{
				Zend_Session::start(); // Start a session, so we will have an ID that we can return...
				$this->_redirector->gotoUrl("registration?sid=".Zend_Session::getId()); // redirect the browser so it has the 'sid' in the URL.
			}
			else
			{
				echo "Error (".$e->getError()."): ".$e->getMessage();
				exit();
			}
		}
		catch (Exception $e)
		{
				echo "Error (".$e->getCode()."): ".$e->getMessage();
				exit();
		}
		
		// Next, make sure the database can be connected:
		try
		{
			Zend_Registry::get("db")->getConnection();
		}
		catch (Exception $e)
		{
			exit("Error (".$e->getCode()."):".$e->getMessage());
		}
		
		// All good...lets get the session info:
		$this->view->sessionInfo = MapChat_Session::getSessionInfo();
		
		// And the debug setting:
		$this->view->debug = Zend_Registry::get("config")->debug;
		
		// Determine whether to include all scripts in the document:
		$this->view->scripts = Zend_Registry::get("config")->debug && !Zend_Registry::get("config")->minify;
		
		// Setup the Captcha information used for registration:
		$this->view->captchaID = MapChat_Captcha::generate();
		$this->view->CaptchaConfig = Zend_Registry::get("config")->CaptchaConfig;
		
		
	}
	
	public function confirmAction()
	{
		// Get the parameters submitted with the request:
		$params = $this->getRequest()->getParams();
		
		// Redirect to https if we're not already in HTTPS mode and useHTTPS is set to true in the config:
		if (Zend_Registry::get("config")->useHTTPS && 
			!(isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"]) &&
			!isset(MapChat_Session::ns()->load_discussion))
		{
			$this->_redirector->gotoUrl("https://".$_SERVER["HTTP_HOST"].Zend_Registry::get("request")->getRequestUri());
		}
				
		// Only start the session if using cookies, or if an SID was given in the request (i.e., sessionExists).
		try
		{
			MapChat_Session::startSession();
		}
		catch (MapChat_Exception $e)
		{
			// This would happen on startup, where no 'sid' is supplied in the URL, and cookies are disabled.
			if ($e->getError()==1)
			{
				Zend_Session::start(); // Start a session, so we will have an ID that we can return...
				$this->_redirector->gotoUrl("registration/confirm?sid=".Zend_Session::getId()."&c=".$params["c"]); // redirect the browser so it has the 'sid' in the URL.
			}
			else
			{
				echo "Error (".$e->getError()."): ".$e->getMessage();
				exit();
			}
		}
		catch (Exception $e)
		{
				echo "Error (".$e->getCode()."): ".$e->getMessage();
				exit();
		}
		
		// Next, make sure the database can be connected:
		try
		{
			Zend_Registry::get("db")->getConnection();
		}
		catch (Exception $e)
		{
			exit("Error (".$e->getCode()."):".$e->getMessage());
		}
		
		// All good...lets get the session info:
		$this->view->sessionInfo = MapChat_Session::getSessionInfo();
		
		// And the debug setting:
		$this->view->debug = Zend_Registry::get("config")->debug;
		
		// Determine whether to include all scripts in the document:
		$this->view->scripts = Zend_Registry::get("config")->debug && !Zend_Registry::get("config")->minify;
		
		// Setup the Captcha information used for registration:
		$this->view->captchaID = MapChat_Captcha::generate();
		$this->view->CaptchaConfig = Zend_Registry::get("config")->CaptchaConfig;
		
		$user = new MapChat_User();
		
		// Check if a valid confirmation code was provided
		$this->view->validConf = $user->confirmRegistration($params["c"]);
		if ($this->view->validConf)
		{
			// Get the user's display name
			$this->view->displayName = $user->displayName();
			$this->view->code = $params["c"];
			$this->view->sid = Zend_Session::getId();
			
			if (isset($params["password"]))
			{
				// If a password is included, try validating the user...
				try
				{
					$user->completeRegistration($params["c"],$params["password"]);
					$this->view->regCompleted = true;
					$auth = new MapChat_Auth();
					MapChat_Session::ns()->me = $user;
					$auth->authenticate($user,$params["password"]);
				}
				catch (MapChat_Exception $e)
				{
					$this->view->regCompleted = false;
				}
				
				// Track that an attempt was made:
				$this->view->confAttempted = true;
			}
			else
			{
				$this->view->confAttempted = false;
			}
		}
	}
}

?>