<?php

// Add the library to the include path:
$path = '../library';
set_include_path(get_include_path() . PATH_SEPARATOR . $path);

// Include Zend's loader class, and setup the application to
// autoload classes located in the include path, where the
// include path for a given class is determined by its name:
include 'Zend/Loader.php';
function __autoload($class)
{
   Zend_Loader::loadClass($class);
}

// Load the application config:
$config = new Zend_Config(require('config.php'));
Zend_Registry::set('config', $config);

// Setup the Zend Session config:
Zend_Session::setOptions($config->session->toArray());

// Setup the Zend_Auth instance:
$auth = Zend_Auth::getInstance();
Zend_Registry::set("auth",$auth);

// Setup the PostgreSQL database connection:
$db = Zend_Db::factory('PDO_PgSQL', $config->database->toArray());
Zend_Registry::set('db', $db);

// Save a connection string (for PostGIS layers in MapServer - empty parameters such as the
// port or password can confuse mapserver, so this saves having to figure it all out later on):
$pg_conn = "dbname=".$config->database->dbname.
	(strlen($config->database->host)?" host=".$config->database->host:"").
	(strlen($config->database->port)?" port=".$config->database->port:"").
	(strlen($config->database->username)?" user=".$config->database->username:"").
	(strlen($config->database->password)?" password=".$config->database->password:"");
Zend_Registry::set("pg_conn",$pg_conn);

// Seems like $_REQUEST, $_POST and $_GET are inaccessible, use this object to get post/get params:
Zend_Registry::set("request",new Zend_Controller_Request_Http());

try
{
	/* Set the singleton instance of the front controller */
	$frontController = Zend_Controller_Front::getInstance();

	/* Disable error handler so it doesn't intercept all those errors we enabled above */
	if($config->debug == true)
	{
		$frontController->throwExceptions(true);
	} 
	else
	{
		$frontController->throwExceptions(false);
	}

	/* Point the front controller to your action controller directory */
	$frontController->setControllerDirectory($config->MVCDir.'/controllers');
	
	/* Register the MapChat_KaMapRedirector to redirect Ka-Map requests for init.php, tile.php, etc... */
	$frontController->registerPlugin(new MapChat_KaMapRedirector());
	
	/* OK, do your stuff, front controller */
	$frontController->dispatch();
}
catch(Exception $e)
{
	echo "Error: ".$e->getMessage()."";
}

?>