// Aliases that can help reduce text:
var F = false, T = true, N = undefined, D = "disabled";

// MapChat will be the global namespace for MapChat functions/variables.
$.extend(MapChat,{useCookies:F,scriptsLoaded:[],cssLoaded:[],jsScripts:{},eventsQueue:[],timeDelta:0,singletons:[],nextID:0,events:$({})});

// Kill the document.write function - wz_dragdrop.js has a document.write
// command in SET_DHTML(); that is meant to add HTML for ancient browsers, 
// but it still writes a blank string for modern browsers - MapChat doesn't
// need to run on old browsers, and calling document.write() after the 
// document has already been loaded in the browser is bad form, and actually
// crashes FF (maybe IE too - didn't bother checking).  I'm trying to avoid
// editing third-party code as much as possible, so this solves this problem:
document.write = function(){return F;};

$.extend(MapChat,
{
	/**
	 * Run API functions, with the function name corresponding to the
	 * class name in the MapChat API library, and an object array of
	 * arguments, where each argument must be named as a numbered
	 * item in an 'args[]' array, or a named item where the 
	 *
	 * @param string func name of the API function to be called
	 * @param object argArr object array with elements named as 'args[0]', 
	 *   'args[1]', etc., or as 'args[paramName1]', 'args[paramName2]' etc.
	 * @param object opts - defines the asynchronous behaviour of the ajax request.  For
	 *  compatibility with older methods, if opts is a true or a function, the request 
	 *  will be treated as asynchronous (with opts as the 'complete' callback if it is 
	 *  a function).  Otherwise, the properties of opts should be values that correspond
	 *  to the options available for the $.ajax() method.
	 *   Appropriate properties for aopts would be:
	 *    async: if true, the request will be asynchronous.
	 *    complete: if defined/true, the request is sent asyncrhonously.  If async is 
	 *              a function, then it is set as the callback for the $.ajax request.
	 *    error: if this is set as a function, it will be used to handle errors for
	 *           the current request.
	 */
	RunFunction: function(func, argArr, opts)
	{
		var Response = F, args = {};
		if (!argArr) argArr = {};
		
		// Only add sid to the parameters if we're not using cookies:
		if (!MapChat.useCookies && MapChat.session.sid) argArr.sid = MapChat.session.sid;
		
		// Allows an associative-array called 'args' to be used, instead of individual
		// parameters specified as 'args[argname]':
		if (argArr.args)
		{
			$.each(argArr.args,function(n,v){ argArr["args["+n+"]"] = v; });
			delete argArr.args;
		}
		
		//var postOpts = { function: func };
		var postOpts = $.extend({ "function": func }, argArr);
		
		// Add a list of events to the POST parameters if any have been queued.
		if (MapChat.eventsQueue.length>0)
		{
			postOpts["events[]"] = [];
			while (e = MapChat.eventsQueue.shift())
			{
				postOpts["events[]"].push($.toJSON(e));
			}
		}
		
		// If the server-side API function requires the client's current map extent, it is possible to 
		// add a paramter called 'extent[]' with kaMap.getGeoExtents() output - the ApiController.php will
		// check for this this and set in the Zend_Registry as "client_extent", which can be used within the
		// context of the API function call.
		
		// Default ajax options:
		var aopts = {"data": postOpts, "cache": F};
		
		// Examine the opts argument, and alter the $.ajax options accordingly:
		if (opts && opts===T) aopts.async = T; // Just do it asynchronously, no questions asked.
		else if (opts && $.isFunction(opts)) $.extend(aopts,{async:T,complete:opts});  // Asynchronous, with a 'complete' callback
		else if (typeof(opts)=="object") $.extend(aopts,opts); // Whatever override options are provided.
		
		// For synchronous requests, pass the XMLHttpRequest object through the 
		// MapChat.CheckErrors and MapChat.ParseEvents functions, and return the result.
		if (!aopts.async) return MapChat.CheckErrors(MapChat.ParseEvents($.ajax(aopts))).responseXML;
		
		// For asynchronous requests, just run the request - presumably sufficient callbacks are provided.
		else
		{
			// If there is a 'complete' callback function, then we need to substitute it with one that
			// processes ParseEvents first.
			if (aopts.complete)
			{
				var new_aopts = {};
				$.each(aopts,function(i,v){
						if (i!="complete")
							new_aopts[i] = v;
						else
							new_aopts[i] = function(xhr,s){v(MapChat.CheckErrors(MapChat.ParseEvents(xhr)),s);};
				});
				$.ajax(new_aopts);
			}
			else $.ajax(aopts);
		}
	},
	
	/**
	 * Queues event data to be submitted to the server upon the next
	 * RunFunction call.  This makes use of the MapChat.timeDelta variable
	 * in order to record a timestamp that matches the server's time.
	 *
	 * @param integer type - the type of event in the server database.
	 * @param string text - text that describes this event.
	 * @param array data - array of data relevant to this event.
	 * @param Object map - if specified, the extents of this kaMap object will
	 *  be included in the data sent.
	 * @param string geom - geometry in well-known-text format
	 *  for a geometry associated with this event.
	 * @param boolean noTime - if true, the local time is not recorded (under the
	 *  assumption that this event will be submitted immediately).
	 * @param integer event_parent - if specified, the id of a previously recorded
	 *  event will be included as the parent_id for the event.
	 * @param Object queue_parent - if specified, an event currently in the queue
	 *  will be used as the parent this event - the actual parent_id will be 
	 *  determined after it has been logged in the server database.
	 */
	QueueEvent: function(type,text,data,map,geom,noTime,event_parent,queue_parent)
	{
		var qe = {text:text,data:data,geom:geom,type:type};
		if (!noTime) qe.time=(new Date()).getTime()+MapChat.timeDelta;
		if (event_parent) qe.event_parent = event_parent;
		if (queue_parent) qe.queue_parent = $.inArray(queue_parent,MapChat.eventsQueue);
		if (map) qe.extent=map.getGeoExtents();
		MapChat.eventsQueue.push(qe);
		if (MapChat.eventsQueue.length>20) MapChat.PushQueue();
	},
	
	/**
	 * Pushes any queued events to the server by calling an arbitrary RunFunction.
	 */
	PushQueue: function()
	{
		MapChat.RunFunction("Event_PushQueue");
	},
	
	/**
	 * Check an XML document for an <events> tag - if found, trigger
	 * each event with the parameters included with it (the event must
	 * be added to the MapChat.events object using the jQuery bind
	 * function: MapChat.events.bind([name],[function]).  If no events
	 * tag is found, return the xml object.
	 *
	 * @param Object xhr the XMLHttpRequest object to check for events.
	 */
	ParseEvents: function(xhr)
	{
		var e = $("events:first",xhr.responseXML);
		if (e && e.length!=0)
		{
			e = $.parseJSON(e.text());
			for (var i=0;i<e.length;i++) MapChat.events.trigger(e[i][0],[e[i][1]]);
		}
		return xhr;
	},
	
	/**
	 * Check an XML document for <error> tags - alert the first
	 * one to the user and return false.  If no error tags are found,
	 * return the xml object.
	 *
	 * @param Object xhr the XMLHttpRequest object to check for errors.
	 */
	CheckErrors: function(xhr)
	{
		e = $(xhr.responseXML).find("error:first");
		if (e.length>0)
		{
			alert($(e).find("code:first").text()+": "+$(e).find("msg:first").text());
			$.unblockUI();
			return F;
		}
		else
		{
			return xhr;
		}
	},
	
	/**
	 * Dynamically & synchronously load scripts as needed from the server.
	 *
	 * @param string url - location to retrieve javascript from
	 * @param boolean force - force loading of the script, even if it has already been loaded before.
	 */
	LoadScript: function(url,force)
	{
		if ($('html > head > script[src$="'+url+'"]').length>0 && !force)
		{
			MapChat.Debug("warn","Ignoring MapChat.LoadScript('"+url+"'); - script already exists in the page.");
			MapChat.scriptsLoaded.push(url);
			return T;
		}
		if ($.inArray(url,MapChat.scriptsLoaded)==-1||force)
		{
			MapChat.Debug(["info","Fetching script: "+url]);
			$.ajax({"url":url,async:F,"type":"GET",dataType:"script"});
			MapChat.scriptsLoaded.push(url);
			return T;
		}
		else
		{
			MapChat.Debug(["info","Ignoring second request for script: "+url]);
			return T;
		}
	},
	
	/**
	 * Register script URLs, indexed by name, for lazy-loading at a later 
	 * time (replaces the asynchronous approach of MapChat.LoadScript):
	 *
	 * @param string name index name of the script
	 * @param string url location of the script
	 * @param array [optional] an array of index names of required scripts
	 */
	RegisterScript: function(name,url)
	{
		if (!MapChat.jsScripts[name])
		{
			MapChat.jsScripts[name] = {url:url,loaded:F,requires:arguments[2]};
		}
		else if (MapChat.jsScripts[name] && MapChat.jsScripts[name].url==url)
		{
			MapChat.Debug(["warn","The index %s is already allocated with the URL %s - ignoring overwiting with url, but appending any additional requirements.",name,MapChat.jsScripts[name].url]);
			if (arguments[2])
			{
				$.each(arguments[2],function(i,v){
					MapChat.jsScripts[name].requires.push(v);
				});
			}
		}
		else
		{
			MapChat.Debug(["warn","The index %s is already allocated with the URL %s - overwriting with new URL: "+url,
				name,MapChat.jsScripts[name].url]);
			MapChat.jsScripts[name] = {url:url,loaded:F,requires:arguments[2]};
		}
	},
	
	/**
	 * Load script URLs synchronously - forces any code that follows calls
	 * to this function to wait for the loaded javascript.
	 *
	 * @param array arguments - one or more index names of scripts to load
	 */
	Using: function(name)
	{
		$.each(arguments,function(i,a){
			if (MapChat.jsScripts[a] && !MapChat.jsScripts[a].loaded)
			{
				if (MapChat.jsScripts[a].requires)
				{
					$.each(MapChat.jsScripts[a].requires,function(i,v){
						MapChat.Using(v);
					});
				}
				MapChat.Debug(["info","Requesting '%s'",a]);
				if (MapChat.LoadScript(MapChat.jsScripts[a].url))
					MapChat.jsScripts[a].loaded = T;
			}
			else if (!MapChat.jsScripts[a])
			{
				MapChat.Debug(["warn","The index %s has not been registered to a script URL - ignoring request.",a]);
			}
		});
	},
	
	/**
	 * Verify that a namespace exists in the JavaScript environment, and
	 * if it doesn't exist, create it.
	 * 
	 * @param string ns - a string respresenting the namespace.
	 */
	UseNS: function(ns)
	{
		var aNS = ns.split("."),
			curNS = aNS.shift();
		if (!window[curNS]) window[curNS] = {};
		var oNS = window[curNS];
		$.each(aNS,function(){
			curNS = aNS.shift();
			if (!oNS[curNS]) oNS[curNS] = {};
			oNS = oNS[curNS];
		});
		return oNS;
	},
	
	/**
	 * Dynamically load Classes for MapChat.  These will be located
	 * from the js folder - any namespace separators (dots) will be 
	 * treated as slashes in the URL. The 'js' folder and '.js' 
	 * file extension will be added to build the URL to the script.  If
	 * is found at the start of the class name, it will be replaced with
	 * the full name 'jquery' for the jQuery class.
	 * 
	 * @param array arguments the first element must be the class name. The
	 *   class's constructor will be called with any remaining arguments.
	 *   An alternative option for the first argument is to provide an array
	 *   like ["classname",[list_of_registered_scripts]], so that any 
	 *   dependencies can be specified at the time of calling the class;
	 *   NEW: a third array element can be added to the first argument, which
	 *   may be a list of arguments in an array, rather than having them included
	 *   as individual arugments separately.
	 */
	Class: function()
	{
		if (typeof(arguments[0])=="string"||typeof(arguments[0][0])=="string")
		{
			if (typeof(arguments[0])=="string")
			{
				var classname = arguments[0];
			}
			else
			{
				var classname = arguments[0][0];
			
				// If there are requrements specified for the class, make sure they are loaded
				if (arguments[0][1]&&arguments[0][1].length>0)
				{
					$.each(arguments[0][1],function(){MapChat.Using(this);});
				}
			}
			
			// Collect the remaining arguments in an array...
			var args = [];
			if (typeof(arguments[0])=="object" && arguments[0][2] && arguments[0][2].length>0)
			{
				args = arguments[0][2];
			}
			else
			{
				for (var i=1;i<arguments.length;i++)
				{
					args.push(arguments[i]);
				}
			}
			
			// split the class name into namespace parts:
			var nameparts = classname.split(".");
			
			// If the shorthand jQuery namespace was included, replace it with the full name:
			if (nameparts[0]=="$") { nameparts[0] = "jquery"; };
			
			// If the class doesn't exist, then load the script associated with it's name:
			if (!$.isFunction(MapChat.getFunction(classname)))
			{
				MapChat.Debug(["info",classname,"not found - fetching 'js/",nameparts.join("/"),".js'"]);
				var url = "js/"+nameparts.join("/")+".js";
				MapChat.LoadScript(url);
			}
			
			// Ok...using eval is not my preference, by the ideal result of this is to return
			// a proper instance of the class - unless I can figure out a better way to apply
			// a non-predetermined number of arguments to a 'new ClassName()' call, we'll build
			// a string that does it.  This should be fairly safe though, since we're not
			// actually using user-input values in the string, but instead names of locally-
			// defined variables that contain the classname and the supplied arguments:
			if ($.isFunction(MapChat.getFunction(classname)))
			{
				var c = MapChat.getFunction(classname), a = [];
				$.each(args,function(i){a.push("args["+i+"]");});
				return eval("new c("+a.join(",")+")");
			}
			else
			{
				MapChat.Debug(["error","Could not load class named '%s'",classname],T);
			}
		}
		else
		{
			MapChat.Debug(["error","Invalid input for MapChat.Class()"]);
		}
	},
	
	/**
	 * Returns a common instance of a specific Class object - these objects are stored in the
	 * MapChat.singletons array object.
	 *
	 * @param string cls - the name (or array with first element being the name) of the class object.
	 * @param object opts - an array of options that may be passed as the first argument to the class
	 *  constructor function.
	 */
	Singleton: function(cls,opts)
	{
		$.blockUI();
		var classname = ((typeof(cls)=="string")?cls:cls[0]);
		if (!MapChat.singletons[classname]) MapChat.singletons[classname] = new MapChat.Class(cls,opts);
		$.unblockUI();
		return MapChat.singletons[classname];
	},
	
	/** NOTE:  The following three functions should eventually be deprecated/removed, as this is
	 *         redundant of the events methods that are already available from the jQuery library.
	 */
	
	/**
	 * Adds event callbacks to objects - this assumes the object has an eventCallbacks
	 * property, and has been programmed to trigger the requested type of event.
	 *
	 * @param object obj - the object that events are triggered on.
	 * @param string type - name of the event that the callback should be linked to.
	 * @param object func - the function that should be executed.
	 */
	AddEventCallback: function(obj,type,func)
	{
		if (obj && obj.eventCallbacks && obj.eventCallbacks[type] && $.inArray(func,obj.eventCallbacks[type])==-1)
		{
			obj.eventCallbacks[type].push(func);
		}
	},
	
	/**
	 * Removes event callbacks from objects.
	 *
	 * @param object obj - the object that events are triggered on.
	 * @param string type - name of the event that the callback should be linked to.
	 * @param object func - the function that should be removed - if ommitted, all callbacks are removed.
	 */
	RemoveEventCallback: function(obj,type,func)
	{
		
		if (obj && obj.eventCallbacks && obj.eventCallbacks[type])
		{
			if (func && $.inArray(func,obj.eventCallbacks[type])!=-1)
			{
				obj.eventCallbacks[type].splice($.inArray(func,obj.eventCallbacks[type]),1);
			}
			else
			{
				obj.eventCallbacks[type] = [];
			}
		}
	},
	
	/**
	 * Triggers callback functions for events.  From the callback function's perspective,
	 * it will have the object that the event was triggered on as the first argument, and
	 * a second argument with any additional information that was passed to the event.
	 *
	 * @param object obj - the object that events are triggered on.
	 * @param string type - name of the event that the callback should be linked to.
	 * @param object data - data that can be associated with the event.
	 */
	TriggerEvent: function(obj,type,data)
	{
		if (obj && obj.eventCallbacks && obj.eventCallbacks[type])
		{
			$.each(obj.eventCallbacks[type],function(i,f){
				if ($.isFunction(f)) f(obj,data);
			});
		}
	},
	
	/**
	 * For debugging:
	 *
	 * @param mixed msg - debug message. If it is not a string, then the first
	 *  element is expected to be the log type (e.g., 'log', 'info', 'warn', or
	 *  'error'), and the rest is applied to the corresponding console method.  If
	 *  the first element is not a member of the console object, then it is left
	 *  as the first argument sent to console 'log' method.
	 * @param boolean fatal indicates that an alert should be made in the
	 *   absence of console.log() (i.e., in non-Mozilla browsers).
	 *
	 * An alternative is to supply any arguments desired, optionally with the first being
	 *  the name of one of the console[logType] methods.
	 */
	Debug: function(msg,fatal)
	{
		var logType = "log";
		
		if (typeof(msg)!="object" || !(msg && msg.length>0))
		{
			var newMsg = [];
			$.each(arguments,function(i,v){newMsg.push(v);});
			msg = newMsg;
			fatal = F;
			if (typeof(console)=="object" && console[msg[0]]) logType=msg.shift();
		}
		else
		{
			try
			{
				logType = msg.shift();
			}
			catch (e)
			{
				msg = [msg];
			}
		}
		
		try
		{
			if (!console || !console[logType]) { msg.unshift(logType); logType="log"; }
			if (MapChat.debug && typeof(console)=="object" && $.isFunction(console[logType]))
			{
				console[logType].apply(console,msg);
			}
			else if (MapChat.debug!=F && fatal)
			{
				alert(msg);
			}
		}
		catch (e)
		{
			if (MapChat.debug!=F && fatal)
			{
				alert(msg);
			}
		}
	},
	
	/**
	 * Dynamically load CSS files as needed from the server - each script
	 * will only be loaded once - if it has been loaded, the callback
	 * function will be executed immediately.  There is no guarantee that
	 * the CSS file will be loaded before the callback is executed.
	 *
	 * @param string url location to retrieve CSS file from
	 * @param function callback function to execute when the script is requested
	 */
	LoadCSS: function(url,callback)
	{
		var cb = MapChat.getFunction(callback);
		if ($.inArray(url,MapChat.cssLoaded)==-1)
		{
			var cssNode = $('<link type="text/css" rel="stylesheet" href="'+url+'" />');
			
			if ($.browser.msie)
			{
				// This method uses the onload event for a link object - works in IE, but not 
				// in FF.  However, it became apparent that IE does not take all of the CSS 
				// into account if it is not fully loaded before the callback is loaded, so we
				// need to use this approach to ensure the CSS is properly accessible:
				cssNode.load(function(){
					MapChat.cssLoaded.push(url);
					if ($.isFunction(cb))
					{
						cb();
					}
				});
				$(document).find('head:first').find(":first").before(cssNode);	
			}
			else
			{
				// FF doesn't fire any onload events for link elements, so in this approach,
				// we'll force the browser to cache the CSS file, then add the link to the
				// document head - this should make CSS accessible in time for the callback
				// function to be executed, though FF doesn't seem to have a problem with
				// delayed loading of CSS like IE does:
				var cssText = ($.ajax({"url":url,async:F,"type":"GET"}).responseText);
				$(document).find("head:first").find(":first").before(cssNode);
				if ($.isFunction(cb))
				{
					cb();
				}
			}
			MapChat.cssLoaded.push(url);
		}
		else // If the script is already loaded, then just do the callback...
		{
			if ($.isFunction(cb))
			{
				cb();
			}
		}
	},
	
	/**
	 * IE doesn't have a native XMLSerializer object, so this function serves
	 * as a workaround for converting XML objects to plain text:
	 *
	 * @param object xmlNode the xml node/element that will be converted to text
	 */
	xmlText: function (xmlNode)
	{
		if (!$.browser.msie)
		{
			if (!MapChat.xs)
			{
				MapChat.xs = new XMLSerializer()
			}
			return MapChat.xs.serializeToString(xmlNode);
		}
		
		// The Microsoft way:
		else
		{
			return xmlNode.xml;
		}
	},
	
	/**
	 * This will get XHTML from the server that contains the options
	 * available to the current user (e.g., basic user options, available
	 * discussions, admin tools where applicable):
	 *
	 * @param object fadeOjb the object that gets faded out before the
	 *   the options are displayed on the screen (usually the elements
	 *   that contain the login inputs or the loading indicator)
	 */
	showMainOptions: function (fadeObj)
	{
		if (MapChat.session.load_discussion)
		{
			$.blockUI();
			fadeObj.fadeOut(500,function(){
				MapChat.LoadDiscussion(MapChat.session.load_discussion);
				$(this).remove();
			});
		}
		else
		{
			$("#IntroPanel").append('<div id="StartOpts"></div>');
			var startOpts = $(MapChat.RunFunction("Startup_Options",{"args[browser]":$.toJSON($.browser)}));
			var opts = startOpts.find("div")[0];
			MapChat.session.opts = opts;
			if (opts)
			{
				$("#StartOpts").append(MapChat.xmlText(opts));
			}
			else
			{
				$("#StartOpts").append("Error loading startup options...<br /><br />Please contact the website administrator.");
			}
			
			// The options HTML returned by Startup_Options should also be accompanied by
			// a set of bindings, where nodes that correspond to each category of
			// options contain a list of 'opt' nodes with json-encoded paramters that include
			// 'bind' as the name of the event to bind to (e.g., 'click'), and 'run' as
			// text that can be eval'd within a function that will be executed with the event:
			var optBindings = startOpts.find("binds");
			
			optBindings.children().each(function(){
				$(this).children().each(function(i){
					var optBinding = $.parseJSON($(this).text());
					with ($($("#StartOpts").find("li."+$(this).parent()[0].nodeName)[i]))
					{
						bind(optBinding.bind,function(){eval(optBinding.run);});
						addClass("link");
						hover(
							function(){$(this).css("color","#974d0e");},
							function(){$(this).css("color","#0e4d97");}
						);
					}
				});
			});
			fadeObj.fadeOut(500,function(){
				$("#StartOpts").fadeIn(500);
				$(this).remove();
			});
		}	
	},
	
	/**
	 * Logout
	 */
	logout: function()
	{
		var logout = $(MapChat.RunFunction("Session_Logout"));
		if (logout.find("logout").text()=="true")
		{
			$("#MapChat").children("[id!='IntroPanel']").hide();
			$("#StartOpts").remove();
			var intro = $("#MapChat").find("#IntroPanel")
			if (intro.css("display")=="none")
			{
				intro.fadeIn(500);
			}
			MapChat.Using("Login");
			MapChat.showLogin();
		}
	},
	
	/**
	 * Load the MapChat interface for a given discussion.
	 *
	 * @param string discussion the name of the discussion
	 */
	LoadDiscussion: function(discussion)
	{
		$.blockUI();
		if (MapChat.session.https_redir)
		{
			// Let the server know the redirection is coming:
			var setRedir = $(MapChat.RunFunction("Discussion_Redirect",{'args[d]':discussion}));
			
			if (setRedir.find("success").length>0)
			{
				// This lets us use HTTPS for the main login part, but redirect to HTTP for the rest:
				var url = document.location.href;
				document.location.href = "http"+url.substring(url.indexOf("://"));
			}
			else
			{
				alert("Error entering the selected discussion map.");
				$.unblockUI();
			}
		}
		else
		{
			var startTime = parseInt((new Date()).getTime()/1000);
			MapChat.RunFunction("Discussion_Start",{'args[d]':discussion},{async:T,complete:function(xhr){
				if (xml = MapChat.CheckErrors(xhr).responseXML)
				{
					var xmlDiscussion = $(xml).find("discussion:first");
					MapChat.discussion = {
						name: xmlDiscussion.find("name:first").text(),
						ro: xmlDiscussion.find("ro:first").text()=="true",
						config: xmlDiscussion.find("config:first"),
						users:$("<div />")
					};
					
					// Get the chat node from the config xml, and load the MapChat.Chat class...
					var cConf = $("chat:first", MapChat.discussion.config);
					MapChat.chat = MapChat.Class([cConf.attr("lib")||"MapChat.ChatLib",['jquery.tooltip']],cConf);
					MapChat.chat.AddContext($("#MapChat"));
					
					// This should load the required scripts for the initial map interface (registered below);
					MapChat.Using("application");
					MapChat.StartApplication();
					
					// This is the difference in seconds between the local browser time and the server request timestamp.  Add this
					// number to local Date() timestamps in order to get a timestamp that closely matches the server's time.  This
					// will be useful for recording the time of locally queued events, which will be submitted to/recorded by the
					// server at a later time (minimizing HTTP requests that are not required for immediate functionality).
					MapChat.timeDelta = ((parseInt(xmlDiscussion.find("time:first").text()) - startTime))*1000;
				}
			}});
		}
	},
	
	/**
	 * Get functions by their text name, separated by dots for anything 
	 * that is in the jQuery namespace (e.g., MapChat.getFunction('MapChat.RunFunction').
	 *
	 * @param string|function fName the name of the function to return (or, if
	 *   fName is already a function, return that instead).
	 */
	getFunction: function(fName)
	{
		if (fName && typeof(fName)!="string" && $.isFunction(fName))
		{
			// if the input is already a function, hand it back:
			return fName;
		}
		else if (fName && typeof(fName)=="string" && fName.length>0)
		{
			// Look for functions in a namespace:
			var aFName = fName.split(".");
			var func = window[aFName.shift()];
			
			// Is the first part of the namespace valid?
			if (func)
			{
				// Get the rest of it:
				$.each(aFName,function(){ return !!(func=func[this]); });
			}
			
			// Is it a function?
			if (func && $.isFunction(func))
			{
				return func;
			}
			else
			{
				try
				{
					// Ok, maybe it's an inline function:
					eval("var sf = "+fName+";");
					if (sf && $.isFunction(sf))
					{
						return sf;
					}
				}
				catch (e)
				{
					MapChat.Debug(["warn","Function not found:",fName])
				}
			}
		}
	},
	
	/**
	 * Allow functions to be called by their text name, separated by
	 * dots for anything that is in the jQuery namespace (e.g., 
	 * MapChat.callFunction('MapChat.RunFunction',args).  The function name
	 * must be the first argument, and may be optionally followed by
	 * any number of arguments, as appropriate for the called function:
	 */
	callFunction: function()
	{
		var func = MapChat.getFunction(arguments[0]);
		if ($.isFunction(func))
		{
			var args = [];
			for (var i=1;i<arguments.length;i++)
			{
				args.push(arguments[i]);
			}
			return func.apply(this,args);
		}
	},
	
	
	/*
	 *  MapChat.applyPNGFilter(o) - adapted from ka-Map's startUp.js:
	 *
	 *  Applies the PNG Filter Hack for IE browsers when showing 24bit PNG's
	 *
	 *  var o = object (this png element in the page)
	 *
	 * The filter is applied using a nifty feature of IE that allows javascript to
	 * be executed as part of a CSS style rule - this ensures that the hack only
	 * gets applied on IE browsers :)
	 */
	applyPNGFilter: function(o)
	{
		if (MapChat.ie6)
		{
			var t="images/a_pixel.gif";
			// If a second argument is provided, it should be the name of a
			// css property that has a value of url([url]) in it.  This property
			// will be dropped, and replaced by the filter - this should work for
			// any non-repeating background images.  Actually...it doesn't seem to
			// do anything at all unless the width/height is specified in the
			// elemnent's style properties.
			if (arguments[1])
			{
				s = $(o).css(arguments[1]);
				if (s.substring(0,4)=="url(");
				{
					$(o).css(arguments[1],"none");
					o.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+s.substring(5,s.length-2)+"',sizingMethod='scale')";
				}
			}
			else
			{
				if( o.src != t )
				{
						var s=o.src;
						o.src = t;
						o.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+s+"',sizingMethod='scale')";
				}
			}
		}
	},
	
	/*
	 * Returns unique IDs based on a sequential integer.
	 *
	 * @param string p - if specified, the id is prefixed with this text and an underscore, otherwise
	 * the prefixe of 'uid_' is used.
	 * @param integer n - if specified, an array of this many unique IDs is returned.
	 */
	UID: function(p,n)
	{
		if (n)
		{
			var r = [];
			while (r.length<n)
			{
				r.push(this.UID(p));
			}
		}
		else
		{
			var r = (p?""+p+"_":"uid_")+MapChat.nextID;
			MapChat.nextID++;
		}
		return r;
	},
	
	/*
	 * Returns an array of unique values from the input array.
	 *  Any non-numerical keys in an index/ojbect array will be lost.
	 *
	 * @param array a - an array that might have duplicate values
	 */
	UniqueArray: function(a)
	{
		var o = [];
		$.each(a,function(i,v){
			if ($.inArray(v,o)==-1) o.push(v);
		});
		return o;
	},
	
	/**
	 * Returns a string normalized to lowercase with all non-alphanumeric characters converted to
	 * underscores (e.g., so it is suitable for use as fieldnames in a database).
	 *
	 * @param string s - the string to normalize
	 *
	 * @return string
	 */
	NormalizeString: function(s)
	{
		return s.replace(/[^A-Za-z0-9]/g,'_').toLowerCase();
	},
	
	/**
	 * Sets the disabled state and adds/removes a "disabled" class to an element (usually a button).
	 *
	 * @param object b - the jQuery element for the button (or other element) to be disabled/enabled
	 * @param boolean d - the disabled state (false = disabled, true = enabled)
	 * @param string class - if specified, this class will be used instead of 'disabled'
	 *
	 * @return object - the input object (b)
	 */
	DisableButton: function(b,d,c)
	{
		if (!d) return b.attr(D,T).addClass("ui-state-disabled");
		else return b.removeAttr(D).removeClass("ui-state-disabled");
	},
	
	/**
	 * Alias for DisableButton, with reversed boolean logic.
	 *
	 * @param object b - the jQuery element for the button (or other element) to be disabled/enabled
	 * @param boolean d - the disabled state (true = disabled, false = enabled)
	 * @param string class - if specified, this class will be used instead of 'disabled'
	 *
	 * @return object - the input object (b)
	 */
	EnableButton: function(b,d,c)
	{
		return MapChat.DisableButton(b,!d,c);
	},
	
	/**
	 * Returns an object with the requested user's data.
	 *
	 * @param string un - the username to get info for.
	 * @param boolean f - if true, this forces the information for the user to be re-downloaded
	 */
	UserInfo: function(un,f)
	{
		var t = this, u = $('p[id="'+un+'"]',t.discussion.users);
		
		if (u.length==0 || f)
			return $('<p id="'+un+'" />').data("u",$.parseJSON($("info:first",MapChat.RunFunction("User_Info",{args:{username:un}})).text())).appendTo(t.discussion.users);
		else
			return u;
 	}
});


/**
 * Register scripts that can be called on-demand via MapChat.Using():
 */
MapChat.RegisterScript("Login","js/Login.js");
MapChat.RegisterScript("User","js/admin/user.js");
MapChat.RegisterScript("Admin","js/admin/admin.js");
MapChat.RegisterScript("kamap","kamap");
MapChat.RegisterScript("application","js/MapChat/application.js",["kamap"]);
MapChat.RegisterScript("ui.core","js/jquery/ui/ui.core.js");
MapChat.RegisterScript("ui.position","js/jquery/ui/ui.position.js");
MapChat.RegisterScript("ui.widget","js/jquery/ui/ui.widget.js",["ui.core"]);
MapChat.RegisterScript("ui.mouse","js/jquery/ui/ui.mouse.js",["ui.widget"]);
MapChat.RegisterScript("ui.button","js/jquery/ui/ui.button.js",["ui.widget"]);
MapChat.RegisterScript("ui.draggable","js/jquery/ui/ui.draggable.js",["ui.widget","ui.mouse"]);
MapChat.RegisterScript("ui.resizable","js/jquery/ui/ui.resizable.js",["ui.widget","ui.mouse"]);
MapChat.RegisterScript("ui.dialog","js/jquery/ui/ui.dialog.js",["ui.draggable","ui.resizable","ui.position"]);
MapChat.RegisterScript("ui.tabs","js/jquery/ui/ui.tabs.js",["ui.widget"]);
MapChat.RegisterScript("jquery.tooltip","js/jquery/jquery.tooltip.js");
MapChat.RegisterScript("jquery.autocomplete","js/jquery/jquery.autocomplete.js");
MapChat.RegisterScript("jquery.rte","js/jquery/jquery.rte.js");
MapChat.RegisterScript("jquery.colorPicker","js/jquery/jquery.colorPicker.js");
MapChat.RegisterScript("jquery.notice","js/jquery/jquery.notice.js");
MapChat.RegisterScript("jquery.scrollTo","js/jquery/jquery.scrollTo.js");
MapChat.RegisterScript("jquery.bt","js/jquery/jquery.bt.js");
if ($.browser.msie) MapChat.RegisterScript("excanvas","js/excanvas.js");

/**
 * Get things started when the window is done loading:
 */
$(window).load(function(){
	
	// Only do the following if we're on the main page with the SplashLoading div.
	if ($("#SplashLoading").length>0)
	{
		// Make sure the loading DIV is visible:
		$("#SplashLoading").show();
		
		// Check to see if the browser is IE6 or older:
		MapChat.ie6 = ($.browser.msie&&$.browser.version<7);
		
		// Any IE older than 6 is busted:
		if ($.browser.msie&&$.browser.version<6)
		{
			$("#SplashLoading").text("Your current browser version is unsupported.  Recommended browsers include FireFox 3.x and higher, or Microsoft Internet Explorer 7 and higher.");
			return;
		}
	}
	
	// Load the UI CSS - FF2 doesn't render properly if it isn't loaded before its resources are used:
	// MapChat.LoadCSS("css/ui.css");
	
	// If we got a cookie, then we won't be including
	// the 'sid' parameter in any get/post requests:
	if ($.cookie('sid'))
	{
		MapChat.useCookies = T;
	}
	else if (!(MapChat.session && MapChat.session.sid) && document.location.search)
	{
		// If an SID has been supplied in the URL, get it, and set it
		// in the session object:
		var getvars = document.location.search.substr(1).split("=");
		for (var i=0;i<getvars.length;i=i+2)
		{
			if (getvars[i]=="sid") MapChat.session.sid=getvars[i+1];
		}
	}
	
	// Setup the global AJAX parameters:
	$.ajaxSetup({
		"type": "POST",
		"url": "API/",
		"async": F,
		"cache": T,
		"error": function(xhr,status,error){alert(status+": "+error); $.unblockUI();}
	});
	
	// Setup the UI blocker (call $.blockUI() to make the mouse cursor switch 
	// to hourglass mode, and block access to page elements):
	$.blockUI.defaults = $.extend($.blockUI.defaults, {
		overlayCSS: { backgroundColor:'#FFF', opacity: .0 },
		message: F,
		fadeOut: 0,
		applyPlatformOpacityRules: F,
		baseZ: 99999
	});
	
	// Retrieve the initial session info:
	//MapChat.session = $.parseJSON($(MapChat.RunFunction("Session_Info")).find("session").text());
	
	// Only do the following if we're on the main page with the SplashLoading div.
	if ($("#SplashLoading").length>0)
	{
		if (!MapChat.session.auth_status)
		{
			// The user is not authorized - show a login prompt
			MapChat.Using("Login");
			$("#SplashLoading").fadeOut(500,function(){
					MapChat.showLogin();
			});
		}
		else
		{
			// The user is authorized (maybe he/she has reloaded the webpage) - start
			// loading other stuff for the application:
			MapChat.showMainOptions($("#SplashLoading"));
		}
	}
});
