/**
 * An object that presents a Chat interface.
 *
 * @class MapChat.Chat.Dialog
 *
 * @param map - a kaMap class instance. If not defined, it will default to MapChat.map.
 * @param chat - a ChatLib class instance, which includes the methods required for
 *               keeping chat messages synchronized with the server (e.g., Get(),
 *               Send(), etc.).  If not defined, it will default to MapChat.chat.
 * @param options - an associative array of options that configure the behaviour of the dialog.
 *
 * Depends:
 *	kamap/htdocs/kaMap.js
 *	MapChat.js
 *	MapChat/application.js
 *	MapChat/MapLayers.js
 *	jquery/ui/ui.dialog.js
 *  MapChat/SelectLib.js
 *  MapChat/DrawLib.js
 */

// Get an alias for the the MapChat.Chat namespace (to make the code shorter).
var mcc = MapChat.UseNS("MapChat.Chat"),

	// Alias the disable/enable button functions and the QueueEvent function:
	dbtn = MapChat.DisableButton, ebtn = MapChat.EnableButton, mcqe = MapChat.QueueEvent;
	
mcc.Dialog = function(map,chat,options)
{
	var t = this,
		options = (t.options = options||{}),
		name = (t.name = "MapChat.Chat.Dialog"),
		zIndex = (t.zIndex = options.zIndex||300),
		map = (t.map = map||MapChat.map),
		mapLayers = (t.mapLayers = MapChat.MapSingleton("MapChat.MapLayers",map)),
		chat = (t.chat = chat||MapChat.chat),
		maxUpdatedID = (t.maxUpdatedID = N),
		minUpdatedID = (t.minUpdatedID = N),
		stopLoadAll = (t.stopLoadAll = F),
		UserList = (t.UserList = N),
		
		SelectLib = (t.SelectLib = MapChat.MapSingleton("MapChat.SelectLib",map)),
		DrawLib = (t.DrawLib = MapChat.MapSingleton("MapChat.DrawLib",map)),
		
		filterFlags = (t.filterFlags = []), // Any flags added to this array will be used to filter messages displayed in the chat tab of the dialog
		
		content = (t.content = 
			$('<div class="autoscroll mccd"></div>')
			.dialog({title:"Chat Dialog",width:350,height:425,modal:false,autoOpen:false,closeOnEscape:false,
				resizeStop:function()
				{
					// Make the continer is fit to the dialog's height (so we get scrolling):
					t.content.css({height:$(".ui-resizable-s", t.dialog).offset().top-t.content.offset().top});
					t.divTabs.css({height:t.content.height()-7});
					t.panelHeight = t.content.height() - 
						((t.divTabs.children('div[id^="mccd_"]:visible').offset().top) - t.content.offset().top)-10;
					t.divTabs.children(".ui-tabs-panel").css({height:t.panelHeight});
					t.curTab.trigger("tabResize");
				},
				open:function()
				{
					mcqe(9,"ChatDialog",["Opened"]);
					t.content.css({height:$(".ui-resizable-s", t.dialog).offset().top-t.content.offset().top});
					t.divTabs.css({height:t.content.height()-7});
					t.panelHeight = t.content.height() - 
						((t.divTabs.children('div[id^="mccd_"]:visible').offset().top) - t.content.offset().top)-10;
					t.divTabs.children(".ui-tabs-panel").css({height:t.panelHeight});
					t.curTab.trigger("tabResize");
				},
				close:function()
				{
					mcqe(9,"ChatDialog",["Closed"]);
				}
			})
		),
		
		/** TODO: calculate the number of tabs required based on any config issues **/
		panelIDs = (t.panelIDs = MapChat.UID("mccd",2)),
		panelHeight = (t.panelHeight = N),
		
		/** TODO: include other tabs here somehow **/
		divTabs = (t.divTabs = $('<div><ul><li><a href="#'+t.panelIDs[0]+'"><span>Chat</span></a></li><li><a href="#'+t.panelIDs[1]+'"><span>Users</span></a></li></ul><div id="'+t.panelIDs[0]+'" /><div id="'+t.panelIDs[1]+'" /></div>')).appendTo(t.content)
			.tabs({show:function(){t.ShowTab();}})
			.bind("tabsselect",function(){return t.SelectTab();}),
		
		ulTabs = (t.ulTabs = $("ul:first",t.divTabs)),
		curTab = (t.curTab = $("#"+t.panelIDs[0],divTabs)),
		
		// Setup the Chat tab:
		tabChat = (t.tabChat = $("#"+t.panelIDs[0],divTabs)).append(
			'<div class="chat_toppanel">'+
				'<p class="chat_view_mode">'+
					(t.chat.conf.children("locations:first").children().length>0?
						'<span class="chat_location_filter">Location: '+
							'<select class="chat_select_filter" name="chat_location_filter">'+
								'<option>Entire Map</option>'+
							'</select> <button class="ui-state-default ui-corner-all" name="chat_location_zoom">Zoom</button> <button class="ui-state-default ui-corner-all ui-state-disabled" name="chat_location_describe" disabled="true">Describe</button>'+
						'</span><br />'
					:"")+
					(t.chat.conf.children("views:first").children().length>0?
						'<span class="chat_view_filter">View data: '+
							'<select class="chat_select_filter" name="chat_view_filter"><option>All/Any Views</option></select>'+
							// ' <button class="disabled" name="chat_view_apply">Apply</button>'+ not using this one for now...
						'</span><br />'
					:"")+
					'Display mode: <input type="radio" name="thread_view" /> Threaded &nbsp;&nbsp;&nbsp; '+
					'<input type="radio" name="list_view" checked="true" /> Sequential'+
				'</p>'+
			'</div>'+
			'<div class="chat_history">'+
				'<div class="chat_threaded chat_messages" style="display:none;" /><div class="chat_list chat_messages" />'+
			'</div>'+
			'<div class="buttonpanel">'+
				'<button class="ui-state-default ui-corner-all" name="new_topic">New Topic</button>'+
			'</div>'
		).bind("tabResize",function(){
			var ch = $('div.chat_history',t.tabChat);
			if (!$.browser.msie || $.browser.version.substring(0,1)>6)
			{
				ch.css('height',(ch.parent().height()-ch.prev().height()-ch.next().height()+4));
				ch.children().css('height',ch.height());
			}
			else
			{
				ch.css('height',(t.content.height()-ch.parents('.ui-tabs:first').children('.ui-tabs-nav:first').height()-ch.prev().height()-ch.next().height()-10));
				ch.children().css('height',ch.height());
			}
		}).trigger("tabResize"),
		
		// Setup the Users tab:
		tabUsers = (t.tabUsers = $("#"+t.panelIDs[1],divTabs))
			.append('<div class="user_list autoscroll">Loading...</div><div class="apply_button"><p style="text-align:right;">'+
				'<button class="ui-state-default ui-corner-all ui-state-disabled" disabled="true" name="apply">Apply</button></p></div>')
			.bind("tabResize",function(){
				var ul = $('div.user_list',t.tabUsers), ab = $('.apply_button',t.tabUsers);
				if (!$.browser.msie || $.browser.version.substring(0,1)>6)
				{
					if (ab.length>0) ul.css('height',(ul.parent().height()-ab.height()));
					else ul.css('height',ul.parent().height());
				}
				else
				{
					if (ab.length>0) ul.css('height',(t.content.height()-ul.parents('.ui-tabs:first').children('.ui-tabs-nav:first').height()-ab.height()-10));
					else ul.css('height',(t.content.height()-ul.parents('.ui-tabs:first').children('.ui-tabs-nav:first').height()-10));
				}
			})
			.bind("show",function(){
				if (!t.UserList)
				{
					t.UserList = MapChat.Class(["MapChat.Chat.UserList",['jquery.colorPicker']],t.map,t.chat);
					t.tabUsers.children('.user_list').remove();
					t.UserList.list.insertBefore(t.tabUsers.children('.apply_button'));
					$('button[name="apply"]',t.tabUsers).click(function(){t.UserList.ApplyUserStates();});
				}
				t.tabUsers.trigger("tabResize");
			}),
			
		displayCanvas = (t.displayCanvas = map.defaultCanvas||MapChat.Class("MapChat.Canvas",{map:t.map,zIndex:600})),
		
		// Styles for selection/draw shapes:
		selectOpts = (t.selectOpts = options.selectOpts||{stroke:{stroke:"#00F",'stroke-width':2,'stroke-opacity':0.60},r:5}),
		selectFill = (t.selectFill = $.extend(options.selectFill||{fill:{fill:"#00F",'fill-opacity':0.35}},t.selectOpts)),
		extentOpts = (t.extentOpts = options.extentOpts||{stroke:{stroke:"#FCA",'stroke-width':2,'stroke-opacity':0.60},r:5}),
		extentFill = (t.extentFill = $.extend(options.extentFill||{fill:{fill:"#FCA",'fill-opacity':0.35}},t.extentOpts)),
		drawOpts = (t.drawOpts = options.drawOpts||{stroke:{stroke:"#77F",'stroke-width':2,'stroke-opacity':0.60},r:5}),
		drawFill = (t.drawFill = $.extend(options.drawFill||{fill:{fill:"#77F",'fill-opacity':0.35}},t.drawOpts)),
		
		dialog = (t.dialog = content.parents(".ui-dialog:first"));
	
	t.displayCanvas.activate();
	
	// Add dialog elements to the chat context used for searching/triggering events on messages and related elemens:
	t.chat.AddContext(t.tabChat,t,'message-listener');
	t.chat.AddContext(t.tabUsers,t);
	
	// Add the dialog to the 'map' context, so it will respond to searching/triggering of events by the Select/Draw Libs:
	MapChat.AddContext(t.map,t.dialog);
	
	// Bindings for the chat tab contents:
	
	if (t.chat.conf.children("locations:first").children().length>0)
	{
		var lf = $('select[name="chat_location_filter"]',tabChat);
		lf.change(function(){
			var o = lf.children(':selected'), curFlag = N;
			if (lf.attr("selectedIndex")>0) curFlag = o.val();
			else curFlag = N;
			
			mcqe(9,"ChatDialog",["LocationFilterChanged",curFlag]);
			
			if (lf.data("selected")!=curFlag)
			{
				if (lf.data('selected')!=N)
					t.DropFilterFlag('loc-'+lf.data('selected'));
				
				if (curFlag!=N)
					t.AddFilterFlag('loc-'+curFlag);
				
				t.ApplyFilter();
			}
			lf.data("selected",curFlag);
			if (o) (o.data("desc")==N?dbtn:ebtn)($('button[name="chat_location_describe"]',tabChat));
		}).click(function(){$(this).change();}).keyup(function(e){switch (e.keyCode) { case 37,38,39,40: $(this).change(); } } );
		$('button[name="chat_location_zoom"]',tabChat).click(function(){
			mcqe(9,"ChatDialog",["LocationZoomClicked"]);
			var ex = $(this).prev().children(":selected").data("extent")||t.map.getCurrentMap().defaultExtents;
			t.map.zoomToExtents(ex[0],ex[1],ex[2],ex[3]);
		});
		$('button[name="chat_location_describe"]',tabChat).click(function(){
			var clb = $(this);
			mcqe(9,"ChatDialog",["LocationDescribeClicked"]);
			if (clb.data(lf.data('selected')))
			{
				clb.data(lf.data('selected')).dialog("open");
			}
			else
			{
				var display = $(""+lf.children(':selected').data("desc"));
				display.dialog({
					title:"Location: "+lf.children(':selected').text(),
					width:420,height:350,modal:false,closeOnEscape:false,autoOpen:true,
					open:function(){
						var c = $(this).children('div:first'), dlg = c.parents(".ui-dialog:first");
						c.css({height:$(".ui-resizable-s", dlg).offset().top-c.offset().top});;
					},
					resizeStop:function(){
						var c = $(this).children('div:first'), dlg = c.parents(".ui-dialog:first");
						c.css({height:$(".ui-resizable-s", dlg).offset().top-c.offset().top});
					}
				}).children('div:first').addClass("autoscroll");
				clb.data(lf.data('selected'),display);
			}
		});
	}
		
	if (t.chat.conf.children("views:first").children().length>0)
	{
		var vf = $('select[name="chat_view_filter"]',tabChat);
		vf.change(function(){
			var vf = $(this), o = vf.children(':selected'), curFlag = N;
			if (vf.attr("selectedIndex")>0) curFlag = o.val();
			else curFlag = N;
			
			mcqe(9,"ChatDialog",["ViewFilterChanged",curFlag]);
			
			if (vf.data("selected")!=curFlag)
			{
				if (vf.data('selected')!=N)
					t.chat.DropGlobalFlag('view-'+vf.data('selected'));
					//t.DropFilterFlag('view-'+vf.data('selected'));  // two filters is a bit too much, so this is only going to change views, not limit the messages displayed
				
				if (curFlag!=N)
					t.chat.AddGlobalFlag('view-'+curFlag);
					//t.AddFilterFlag('view-'+curFlag);
				
				t.ApplyView(vf);
				// t.ApplyFilter();
			}
			vf.data("selected",curFlag);
			//$('button[name="chat_view_apply"]',t.tabChat).attr("disabled",(curFlag==N))[(curFlag==N?"addClass":"removeClass")]("disabled");
		}).click(function(){$(this).change();}).keyup(function(e){switch (e.keyCode) { case 37,38,39,40: $(this).change(); } } );
	}
	$('input[name="thread_view"]',tabChat).click(function(){t.SetThreadedView(T);});
	$('input[name="list_view"]',tabChat).click(function(){t.SetThreadedView(F);});
	
	$('button[name="new_topic"]',tabChat).click(function(){
		// Display an input area above the new topic button:
		nt = $(this);
		if (!nt.data("NewMessage"))
		{
			nt.data("NewMessage",
			t.chat.NewMessage({
				obj:nt,
				eflags:t.filterFlags,
				append:'sibling',
				map:t.map,
				SelectLib:t.SelectLib,
				DrawLib:t.DrawLib,
				above:T,
				close:function(e,msg){
					nt.removeData("NewMessage");
					t.tabChat.trigger("tabResize");
					if (msg) t.ScrollTo(T,T);
				}
			}));
			// Some bug is causing the history size to become too small in FF...doing this twice seems to work, though
			// it looks funky as it happens:
			t.tabChat.trigger("tabResize").trigger("tabResize");
		}
	});
	
	// Setup the select options for the location and/or views
	t.chat.conf.children("locations:first").children().each(function(){
		var l = $(this);
		$('<option />').val(l.attr("flag")).append(l.attr("name"))
			.appendTo($("select.chat_select_filter[name='chat_location_filter']",tabChat))
			.data("extent",$.parseJSON(l.attr("extent")))
			.data("desc",(l.children("description:first").length>0?
				MapChat.xmlText(l.children("description:first").children("div:first").get(0))
			:N));
	});
	t.chat.conf.children("views:first").children().each(function(){
		var v = $(this), vf = $("select.chat_select_filter[name='chat_view_filter']",tabChat);
		$('<option />').val(v.attr("flag")).append(v.attr("name")).appendTo(vf).data("view",v);
	});
	
	// Load the messages currently in the chat message data:
	//$.each(t.chat.msgData.children(),function(){t.LoadMessage($(this),F);});
	$(".chat_threaded",t.tabChat).append(t.chat.DisplayAllMessages(T,T));
	$(".chat_list",t.tabChat).append(t.chat.DisplayAllMessages());
	t.chat.BindMessageElems('chat_new_elem',T,{map:t.map,SelectLib:t.SelectLib,DrawLib:t.DrawLib});
	
	// Set both the threaded and list views to scroll to the bottom:
	t.ScrollTo(T,T);
	
	// If there are more messages than currently avialable in the chat message data, then add
	// options to load the rest:
	if (t.chat.msgData.children().length<t.chat.msgData.data("count"))
	{
		$("div.chat_toppanel",t.tabChat).append(
			$('<p class="chat_view_more"/>')
				.append('Showing <span class="chat_more_count">'+
					t.chat.msgData.children().length+
					'</span> of <span class="chat_total_count">'+
					t.chat.msgData.data("count")+'</span>.  Get ')
				.append(
					$('<span class="chat_get_more" style="cursor:pointer">more</span>').click(function(){
						mcqe(9,"ChatDialog",["ChatGetMoreClicked"]);
						var loading = $('<span class="loading_more" style="cursor:pointer">&nbsp;&nbsp;Loading...</span>').insertAfter($(this).parent().children('.chat_get_all')),
							vm = $("div.chat_toppanel .chat_view_more",t.tabChat);
						t.chat.Get({sinceID:t.chat.depthMin[t.chat.historyDepth],reverse:T,depth:t.chat.historyDepth},{ignore:T,delayScroll:T,updateDepth:T,after:function(xhr){
							if (parseInt($("chat_messages:first",xhr.responseXML).attr("length"))<25) t.chat.historyDepth++;
							if (parseInt($("chat_messages:first",xhr.responseXML).attr("length"))==0)
							{
								t.chat.Get({sinceID:t.chat.depthMin[t.chat.historyDepth]||-2,reverse:T,depth:t.chat.historyDepth},{ignore:T,delayScroll:T,updateDepth:T,after:function(xhr){
									if (parseInt($("chat_messages:first",xhr.responseXML).attr("length"))<25) t.chat.historyDepth++;
								}});
							}
							if (t.chat.msgData.children().length>=t.chat.msgData.data("count")) vm.remove();
							t.ScrollTo(t.chat.depthMin[t.chat.historyDepth]||t.chat.depthMin[t.chat.historyDepth-1],t.chat.lastMinID);
							loading.remove();
						}});
					})
				).append('|').append(
					$('<span class="chat_get_all" style="cursor:pointer">all</span>').click(function(){
						mcqe(9,"ChatDialog",["ChatGetAllClicked"]);
						var vm = $("div.chat_toppanel .chat_view_more",t.tabChat),
						loadAll = function(){
							t.chat.Get({sinceID:t.chat.depthMin[t.chat.historyDepth],reverse:T,depth:t.chat.historyDepth},{ignore:T,delayScroll:T,updateDepth:T,after:function(xhr){
								if (parseInt($("chat_messages:first",xhr.responseXML).attr("length"))<25) t.chat.historyDepth++;
								if (t.chat.msgData.children().length<t.chat.msgData.data("count"))
								{
									if (!t.stopLoadAll) setTimeout(loadAll,100);
									else
									{
										t.stopLoadAll = F;
										vm.children(".cancel_loadall").remove();
									}
								}
								else
								{
									vm.remove();
									t.tabChat.trigger("tabResize");
									t.ScrollTo(t.chat.lastMinID,t.chat.lastMinID);
								}
							}});
						},
						loading = $('<span class="cancel_loadall" style="cursor:pointer">&nbsp;&nbsp;Loading... (cancel)</span>').insertAfter($(this)).click(function(){
							mcqe(9,"ChatDialog",["StopLoadAllClicked"]);
							t.stopLoadAll = T;
						});
						loadAll();
					})
				)
		);
		
	}
	
	// Bind events to the chat library...then cool stuff can happen.
	t.tabChat.bind("MessagesReceived",function(e,chat_ids,scroll){t.UpdateChat(chat_ids,scroll);});
	
	MapChat.KeyProtect($(".chat_select_filter",tabChat),{focus:true});
	window.test = t;
}

var mccd = mcc.Dialog;

/**
 * Opens the Chat Messages interface
 */
mccd.prototype.Open = function()
{
	
	$.blockUI();
	
	var t = this;
	
	t.content.dialog("open").dialog("moveToTop");
	
	// Set both the threaded and list views to scroll to the bottom:
	t.ScrollTo(T,T);
	
	$.unblockUI();
	
	return t;
}

/**
 * Loads a message currently stored in the chat module.
 */
mccd.prototype.LoadMessage = function(m,scroll)
{
	
	var t = this, md = m.data("md"), tID = N, lID = N,
		ct = $(".chat_threaded",t.tabChat),
		cl = $(".chat_list",t.tabChat);
	
	// Add pointers to the map, SelectLib and DrawLib:
	$.extend(md,{map:t.map,SelectLib:t.SelectLib,DrawLib:t.DrawLib});
	
	// This first case is likely from a request to load historical messages - t.ScrollTo will be called with this ID:
	if (md.id<t.minUpdatedID)
	{
		tID = lID = md.id;
	}
	// This is likely a new incoming message.  Call t.ScrollTo with 'true' arguments to indicate the
	// scroll should stick to the bottom if applicable (replies to earlier threads shouldn't move the scroll)
	else if (md.id>t.maxUpdatedID)
	{
		// Calculate the distance of the scroll from the bottom for both the threaded and list views:
		tID = ((ct.scrollTop()+ct.height())-(ct.children(":last").offset().top-ct.children(":first").offset().top+ct.children(":last").outerHeight()))>=-5;
		lID = ((cl.scrollTop()+cl.height())-(cl.children(":last").offset().top-cl.children(":first").offset().top+cl.children(":last").outerHeight()))>=-5;
	}
	
	// define a new element that will display the message and related controls:
	var nm = t.chat.DisplayMessage(m), nTm = t.chat.DisplayMessage(m,{showReplies:T});
	nm.data("eflags",t.filterFlags);
	nTm.data("eflags",t.filterFlags);
	
	// Bind ScrollTo events, so they can be called into focus from the ChatLib:
	nm.bind("ScrollTo",{t:t},mccd.msgScrollTo);
	nTm.bind("ScrollTo",{t:t},mccd.msgScrollTo);
	
	// Hide messages if they don't match any currently selected filter flags:
	if (t.filterFlags.length>0)
	{
		nm.filter(':not(.'+t.filterFlags.join('.')+')').addClass('chat_message_hidden');
		nTm.filter(':not(.'+t.filterFlags.join('.')+')').addClass('chat_message_hidden');
	}
	
	// Append one to the sequential view:
	MapChat.ChatLib.AppendList(nm,$("div.chat_list",t.tabChat));
	
	// Append the other to the threaded view:
	MapChat.ChatLib.AppendThread(nTm,$("div.chat_threaded",t.tabChat));
	
	if (scroll) t.ScrollTo(tID,lID);
	
	
	t.minUpdatedID = (t.minUpdatedID!=N?Math.min(t.minUpdatedID,md.id):md.id);
	t.maxUpdatedID = (t.maxUpdatedID!=N?Math.max(t.maxUpdatedID,md.id):md.id);
	
}

/**
 * This function should trigger whenever new messages are received.
 */
mccd.prototype.UpdateChat = function(chat_ids,scroll)
{
	var t = this;
	$.each(chat_ids,function(i,id){
		t.LoadMessage($("p.chat_id-"+id,t.chat.msgData),scroll==T);
	});
	
	t.chat.BindMessageElems('chat_new_elem',T,{map:t.map,SelectLib:t.SelectLib,DrawLib:t.DrawLib});
	$("span.chat_more_count",$("div.chat_toppanel",t.tabChat)).text(t.chat.msgData.children().length);
	$("span.chat_total_count",$("div.chat_toppanel",t.tabChat)).text(t.chat.msgData.data("count"));
}


/**
 * Handles what happens when tabs are switched
 */
mccd.prototype.ShowTab = function()
{
	var t = this,
		curTabID = (t.divTabs?t.divTabs.tabs('option', 'selected'):0),
		panelID = t.panelIDs[curTabID],
		panel = $("#"+panelID,t.divTabs);
	
	// the ui.tabs script currently has a feature that clears any previously set style properties, so we'll have to
	// reset these each time a tab is displayed...just to be sure.
	t.divTabs.children(".ui-tabs-panel").css({height:t.panelHeight});
	
	t.lastPanelID = panelID;
	t.lastTabID = curTabID;
	
	t.curTab = panel.trigger("tabResize").trigger("show");
	
	mcqe(9,"ChatDialog",["ShowTab",(t.curTab?t.curTab.data("name"):'')]);
	
}

/**
 * Handles what happens when tabs are switched.
 */
mccd.prototype.SelectTab = function()
{
	var t = this;
	if (t.lastPanelID)
	{
		if ($("#"+t.lastPanelID,t.divTabs).data("changed"))
		{
			if (t.lastTabID!=N && !confirm("Discard unsaved changes?")) return F;
// 			else t.DiscardChanges($("#"+t.lastPanelID,t.divTabs));
		}
	}
}

/**
 * Switches the view in the Chat tab between threaded and sequential views.
 *
 * @param boolean threaded - if true, set view as threaded; if false, view as list
 */
mccd.prototype.SetThreadedView = function(threaded)
{
	mcqe(9,"ChatDialog",["SetThreadedView",(threaded?"true":"false")]);
	
	var t = this,
		rThr = $('input[type="radio"][name="thread_view"]',t.tabChat),
		rList = $('input[type="radio"][name="list_view"]',t.tabChat),
		ct = $("div.chat_threaded",t.chatTab),
		cl = $("div.chat_list",t.chatTab);
	rThr.attr("checked",threaded);
	rList.attr("checked",!threaded);
	cl[threaded?"hide":"show"]();
	ct[threaded?"show":"hide"]();
	if (ct.data("stick") && threaded)
	{
		t.ScrollTo(ct.data("stick"));
		ct.data("stick",F);
	
	}
	if (cl.data("stick") && !threaded)
	{
		t.ScrollTo(N,cl.data("stick"));
		cl.data("stick",F);
	}
}

/**
 * Sets the scroll height of the chat views to display a particular message
 *
 * @param mixed tID - the id of the message to scroll to or T to scroll to the end in the threaded view
 * @param mixed lID - the id of the message to scroll to or T to scroll to the end in the list view
 * @param boolean noStick - don't remember this scroll next time the threaded or list divs are displayed
 */
mccd.prototype.ScrollTo = function(tID,lID,noStick)
{
	var t = this;
		ct = $("div.chat_threaded",t.tabChat),
		cl = $("div.chat_list",t.tabChat),
		threaded = $('input[type="radio"][name="thread_view"]',t.tabChat).attr("checked");
	
	if (tID!=N && !noStick) ct.data("stick",tID);
	if (lID!=N && !noStick) cl.data("stick",lID);
	
	if (threaded)
	{
		MapChat.ChatLib.ScrollTo(((ct.data("stick")===T)?ct:$(".chat_id-"+ct.data("stick"),ct)),ct.data("stick"));
		ct.data("stick",N);
	}
	else
	{
		MapChat.ChatLib.ScrollTo(((cl.data("stick")===T)?cl:$(".chat_id-"+cl.data("stick"),cl)),cl.data("stick"));
		cl.data("stick",N);
	}
}

/**
 * Adds flags to the filterFlags array.
 *
 * @param string|array f - the flag or flags to add
 */
mccd.prototype.AddFilterFlag = function(f)
{
	var t = this;
	if (typeof(f)=="string") f = [f];
	$.each(f,function(i,v){
		if ($.inArray(f,t.filterFlags)==-1) t.filterFlags.push(v);
	});
}

/**
 * Removes flags from the filterFlags array.
 *
 * @param string|array f - the flag or flags to drop
 */
mccd.prototype.DropFilterFlag = function(f)
{
	var t = this;
	if (typeof(f)=="string") f = [f];
	$.each(f,function(i,v){
		var idx = $.inArray(v,t.filterFlags);
		if (idx!=-1) t.filterFlags.splice(idx,1);
	});
}

/**
 * Applies the selected location and/or view filters to the messages displayed (like limiting the scope of the chat)
 */
mccd.prototype.ApplyFilter = function()
{
	var t = this;
	
	if (t.filterFlags.length>0)
	{
		$('.chat_message:not(.'+t.filterFlags.join('.')+')',t.tabChat).addClass('chat_message_hidden');
		$('.chat_message.'+t.filterFlags.join('.'),t.tabChat).removeClass('chat_message_hidden');
		$('.chat_fake_message',t.tabChat)
			.filter(function(){ return $(this).children('.chat_message:not(.chat_message_hidden)').length>0; }).removeClass('chat_message_hidden');
		$('.chat_fake_message',t.tabChat)
			.filter(function(){ return $(this).children('.chat_message:not(.chat_message_hidden)').length==0; }).addClass('chat_message_hidden');
	}
	else
		$('.chat_message,.chat_fake_message',t.tabChat).removeClass('chat_message_hidden');
	
	t.ScrollTo(T,T);
}

/**
 * Applies the selected view to the map - essentially, it turns off all layers associated with the other views,
 *  turns on the the layers for the selected views, then updates the map.
 *
 * @param jQuery s - the select list containing the available view options
 */
mccd.prototype.ApplyView = function(s)
{
	var t = this,
		toggleLayers = N,
		dg = t.mapLayers.getDisplayGroups();
	
	if (s.attr("selectedIndex")>0)
	{
		s.children(":not(:first):not(:selected)").each(function(){
			var oDGroup = dg[$(this).data("view").attr("displaygroup")];
			oDGroup.getLayers();
			$.each(oDGroup.Layers,function(i,oLayer){
				if (oLayer.status)
				{
					if (!toggleLayers) toggleLayers = {};
					if (!toggleLayers[oLayer.mapfile.name]) toggleLayers[oLayer.mapfile.name] = {};
					toggleLayers[oLayer.mapfile.name][oLayer.name] = F;
				}
			});
			
			// This should really be handled by SubmitLayerStates
			oDGroup.status = F;
			oDGroup.partial_status = F;
			
		});
		
		var oDGroup = dg[s.children(":selected").data("view").attr("displaygroup")];
		oDGroup.getLayers();
		$.each(oDGroup.Layers,function(i,oLayer){
			if (!oLayer.status)
			{
				if (!toggleLayers) toggleLayers = {};
				if (!toggleLayers[oLayer.mapfile.name]) toggleLayers[oLayer.mapfile.name] = {};
				toggleLayers[oLayer.mapfile.name][oLayer.name] = T;
			}
		});
		
		// This should really be handled by SubmitLayerStates
		oDGroup.status = T;
		oDGroup.partial_status = T;
		
		if (toggleLayers)
		{
			t.mapLayers.SubmitLayerStates(toggleLayers);
		}
		
	}
	
}


// Functions that bound to various events...defined here so that it doesn't happen every time
//  an element is bound to the same function, rather than defining a new anonymous function
//  each time:

// For message div elements:
mccd.msgScrollTo = function(e,tID,lID){
	e.stopPropagation();
	if (arguments.length==2) lID = tID;
	e.data.t.ScrollTo(tID,lID);
};
