/**
 * A tool that populates the map with icons that, when clicked, display callout bubbles
 * containing spatially-linked chat messages.
 *
 * @class MapChat.Chat.Map
 *
 * Depends:
 *	kamap/kaMap.js
 *	MapChat.js
 *	MapChat/application.js
 *  excanvas.js
 *  jquery/jquery.bt.js
 */

// Get an alias for the the MapChat.Chat namespace (to make the code shorter).
var mcc = MapChat.UseNS("MapChat.Chat"),
	
	// used to stop propagation of click events on the chat icons.
	rf = function(e){e.stopPropagation();},
	
	mcqe = MapChat.QueueEvent;;

mcc.Map = function(options)
{
	var t = this,
		map = (t.map = options.map),
		ChatLib = (t.ChatLib = options.ChatLib),
		name = (t.name = "MapChat.Chat.Map"),
		zIndex = (t.zIndex = options.zIndex||700),
		container = (t.container = $('<div />').appendTo(t.map.theInsideLayer)),
		wasActive = (t.wasActive = F),
		ids = (t.ids = []),
		
		SelectLib = (t.SelectLib = MapChat.MapSingleton("MapChat.SelectLib",t.map)),
		DrawLib = (t.DrawLib = MapChat.MapSingleton("MapChat.DrawLib",t.map)),
		
		displayCanvas = (t.displayCanvas = map.defaultCanvas||MapChat.Class("MapChat.Canvas",{map:t.map,zIndex:600})),
		
		// Styles for selection/draw shapes:
		selectOpts = (t.selectOpts = options.selectOpts||{stroke:{stroke:"#00F",'stroke-width':2,'stroke-opacity':0.60},r:5}),
		selectFill = (t.selectFill = $.extend(options.selectFill||{fill:{fill:"#00F",'fill-opacity':0.35}},t.selectOpts)),
		extentOpts = (t.extentOpts = options.extentOpts||{stroke:{stroke:"#FCA",'stroke-width':2,'stroke-opacity':0.60},r:5}),
		extentFill = (t.extentFill = $.extend(options.extentFill||{fill:{fill:"#FCA",'fill-opacity':0.35}},t.extentOpts)),
		drawOpts = (t.drawOpts = options.drawOpts||{stroke:{stroke:"#77F",'stroke-width':2,'stroke-opacity':0.60},r:5}),
		drawFill = (t.drawFill = $.extend(options.drawFill||{fill:{fill:"#77F",'fill-opacity':0.35}},t.drawOpts));
	
	t.displayCanvas.activate();
	
	
	
	kaTool.apply( t, [map] );
	
	t.bInfoTool = T;
	
	/** get all the prototype functions from the kaTool object */
	for (var p in kaTool.prototype) {
		if (!mcc.Map.prototype[p])
			mcc.Map.prototype[p]= kaTool.prototype[p];
	}
	
	t.container.bind("EditedMessages",function(e,em){t.CheckEditedMessages(e,em);});
	
	t.container.bind("MessagesReceived",function(e,nm){
		t.AddNewMessages(e,nm);
		t.UpdateChat(nm);
	});
	
	t.container.bind("FakeMessagesReceived",function(e,fm){
		t.UpdateChat(fm);
	});
	
	t.container.bind("MapRefreshing",function(){t.container.hide();});
	
	t.container.bind("MapRefreshed",function(){t.repositionIcons(F,T);});
	
	// Need to make sure this happens before adding to the ChatLib context:
	t.container.bind("click.MCCM",function(e){
		ep = $(e.target).parent();
		if (ep.hasClass("chat_map_icon"))
		{
			mcqe(9,"ChatMap",["IconToggled",!ep.hasClass('bt-active')]);
			if (ep.data('content'))
			{
				if (ep.hasClass('bt-active')) ep.btOff();
				else
				{
					ep.btOn();
					t.FixBTHeight(ep.data('content'));
				}
			}
			else t.ShowCallout(ep);
			e.stopImmediatePropagation();
		}
		else if (ep.hasClass('chat_message_options') && !$(ep.context).filter('button').length>0)
		{
			mcqe(9,"ChatMap",["IconToggledOffByShowSVG",t.user]);
			$('.bt-active').btOff();
		}
		else if (!$.browser.msie && ep.hasClass('chat_message_options') && $(ep.context).filter('button').length>0)
		{
			mcqe(9,"ChatMap",["ButtonClicked",$(ep.context).attr("name")]);
			setTimeout(function(){t.FixBTHeight($('.bt-active').data('content'))},300);
		}
	});
	
	t.ChatLib.AddContext(t.container,t,'message-listener edit-listener chat-map-listener');
	MapChat.AddContext(t.map,t.container);
	
	return t;
}

var mccm = mcc.Map;

/**
 * activate this tool.  Activating the tool causes any existing tools to be
 * deactivated.
 */
mccm.prototype.activate = function()
{
	var t = this;
	
	if (t.active) return;
	
	t.map.registerForEvent( KAMAP_SCALE_CHANGED, t, t.repositionIcons );
	
	t.map.registerForEvent( KAMAP_EXTENTS_CHANGED, t, t.updateIcons );
	
	if (t.wasActive) t.repositionIcons();
	
	t.updateIcons();
	
	t.container.show();
	
	kaTool.prototype.activate.apply(t);
	
	t.active = t.wasActive = T;
	
	mcqe(9,"ChatMap",["IconsActivated"]);
	
};

/**
 * deactivate this tool.
 */
mccm.prototype.deactivate = function()
{
	var t = this;
	
	if (!t.active) return;
	
	t.map.deregisterForEvent( KAMAP_SCALE_CHANGED, t, t.repositionIcons );
	
	t.map.deregisterForEvent( KAMAP_EXTENTS_CHANGED, t, t.updateIcons );
	
	kaTool.prototype.deactivate.apply(t);
	
	t.container.hide();
	
	t.active = F;
	
	mcqe(9,"ChatMap",["IconsDeactivated"]);
	
};


/**
 * MapChat.Chat.Map.updateIcons()
 *
 * update the icons for spatially-linked chat.
 */
mccm.prototype.updateIcons = function()
{
	var t = this,
		ext = t.map.getGeoExtents();
	
	t.ChatLib.Get({listgeo:T,extent:ext,sinceID:0,limit:0,geocache:T},{ignore:T,after:function(xhr){t.ParseChatLinks(xhr);}});
	
	t.repositionIcons(N,T);
	
	$('.bt-active',t.container).btRefresh();
}

/**
 * MapChat.Chat.Map.placeIcons()
 *
 * update the positions of icons for spatially-linked chat messages:
 */
mccm.prototype.repositionIcons = function(icon,fast)
{
	var t = this;
	// This function will handle the repositioning/placement of any icons on theInsideLayer after the scale has changed.
	(icon&&icon["each"]?icon:t.container.children('.chat_map_icon')).each(function(){
		var geoxy = $(this).data("geoxy"),
			pix = t.map.geoToPix(geoxy[0],geoxy[1]),
			apix = [pix[0]-t.map.xOrigin,pix[1]-t.map.yOrigin];
			
		$(this)[(fast?'css':'animate')]({left:apix[0],top:apix[1]});
		
	});
	t.container.show();
	$('.bt-active',t.container).btRefresh();
}


/**
 * Icon object
 */
mccm.prototype.newIcon = function(geoxy,md,cl)
{
	var t = this,
		pix = t.map.geoToPix(geoxy[0],geoxy[1]),
		apix = [pix[0]-t.map.xOrigin,pix[1]-t.map.yOrigin];
	
	return $('<a chat_map_icon_id="'+md.id+'" class="chat_map_icon icon new_map_icon'+(cl?' '+cl:'')+'" role="button" unselectable="on" style="-moz-user-select: none; z-index:'+t.zIndex+'; left:'+apix[0]+'px; top:'+apix[1]+'px;">'+
			'<span class="ui-icon">&nbsp;</span>'+
		'</a>').data("geoxy",geoxy).data("md",md).appendTo(t.container);
		//.children('span.ui-icon').click(function(){t.ShowCallout($(this).parent());});
}

/**
 * Parse chat links.
 */
mccm.prototype.ParseChatLinks = function(xhr)
{
	var t = this;
	
	$.each($.parseJSON($("chat_messages:first",$(xhr.responseXML)).text()),function(i,md){
		if ($.inArray(md.id,t.ids)<0)
		{
			t.ids.push(md.id);
			
			MapChat.ChatLib.FixMsgExtents(md);
			
			if (md.extent)
				t.newIcon([(md.extent[0]+md.extent[2])/2,(md.extent[1]+md.extent[3])/2],md,"extent");
			if (md.selection)
				t.newIcon(md.selection.point,md,"selection");
			if (md.drawing)
				t.newIcon(md.drawing.point,md,"drawing");
		}
		else
		{
			t.UpdateEditedIcon(md);
		}
	});
	
	t.container.dblclick(rf).mousedown(rf).mouseup(rf)
		.children('.chat_map_icon.new_map_icon')
		.removeClass('.new_map_icon');
}

/**
 * Check any edited links.
 */
mccm.prototype.CheckEditedMessages = function(e,em)
{
	var t = this,
		dids = [];
	$.each(em,function(i,id){
		var md = t.ChatLib.msgData.children('.chat_id-'+id).data("md");
		if ($.inArray(parseInt(id),t.ids)>-1) dids.push(id);
		else if (md.extent || md.selection || md.drawing)
		{
			t.ids.push(md.id);
			
			MapChat.ChatLib.FixMsgExtents(md);
			
			if (md.extent)
				t.newIcon([(md.extent[0]+md.extent[2])/2,(md.extent[1]+md.extent[3])/2],md,"extent");
			if (md.selection)
				t.newIcon(md.selection.point,md,"selection");
			if (md.drawing)
				t.newIcon(md.drawing.point,md,"drawing");
		}
	});
	if (dids.length>0)
		t.ChatLib.Get({listgeo:T,sinceID:0,limit:0,geocache:T,describe:dids},{ignore:T,after:function(xhr){t.ParseChatLinks(xhr);}});
}

/**
 * Add any new links.
 */
mccm.prototype.AddNewMessages = function(e,nm)
{
	var t = this;
	$.each(nm,function(i,id){
		var md = t.ChatLib.msgData.children('.chat_id-'+id).data("md");
		if (md.extent || md.selection || md.drawing)
		{
			t.ids.push(md.id);
			
			MapChat.ChatLib.FixMsgExtents(md);
			
			if (md.extent)
				t.newIcon([(md.extent[0]+md.extent[2])/2,(md.extent[1]+md.extent[3])/2],md,"extent");
			if (md.selection)
				t.newIcon(md.selection.point,md,"selection");
			if (md.drawing)
				t.newIcon(md.drawing.point,md,"drawing");
		}
	});
}

/**
 * Update edited links.
 */
mccm.prototype.UpdateEditedIcon = function(md)
{
	var t = this,
		icons = t.container.children('[chat_map_icon_id="'+md.id+'"]'),
		extIcon = icons.filter('.extent'),
		selIcon = icons.filter('.selection'),
		drawIcon = icons.filter('.drawing');
	
	MapChat.ChatLib.FixMsgExtents(md);
	
	if (md.extent)
	{
		var geoxy = [(md.extent[0]+md.extent[2])/2,(md.extent[1]+md.extent[3])/2];
		if (extIcon.length>0)
		{
			extIcon.data("geoxy",geoxy);
			t.repositionIcons(extIcon);
		}
		else t.newIcon(geoxy,md,"extent");
	}
	else if (extIcon.length>0) extIcon.remove();
	
	if (md.selection)
	{
		if (selIcon.length>0)
		{
			selIcon.data("geoxy",md.selection.point);
			t.repositionIcons(selIcon);
		}
		else t.newIcon(md.selection.point,md,"selection");
	}
	else if (selIcon.length>0) selIcon.remove();
	
	if (md.drawing)
	{
		if (drawIcon.length>0)
		{
			drawIcon.data("geoxy",md.drawing.point);
			t.repositionIcons(drawIcon);
		}
		else t.newIcon(md.drawing.point,md,"drawing");
	}
	else if (drawIcon.length>0) drawIcon.remove();
	
	if (!(md.extent || md.selection || md.drawing))
	{
		t.ids.splice($.inArray(md.id,t.ids),1);
	}
	
}


/**
 * Show the callout for an icon...
 */
mccm.prototype.ShowCallout = function(icon)
{
	var t = this;
	
	if (icon.hasClass('bt-active')) icon.btOff();
	else
	{
		var md = icon.data("md"),
			w = icon.width()/2,
			h = icon.height()/2,
			l = parseInt(icon.css("left")),
			tp = parseInt(icon.css("top")),
			
			// look for neighbours - this should return at least the original icon, but
			// also any others that overlap it:
			neighbours = t.container.children('.chat_map_icon').filter(function(){
				return (Math.abs(parseInt($(this).css("left"))-l)<w && Math.abs(parseInt($(this).css("top"))-tp)<h);
			}),
			nbIDs = [];
			
			// Now get all the chat-ids
			neighbours.each(function(){
				if ($.inArray($(this).data("md").id,nbIDs)==-1)
					nbIDs.push($(this).data("md").id);
			});
		
		var btct = $('<div class="autoscroll mccd chat_messages" style="cursor:default;">'+
			'<a style="cursor:pointer;">'+
				'<img style="position:absolute;top:3px;right:3px;width:9px;height:9px;font-size:9px;" src="images/ui/888888_9x9_icon_close.gif" alt="x" />'+
			'</a>'+
		'</div>').data("icon",icon),
			msgData = t.ChatLib.msgData.children('.chat_id-'+nbIDs.join(',.chat_id-'));
		
		btct.children('a').click(function(){icon.btOff();})
			.hover(function(){$(this).attr("src","images/ui/222222_9x9_icon_close.gif");},
				function(){$(this).attr("src","images/ui/888888_9x9_icon_close.gif");});
		
		icon.data('btct',btct);
		
		$.blockUI();
		
		// If the messages are already cached:
		if (msgData.length>=nbIDs.length)
		{
			t.PopulateBT(btct,icon,msgData);
		}
		
		// Otherwise, we need to download them first:
		else
		{
			dIDs = [];
			$.each(nbIDs,function(i,id){
				if (msgData.filter('.chat_id-'+id).length==0) dIDs.push(id);
			});
			if (dIDs.length>0)
			{
				t.ChatLib.Get({describe:dIDs},{ignore:T,after:function(){
					var newMsgData = t.ChatLib.msgData.children('.chat_id-'+nbIDs.join(',.chat_id-'));
					t.PopulateBT(btct,icon,newMsgData);
				}});
			}
		}
	}
}

/**
 * Fill the BT element with a selection of messages:
 */
mccm.prototype.PopulateBT = function(btct,icon,msgData)
{
	var t = this;
	
	btct.append(t.ChatLib.DisplayAllMessages(T,T,msgData));
	
	icon.bt({cssClass:'callout',contentSelector:btct,noClone:T,
		trigger:'none',clickAnywhereToClose:F,closeWhenOthersOpen:T,
		width:'350px',cornerRadius:3,fill:'#FFF'}).btOn();
	
	t.ChatLib.BindMessageElems('chat_new_elem',T,{map:t.map,SelectLib:t.SelectLib,DrawLib:t.DrawLib});
	btct.dblclick(rf).mousedown(rf).mouseup(rf);
	if (window.addEventListener && navigator.product && navigator.product == "Gecko") {
		btct.get(0).addEventListener( "DOMMouseScroll", function(e){e.stopPropagation();}, false );
	}
	t.FixBTHeight(btct);
	btct.parents('.bt-wrapper:first').mouseenter(function(){t.map.scrollDisabled=T;})
		.mouseleave(function(){t.map.scrollDisabled=F;});
	$.unblockUI();
}

/**
 * This function should trigger whenever new messages are received.
 */
mccm.prototype.UpdateChat = function(chat_ids,scroll)
{
	var t = this,
		newids = [],
		icon = t.container.children('.bt-active'),
		btct = icon.data('content');
	
	if (!btct) return;
	
	$('.chat_message,chat_fake_message',btct).each(function(){
		var dm = $(this),
			md = dm.data("md")||{replies:[]};
		$.each(chat_ids,function(i,id){
			if ($.inArray(id,newids)<0 && (dm.attr('chat_id')==id || dm.hasClass('.chat_pid-'+id) || 
				$.inArray(id,md.replies)>-1)) newids.push(id);
		});
	});
	
	msgData = t.ChatLib.msgData.children('.chat_id-'+newids.join(',.chat_id-'));
	
	msgData.each(function(){
		var msg = $(this),
			dm = t.ChatLib.DisplayMessage(msg,{showReplies:T});
		MapChat.ChatLib.AppendThread(dm,btct);
	});
	t.ChatLib.BindMessageElems('chat_new_elem',T,{map:t.map,SelectLib:t.SelectLib,DrawLib:t.DrawLib});
	t.FixBTHeight(btct);
	
}


/**
 * Fix the height of any callout that is taller than the window:
 */
mccm.prototype.FixBTHeight = function(btct)
{
	var t = this,
		icon = btct.data("icon"),
		btparent = btct.parent();
	
	if (!icon.hasClass('bt-active')) return;
	
	if (icon && icon.length>0)
	{
		if ((btct.attr('scrollHeight')+40)>$(window).height())
		{
			btct.height($(window).height()-40);
			icon.btRefresh();
		}
		else
		{
			btct.height(btct.attr('scrollHeight')+($.browser.msie?22:0));
			icon.btRefresh();
		}
		
		if ($.browser.msie)
		{
			if ((btct.attr('scrollHeight')+22)>btct.height())
				btct.children().css('width','327px');
			else
				btct.children().css('width','345px');
		}
	}
}
