/**
 * An object that presents a user list, suitable primarily for the user tab in the chat dialog.
 *
 * @class MapChat.Chat.UserList
 *
 * @param map - a kaMap class instance. If not defined, it will default to MapChat.map.
 * @param chat - a ChatLib class instance, which includes the methods required for
 *               keeping chat messages synchronized with the server (e.g., Get(),
 *               Send(), etc.).  If not defined, it will default to MapChat.chat.
 * @param options - an associative array of options that configure the behaviour of the dialog.
 *
 * Depends:
 *	kamap/htdocs/kaMap.js
 *	MapChat.js
 *	MapChat/application.js
 *	MapChat/MapLayers.js
 *  jquery/jquery.colorPicker.js
 */

// Get an alias for the the MapChat.Chat namespace (to make the code shorter).
var mcc = MapChat.UseNS("MapChat.Chat"),

	// Alias the disable/enable button functions and the QueueEvent function:
	dbtn = MapChat.DisableButton, ebtn = MapChat.EnableButton, mcqe = MapChat.QueueEvent;
	
mcc.UserList = function(map,chat,options)
{
	var t = this,
		changedItems = (t.changedItems = {me:F,all:F,users:{}}),
		map = (t.map = map||MapChat.map),
		mapLayers = (t.mapLayers = MapChat.MapSingleton("MapChat.MapLayers",t.map)),
		ChatLib = (t.ChatLib = chat||MapChat.chat),
		list = (t.list = $('<div class="user_list autoscroll">'+
			'<div class="list_loading">Loading...</div>'+
			'<div class="current_user hidden" name="'+MapChat.session.user_identity+'">'+
				'<p style="line-height:18px;">'+
					'<a class="chat_user_icon icon" name="'+MapChat.session.user_identity+'" role="button" unselectable="on" style="-moz-user-select: none;">'+
						'<span class="ui-icon">&nbsp;</span>'+
					'</a>'+
					' <strong>'+MapChat.session.display_name+'</strong>'+
				'</p>'+
			'</div>'+
			'<div class="all_users hidden">'+
				'<p style="line-height:18px;">'+
					'<a class="chat_user_icon icon" role="button" unselectable="on" style="-moz-user-select: none;">'+
						'<span class="ui-icon">&nbsp;</span>'+
					'</a>'+
					' <strong>All Users</strong>'+
				'</p>'+
			'</div>'+
		'</div>'));
	
	$('.chat_user_icon',t.list).bind("UserOnline",MapChat.ChatLib.UserOnline).bind("UserOffline",MapChat.ChatLib.UserOffline);
	
	// Get the users list, and populate the list div on the callback:
	MapChat.RunFunction("User_ListAll",{args:{mapfile:map.currentMap}},{
		async:T,
		complete: function(xhr){
			try
			{
				t.ParseUserList(xhr);
			}
			catch (e)
			{
				MapChat.Debug(["warn","User_List: Error parsing AJAX response: ",e,xhr.responseText]);
			}
		},
		error: function(xhr,sStatus,nError){
			MapChat.Debug(["warn","User_List: Error on AJAX request. Status: %s Error: %s ",sStatus,nError]);
		}
	});
	
	// Delegating some stuff here:
	t.list.click(function(e){
		var et = $(e.target);
		switch (true)
		{
			case (et.hasClass("private_message")):
				var username = et.parents(".discussion_member:first").attr("name");
				t.ChatLib.GetPrivateDialog({target_users:([username,MapChat.session.user_identity].sort())},t.map).Open();
			break;
		}
	});
}

mcc.UserList.prototype.ParseUserList = function (xhr)
{
	var t = this,
		users = $.parseJSON($("user_list:first",xhr.responseXML).text()),
		du = MapChat.discussion.users;
	
	t.list.children('.list_loading').remove();
	t.list.children('.hidden').removeClass('hidden');
	
	$.each(users,function(i,u){
		user = du.children('p[id="'+u.username+'"]');
		if (user.length==0)
			user = $('<p id="'+u.username+'" />').data("u",u)
				.appendTo(MapChat.discussion.users);
		else
			user.data("u",u);
		
		if (u.username == MapChat.session.user_identity)
		{
			t.list.children('div.current_user')
				.append('<p><input type="text" name="draw_color" value="'+u.draw_color+'" style="display:none;" /> '+
					'<input type="checkbox" name="show_drawings"'+(u.show_drawings?' checked="true"':'')+
					' value="'+MapChat.session.user_identity+'" /> Display my drawings</p>');
			$('input[name="draw_color"]',t.list.children('div.current_user')).colorPicker().change(function(){
				t.SetUserDrawState("me",t.list.children('div.current_user'));
			});
			$('input[type="checkbox"]',t.list.children('div.current_user')).click(function(){
				t.SetUserDrawState("me",t.list.children('div.current_user'));
			});
		}
		else if (u.all_users)
		{
			t.list.children('div.all_users')
				.append('<p><input type="text" name="draw_color" value="'+u.draw_color+'" style="display:none;" /> '+
					'<input type="checkbox" name="show_drawings"'+(u.show_drawings?' checked="true"':'')+
					' /> Display all users\' drawings</p>');
			$('input[name="draw_color"]',t.list.children('div.all_users')).colorPicker().change(function(){
				t.SetUserDrawState("all",t.list.children('div.all_users'));
			});
			$('input[type="checkbox"]',t.list.children('div.all_users')).click(function(){
				t.SetUserDrawState("all",t.list.children('div.all_users'));
			});
		}
		else
		{
			var userP = $('<div class="discussion_member" name="'+u.username+'">'+
					'<p style="float:left;line-height:18px;">'+
						'<a class="chat_user_icon icon '+(u.online?'':'offline')+'" name="'+u.username+'" role="button" unselectable="on" style="-moz-user-select: none;">'+
							'<span class="ui-icon">&nbsp;</span>'+
						'</a>'+
						' <strong>'+u.display_name+'</strong>'+
					'</p>'+
					'<p class="alignright" style="line-height:18px;">'+
						'(<span class="link private_message" name="'+u.username+'">private message</span>)'+
					'</p>'+
					'<p>'+
						'<input type="text" name="draw_color" value="'+u.draw_color+'" style="display:none;" /> '+
						'<input type="checkbox" name="show_drawings"'+
							(u.show_drawings?' checked="true"':'')+' value="'+u.username+'" /> Display drawings'+
					'</p>'+
				'</div>').appendTo(t.list);
			$('input[name="draw_color"]',userP).colorPicker().change(function(){
				t.SetUserDrawState("user",userP);
			});
			$('input[type="checkbox"]',userP).click(function(){t.SetUserDrawState("user",userP);});
		}
		
		$('.chat_user_icon',t.list.children('div.discussion_member'))
			.bind("UserOnline",MapChat.ChatLib.UserOnline).bind("UserOffline",MapChat.ChatLib.UserOffline);
	
	});
}

// Adds this user to the states to send to the server when the 'apply' button is clicked.
mcc.UserList.prototype.SetUserDrawState = function(utype,obj)
{
	var t = this;
	switch (utype)
	{
		case "me":
		case "all":
			t.changedItems[utype] = obj;
		break;
		
		default:
			t.changedItems.users[obj.attr("name")] = obj;
		break;	
	}
	
	mcqe(9,"ChatDialog",["SetUserDrawState",utype,
		($('input[name="show_drawings"]',obj).attr("checked")?"on":"off"),
		$('input[name="draw_color"]',obj).val()]);
	
	ebtn($('button[name="apply"]',t.list.parent()));
}

mcc.UserList.prototype.ApplyUserStates = function()
{
	var t = this,
		ustates = {users:[]};
		
	$.each(t.changedItems,function(utype,obj){
		
		switch(utype)
		{
			case "me":
			case "all":
				if (obj) ustates[utype] = {
					show:$('input[name="show_drawings"]',obj).attr('checked'),
					color:$('input[name="draw_color"]',obj).val()
				};
			break;
			
			default:
				$.each(obj,function(i,u){
					ustates.users.push({
						username:i,
						show:$('input[name="show_drawings"]',u).attr('checked'),
						color:$('input[name="draw_color"]',u).val()
					});
				});
			break;
		}
	});
	
	var aus = $.parseJSON($('user_states:first',
		MapChat.RunFunction("User_States",{args:{mapfile:t.map.currentMap,ustates:$.toJSON(ustates)}})).text()
	);
	
	if (aus)
	{
		dbtn($('button[name="apply"]',t.list.parent()));
		t.changedItems = {me:F,all:F,users:{}};
		
		// Make sure the drawing layers are displayed if one or more are turned off:
		var ul = t.mapLayers.getDisplayGroups()["User Drawings"];
		if (!ul.status || ul.partial_status)
		{
			var toggleUL = {};
			toggleUL[t.map.currentMap] = {"User Drawings (point)":true,"User Drawings (line)":true,"User Drawings (polygon)":true};
			t.mapLayers.SubmitLayerStates(toggleUL);
		}
		else
		{
			// Otherwise, force the layer to redraw:
			t.map.getCurrentMap().getLayer("USER_LAYERS").setTileLayer(true);
		}
	}
	
	$(".display-group-listener",t.map.context).trigger("ResetDisplayGroup",["User Drawings"]);
	
}