/**
 * A library for handling the features selected in a map object
 *
 * @class MapChat.SelectLib
 *
 * @param object map - the kaMap map object that features will be selected in.
 * @param object opts - options
 *
 * Depends:
 *	kamap/kaMap.js
 *	MapChat.js
 *	MapChat/application.js
 */
MapChat.SelectLib = function(map,opts)
{
	var t = this,
		map = (t.map = map||MapChat.map),
		features = (t.features = {}),
		selectIDs = (t.selectIDs = []),
		count = (t.count = 0),
		hasLinked = F;
	
	// Check for any existing selection to start off:
	t.GetSelection({map:map.currentMap,cellSize:map.cellSize,mode:'list'});
}

//shortcut to the MapChat.SelectLib class:
var mcsl = MapChat.SelectLib;

/**
 * Sends selection options to the server, and parses the results.
 *
 * @param object opts - options for the Select_Feature API
 * @param boolean quiet - if true, don't trigger the listeners (i.e., when the lib is first loaded).
 *
 * Possible values that can be included in the opts parameter:
 *  mode: one of 'add', 'remove', 'clear', 'list' - defines wither to add new features, remove features, clear, or list the current selection
 *  style: one of 'point', 'circle', 'rectangle', 'by_id' - defines how the selection will be performed
 *  layer: either the name of a vector layer in the map, or '[All Visible Layers]'
 *  coords: the coordinates that are to be used to search for features
 *  ids: if style is 'by_id', and mode is 'remove', this should be a list of ids from the user_selection table to remove
 */
mcsl.prototype.GetSelection = function(opts,quiet)
{
	var t = this;
	$.extend(opts,{map:t.map.currentMap,cellSize:t.map.cellSize});
	MapChat.RunFunction("Select_Features",{args:{opts:$.toJSON(opts)}},{
		async:T,
		complete:function(xhr){
			try
			{
				t.ParseSelection(xhr,quiet);
			}
			catch (e)
			{
				MapChat.Debug(["error","Select_Features: Error parsing AJAX response: ",e,xhr.responseText]);
			}
		},
		error: function(xhr,sStatus,nError){
			MapChat.Debug(["warn","Select_Features: Error on AJAX request. Status: %s Error: %s ",sStatus,nError]);
		}
	});
}

/**
 * Parses a selection result, and triggers events on the map's context elements
 * that are classed as select-listener:
 */
mcsl.prototype.ParseSelection = function(xhr,quiet)
{
	var t = this,
	 sel = $("select_results:first",xhr.responseXML);
	switch(sel.attr("mode"))
	{
		case "clear":
			t.features = {};
			t.count = 0;
			t.hasLinked = F;
			t.selectIDs.splice(0,t.selectIDs.length);
			if (!quiet) $(".select-listener",t.map.context).trigger("UpdateSelection");
		break;
		
		case "list":
			t.features = $.parseJSON(sel.text()).list||{};
			t.count = 0;
			t.selectIDs.splice(0,t.selectIDs.length);
			$.each(t.features,function(l,f){$.each(f,function(i){ t.selectIDs.push(i); t.count++;});});
			t.hasLinked = T;  // 'list' usually happens at startup, which means this is an old selection.
			if (!quiet) $(".select-listener",t.map.context).trigger("UpdateSelection");
		break;
		
		default:
			var f = $.parseJSON(sel.text());
			if (sel.attr("mode")=="remove" && f===T)
			{
				t.features = {};
				t.count = 0;
				t.selectIDs.splice(0,t.selectIDs.length);
			}
			else
			{
				if (f.add) $.each(f.add,function(layer,af){
					if (!t.features[layer]) t.features[layer] = {};
					$.each(af,function(id,fd){
						if ($.inArray(id,t.selectIDs)==-1) t.selectIDs.push(id);
						t.features[layer][id] = fd;
						t.count++;
					});
				});
				
				if (f.remove) $.each(f.remove,function(layer,rf){
					if (t.features[layer]) $.each(rf,function(id){
						if ($.inArray(id,t.selectIDs)>=0) t.selectIDs.splice($.inArray(id,t.selectIDs),1);
						if (t.features[layer][id]) delete t.features[layer][id];
						t.count--;
					});
				});
			}
			t.hasLinked = F;
			if (!quiet) $(".select-listener",t.map.context).trigger("UpdateSelection");	
		break;
		
	}
}
