/**
 * An object that presents tools for Selecting and Drawing Features
 *
 * @class MapChat.SpatialTools.Dialog
 *
 * @param map - a kaMap class instance. If not defined, it will default to MapChat.map.
 * @param options - an associative array of options that configure the behaviour of the dialog.
 *
 * Depends:
 *	kamap/htdocs/kaMap.js
 *	MapChat.js
 *	MapChat/application.js
 *	MapChat/MapLayers.js
 *	jquery/ui/ui.dialog.js
 *  jquery/jquery.scrollTo.js
 */

MapChat.Using("jquery.scrollTo");

// Get an alias for the the MapChat.SpatialTools namespace (to make the code shorter).
var mcst = MapChat.UseNS("MapChat.SpatialTools"),

	// Alias the disable/enable button functions and the QueueEvent function:
	dbtn = MapChat.DisableButton, ebtn = MapChat.EnableButton, mcqe = MapChat.QueueEvent;

mcst.Dialog = function(map,options)
{
	var t = this,
		options = (t.options = options||{}),
		name = (t.name = "MapChat.SpatialTools.Dialog"),
		zIndex = (t.zIndex = options.zIndex||300),
		map = (t.map = map||MapChat.map),
		mapLayers = (t.mapLayers = MapChat.MapSingleton("MapChat.MapLayers",map)),
		SelectLib = (t.SelectLib = MapChat.MapSingleton("MapChat.SelectLib",map)),
		DrawLib = (t.DrawLib = MapChat.MapSingleton("MapChat.DrawLib",map)),
		layers = (t.layers = []),
		drawings = (t.drawings = []),
		drawingIDs = (t.drawingIDs = {}),
		mode = (t.mode = "select"),
		tbItems = (t.tbItems = N),
		selectTool = (t.selectTool = N),
		drawTool = (t.drawTool = N),
		curTool = (t.curTool = N),
		toolOpts = (t.toolOpts = N),
		
		// Styles for displaying selected items:
		displayOpts = (t.displayOpts = options.displayOpts||{stroke:{stroke:"#0FF",'stroke-width':2,'stroke-opacity':0.60},r:5}),
		displayFill = (t.displayFill = $.extend(options.displayFill||{fill:{fill:"#0FF",'fill-opacity':0.35}},t.displayOpts)),
		hoverOpts = (t.hoverOpts = options.hoverOpts||{stroke:{stroke:"#FF0",'stroke-width':2,'stroke-opacity':0.60}}),
		hoverFill = (t.hoverFill = $.extend(options.hoverFill||{fill:{fill:"#FF0",'fill-opacity':0.35}},t.hoverOpts)),
		simpleFill = (t.simpleFill = {display:'none',stroke:{stroke:"#FA0",'stroke-width':2,'stroke-opacity':0.50},
			fill:{fill:"#FA0",'fill-opacity':0.25}}),
		
		content = (t.content = 
			$('<div class="autoscroll mcst spatialtool-listener" style="font-size:.8em;line-height:16px"></div>')
			.dialog({title:"Select/Draw Features",width:300,height:300,modal:false,autoOpen:false,closeOnEscape:false,
				resizeStop:function()
				{
					// Make the continer is fit to the dialog's height (so we get scrolling):
					t.content.css({height:$(".ui-resizable-s", t.dialog).offset().top-t.content.offset().top});
					t.divTabs.css({height:t.content.height()-7});
					t.divTabs.children(".ui-tabs-panel").css({height:t.divTabs.height() - t.divTabs.children('ul:first').outerHeight()});
					t.curTab.trigger("tabResize");
				},
				open:function()
				{
					t.content.css({height:$(".ui-resizable-s", t.dialog).offset().top-t.content.offset().top});
					t.divTabs.css({height:t.content.height()-7});
					t.divTabs.children(".ui-tabs-panel").css({height:t.divTabs.height() - t.divTabs.children('ul:first').outerHeight()});
					t.curTab.trigger("tabResize");
				},
				close:function()
				{
					mcqe(8,"SpatialTools",["DialogClosed"]);
					if (t.map.toolGroups["nav"] && (ai = t.map.toolGroups["nav"].activeItem))
						ai.deactivate();
					t.map.defaultTool.activate();
					t.displayCanvas.deactivate();
				}
			})
		),
		
		/** TODO: calculate the number of tabs required based on any config issues **/
		panelIDs = (t.panelIDs = MapChat.UID("mcst",2)),
		panelHeight = (t.panelHeight = N),
		
		/** TODO: include other tabs here somehow **/
		divTabs = (t.divTabs = $('<div><ul><li><a href="#'+t.panelIDs[0]+'"><span>Select</span></a></li><li><a href="#'+t.panelIDs[1]+'"><span>Draw</span></a></li></ul><div id="'+t.panelIDs[0]+'" /><div id="'+t.panelIDs[1]+'" /></div>')).appendTo(t.content)
			.tabs({show:function(){t.ShowTab();}})
			.bind("tabsselect",function(){return t.SelectTab();}),
		
		ulTabs = (t.ulTabs = $("ul:first",t.divTabs)),
		curTab = (t.curTab = $("#"+t.panelIDs[0],divTabs)),
		
		// Setup the Select tab:
		tabSelect = (t.tabSelect = $("#"+t.panelIDs[0],divTabs)).append(
			'<div class="select_toppanel">'+
				'Select from: <br /><input name="select_layer" class="dropDown" type="text" style="width:250px;" /><div class="dropDownButton" />'+
				'<p class="select_mode" style="clear:both;">'+
					'Select style: <img class="icon png24 select_rectangle" name="rectangle" src="images/icons/select_rectangle.png" title="Point or Rectangle" /> | <img class="icon png24 select_circle" name="circle" src="images/icons/select_circle.png" title="Circle" /><br />'+
					'Select mode: <input value="add" type="radio" checked="true" /> Add &nbsp;&nbsp;&nbsp;<input value="remove" type="radio" /> Remove'+
				'</p>'+
			'</div>'+
			'<div class="select_history autoscroll">No features currently selected.</div>'+
			'<div class="buttonpanel">'+
				'<button class="ui-state-default ui-corner-all" name="select_clear">Clear Selection</button>'+
				'<button class="ui-state-default ui-corner-all" style="display:none" name="select_save">Save Selection</button>'+
			'</div>'
		).bind("tabResize",function(){
			var sh = $('div.select_history',t.tabSelect);
			sh.css({height:(sh.parent().height()-sh.prev().outerHeight()-sh.next().outerHeight())});
		}).trigger("tabResize").data("mode","select"),
		
		// Setup the Draw tab:
		tabDraw = (t.tabDraw = $("#"+t.panelIDs[1],divTabs)).append(
			'<div class="draw_toppanel">'+
				'Previous drawings: <input name="select_drawing" class="dropDown" type="text" style="width:250px;" /><div class="dropDownButton" />'+
				'<p class="draw_tools" style="clear:both;">'+
					'Draw shape: <img class="icon png24 draw_polygon" name="polygon" src="images/icons/draw_polygon.png" title="Polygon" />'+
					' | <img class="icon png24 draw_circle" name="circle" src="images/icons/draw_circle.png" title="Circle" />'+
					' | <img class="icon png24 draw_line" name="line" src="images/icons/draw_line.png" title="Line" />'+
					' | <img class="icon png24 draw_point" name="point" src="images/icons/draw_point.png" title="Point" /><br />'+
					'<span style="margin-left:20px;"><input type="checkbox" name="freehand" /> Draw freehand polygons/lines<br /></span>'+
					'<span class="draw_mode">Draw mode: <input value="add" type="radio" checked="true" /> Add &nbsp;&nbsp;&nbsp;<input value="cut" type="radio" /> Cut</span>'+
				'</p>'+
			'</div>'+
			'<div class="draw_history autoscroll">'+
				'<span class="no_drawing">No features currently drawn.</span>'+
				'<div class="drawing_options hidden">'+
					'<p>Label: <input type="text" name="drawing_label" /><br />'+
					'<input type="checkbox" name="drawing_visible" /> Visible in map</p>'+
					'<p class="alignright">'+
						'<button name="undo_last_draw" class="ui-state-default ui-corner-all ui-state-disabled" disabled="true">Undo</button>&nbsp;'+
						'<button name="delete_drawing" class="ui-state-default ui-corner-all">Delete</button>&nbsp;&nbsp;'+
						'<button name="apply_settings" class="ui-state-default ui-corner-all ui-state-disabled" disabled="true">Apply</button>'+
					'</p>'+
					'<p class="drawing_measurement">'+
				'</div>'+
			'</div>'+
			'<div class="buttonpanel">'+
				'<button class="ui-state-default ui-corner-all" name="draw_clear">New Drawing</button>'+
				'<button class="ui-state-default ui-corner-all" style="display:none" name="draw_save">Save Drawing</button>'+
			'</div>'
		).bind("tabResize",function(){
			var dh = $('div.draw_history',t.tabDraw);
			dh.css({height:(dh.parent().height()-dh.prev().outerHeight()-dh.next().outerHeight())});
		}).trigger("tabResize").data("mode","draw"),
		
		// Setup the layer-list autocomplete elements:
		layerInput = (t.layerInput = $('input[name="select_layer"]:first',tabSelect)),
		ddButton = (t.ddButton = layerInput.next().click(function(){layerInput.trigger("ddToggle");})),
		
		drawSelect = (t.drawSelect = $('input[name="select_drawing"]:first',tabDraw)),
		dselButton = (t.dselButton = drawSelect.next().click(function(){drawSelect.trigger("ddToggle");})),
		
		// This will be the displayCanvas used by the select/draw tools:
		displayCanvas = (t.displayCanvas = map?MapChat.Class("MapChat.Canvas",{map:t.map,zIndex:600}):N),
		displayGeom = (t.displayGeom = N),
		
		// Styles for the current 'draw' shape:
		drawOpts = (t.drawOpts = options.drawOpts||{stroke:{stroke:"#0F0",'stroke-width':2,'stroke-opacity':0.60},r:0}),
		drawFill = (t.drawFill = $.extend(options.drawFill||{fill:{fill:"#0F0",'fill-opacity':0.35}},t.drawOpts)),
		pointFill = (t.pointFill = $.extend($.extend(options.pointFill||{fill:{fill:"#0F0",'fill-opacity':0.60}},t.drawOpts),{r:5})),
		
		dialog = (t.dialog = content.parents(".ui-dialog:first")),
		
		freehand = (t.freehand = $('input[name="freehand"]',t.tabDraw)),
		addcut = (t.addcut =  $('input[type="radio"]',t.tabDraw));
	
	MapChat.AddContext(t.map,t.dialog);
	
	t.layerInput.data("lastVal","[All Visible Layers]");
	
	t.tabSelect.addClass("select-listener layerlist-listener").bind("LayerListUpdate",function(){
		t.layerInput.unautocomplete();
		var tempLayers = [];
		$.each(t.mapLayers.getLayerNames(null,{type:[0,1,2]}),function(i,v){
			tempLayers.push(v);
		});
		t.layers = tempLayers.sort();
		t.layers.unshift("[All Visible Layers]");
		if ($.inArray(t.layerInput.val(),t.layers)==-1) t.layerInput.val("[All Visible Layers]");
		t.layerInput.autocomplete(t.layers, {
			minChars: 0,
			selectFirst: false,
			max: 999,
			highlight:false,
			mustMatch: true,
			matchContains: true,
			scrollHeight: 220,
			// some quirk in the autocomplete causes troubles if the default multipleSperator of ', ' is encountered in an item's text:
			multipleSeparator: "||multisep||"
		}).result(function(){
			var li = $(this);
			if (li.val()!="")
			{
				t.selectTool.layer = li.val();
				li.data("lastVal",li.val());
			}
			else li.val(li.data("lastVal"));
		}).blur(function(){
			var li = $(this);
			if ($.inArray(li.val(),t.layers)==-1) li.val(li.data("lastVal"));
		});
	}).bind("UpdateSelection",function(){
		t.UpdateSelection();
	});
	
	// Build a straight list of map layers with visibility determined by their on/off states:
	var tempLayers = [];
	$.each(mapLayers.getLayerNames(null,{type:[0,1,2]}),function(i,v){
		tempLayers.push(v);
	});
	t.layerInput.val("[All Visible Layers]");
	t.layers = tempLayers.sort();
	t.layers.unshift(t.layerInput.val());
	
	$("img.icon",t.tabSelect).css({border:'1px solid transparent',margin:'2px 0px -1px 0px'});
	
	$("img.icon",t.tabSelect).hover(
		function(){$(this).css({'border-color':'#00F'});},
		function(){
			var ic = $(this);
			if (ic.attr("name")!=t.selectTool.style) ic.css({'border-color':'transparent'});
			else ic.css({'border-color':'#F00'});
		}
	).tooltip({track:T,delay:0,showURL:F,extraClass:"tooltip"})
	.click(function(){
		tb = t.map.toolGroups["nav"].itemsNamed[t.tbItems[t.mode]];
		if (tb != t.map.toolGroups["nav"].activeItem) tb.el.click();
		$("img.icon",t.tabSelect).css({'border-color':'transparent'});
		$(this).css({'border-color':'#F00'});
		t.selectTool.style = $(this).attr("name");
		mcqe(8,"SpatialTools",["SetSelectStyle",t.selectTool.style]);
	});
	
	$('input[type="radio"]',t.tabSelect).click(function(){
		var ir = $(this);
		t.selectTool.mode=ir.val();
		mcqe(8,"SpatialTools",["SetSelectMode",t.selectTool.mode]);
		$('input[type="radio"]',t.tabSelect).each(function(){
			$(this).attr("checked",$(this).val()==ir.val());
		});
	}).css('margin','3px 1px -2px 1px');
	
	$('button[name="select_clear"]',t.tabSelect).click(function(){
		t.SelectLib.GetSelection({mode:'clear'});
	});
	
	$("img.icon",t.tabDraw).css({border:'1px solid transparent',margin:'2px 0px -1px 0px'});
	
	$("img.icon",t.tabDraw).hover(
		function(){$(this).css({'border-color':'#00F'});},
		function(){
			var ic = $(this);
			if (ic.attr("name")!=t.drawTool.style) ic.css({'border-color':'transparent'});
			else ic.css({'border-color':'#F00'});
		}
	).tooltip({track:T,delay:0,showURL:F,extraClass:"tooltip"})
	.click(function(e,noswitch){
		var ic = $(this);
		tb = t.map.toolGroups["nav"].itemsNamed[t.tbItems[t.mode]];
		if (tb != t.map.toolGroups["nav"].activeItem) tb.el.click();
		$("img.icon",t.tabDraw).css({'border-color':'transparent'});
		ic.css({'border-color':'#F00'});
		
		mcqe(8,"SpatialTools",["BeforeSetDrawStyle",t.drawTool.style,t.drawTool.mode]);
		
		if (t.drawTool.style != ic.attr("name")) t.drawTool.Clear();
		
		t.drawTool.style = ic.attr("name");
		
		if (!(t.drawTool.style=="polygon"||t.drawTool.style=="line"))
		{
			t.freehand.attr("disabled",T);
			t.drawTool.freehand = F;
		}
		else
		{
			t.freehand.attr("disabled",F);
			t.drawTool.freehand = t.freehand.attr("checked");
		}
		
		if (!(t.drawTool.style=="polygon"||t.drawTool.style=="circle"))
		{
			t.addcut.attr("disabled",T);
			t.drawTool.mode = "add";
		}
		else
		{
			t.addcut.attr("disabled",F);
			t.drawTool.mode = t.addcut.filter(":checked").val();
		}
		
		/** deal with switching between geometry types here **/
		if (!(t.drawTool.style=="polygon"||t.drawTool.style=="circle") && t.DrawLib.curGeom && t.DrawLib.geomType!=t.drawTool.style)
		{
			// If switching from polygons/circles to lines/points, then the user will have to clear their drawing before adding drawing lines/points:
			if (confirm("You are switching to draw a different shape type.\n\nDo you want to clear your current drawing?")) t.DrawLib.Clear();
			else $('img.icon[name="'+t.DrawLib.geomType+'"]',t.tabDraw).click();
		}
		else if (t.DrawLib.curGeom && t.DrawLib.geomType!="polygon" && !noswitch)
		{
			// If switching from line/point to polygon/circle, then assume the user wants to 'cut' geometries:
			t.addcut.filter('[value="cut"]').click();
		}
		mcqe(8,"SpatialTools",["AfterSetDrawStyle",t.drawTool.style,t.drawTool.mode]);
	});
	
	t.freehand.click(function(){
		t.drawTool.freehand = $(this).attr("checked");
		mcqe(8,"SpatialTools",["SetDrawFreehand",(t.drawTool.freehand?"true":"false")]);
	});
	
	t.addcut.click(function(){
		var ir = $(this);
		
		mcqe(8,"SpatialTools",["BeforeSetDrawMode",t.drawTool.style,t.drawTool.mode]);
		
		// If the user is choosing to 'add' polygons/circles, but already has points/lines, then the drawing must be cleared first:
		if (ir.val()=="add" && t.DrawLib.curGeom && t.DrawLib.geomType!="polygon")
		{
			if (confirm("You are switching to draw a different shape type.\n\nDo you want to clear your current drawing?")) t.DrawLib.Clear();
			else t.addcut.filter('[value="cut"]').click();
			t.addcut.each(function(){
				$(this).attr("checked",$(this).val()==ir.val());
			});
			t.drawTool.mode = t.addcut.filter(":checked").val();
		}
		else
		{
			t.drawTool.mode=ir.val();
			t.addcut.each(function(){
				$(this).attr("checked",$(this).val()==ir.val());
			});
		}
		
		mcqe(8,"SpatialTools",["AfterSetDrawMode",t.drawTool.style,t.drawTool.mode]);
		
	}).css('margin','3px 1px -2px 1px');
	
	$('button[name="draw_clear"]',t.tabDraw).click(function(){
		t.DrawLib.Clear();
	});
	
	$('button[name="undo_last_draw"]',t.tabDraw).click(function(){
		t.drawTool.UndoLastDraw();
	});
	
	$('button[name="apply_settings"]',t.tabDraw).click(function(){
		t.DrawLib.GetDrawing({mode:"save",props:{
			"label":$('input[name="drawing_label"]',t.tabDraw).val(),
			"visible":$('input[name="drawing_visible"]',t.tabDraw).attr("checked")
		}});
	});
	
	$('input[name="drawing_label"]',t.tabDraw).change(function(){
		ebtn($('button[name="apply_settings"]'));
	}).keyup(function(){$(this).change();});
	
	$('input[name="drawing_visible"]',t.tabDraw).click(function(){
		ebtn($('button[name="apply_settings"]'));
	});
	
	$('button[name="delete_drawing"]',t.tabDraw).click(function(){
		t.DrawLib.GetDrawing({mode:"delete"});
	});
	
	t.tabDraw.addClass("draw-listener").bind("DrawUpdate",function(){
		t.DrawUpdate();
	}).bind("DrawingCleared",function(){
		t.DrawingCleared();
	}).bind("DrawListUpdate",function(){
		t.DrawListUpdate();
	}).bind("DrawError",function(){
		t.DrawError();
	});
	
	t.UpdateSelection();
	t.DrawListUpdate();
	
	MapChat.KeyProtect($('input[type="text"]',t.content),{focus:true});
	
}


/**
 * Opens the dialog, sets up the select tab's autocomplete list, etc.
 *
 * @param jQuery mode - the tab to open up (i.e., select or draw)
 * @param object opts - options that are passed to the corresponding tool
 */
mcst.Dialog.prototype.activate = function(mode,opts)
{
	var t = this, mode = mode||t.mode;
	
	t.toolOpts = opts||{};
	t.tbItems = opts.tbItems;
	t.mode = mode;
	
	t.displayCanvas.activate();
	
	t.layerInput.removeAttr("disabled");
	t.layerInput.autocomplete(t.layers, {
		minChars: 0,
		selectFirst: false,
		max: 999,
		highlight:false,
		mustMatch: true,
		matchContains: true,
		scrollHeight: 220,
		// some quirk in the autocomplete causes troubles if the default multipleSperator of ', ' is encountered in an item's text:
		multipleSeparator: "||multisep||"
	}).result(function(){
		var li = $(this);
		if (li.val()!="")
		{
			t.selectTool.layer = li.val();
			li.data("lastVal",li.val());
		}
		else li.val(li.data("lastVal"));
	}).blur(function(){
		var li = $(this);
		if ($.inArray(li.val(),t.layers)==-1) li.val(li.data("lastVal"));
	});
	
	switch (mode)
	{
		case "select":
			t.divTabs.tabs("select","#"+t.panelIDs[0]);
			if (!t.selectTool)
			{
				t.selectTool = MapChat.Class("MapChat.SpatialTools.Selector",$.extend(opts||{},{map:t.map,canvas:t.displayCanvas}));
				t.tabSelect.data("tool",t.selectTool);
			}
			$('img.select_'+t.selectTool.style,t.tabSelect).css({'border-color':'#F00'});
			t.curTool = t.selectTool;
			break;
		case "draw":
			t.divTabs.tabs("select","#"+t.panelIDs[1]);
			if (!t.drawTool)
			{
				t.drawTool = MapChat.Class("MapChat.SpatialTools.Draw",$.extend(opts||{},{map:t.map,canvas:t.displayCanvas}));
				t.tabDraw.data("tool",t.drawTool);
			}
			$('img.draw_'+t.drawTool.style,t.tabDraw).css({'border-color':'#F00'});
			t.curTool = t.drawTool;
			break;
	}
	
	mcqe(8,"SpatialTools",["SelectDrawDialogActivated",mode]);
	
	t.content.dialog("open").dialog("moveToTop");
	
	t.curTool.activate();
	
	// Pevent keypress events from moving the map around while this object has focus.
	MapChat.KeyProtect(t.layerInput);
	
};

/**
 * Deactivates the this, and the current select or draw tool.
 */
mcst.Dialog.prototype.deactivate = function()
{
	var t = this;
	$("img.icon",t.content).css({border:'1px solid transparent',margin:'2px 0px -1px 0px'});
	t.curTool.deactivate();
	// Do stuff to deactive the select or draw tool
}

/**
 * Handles what happens when tabs are switched.
 */
mcst.Dialog.prototype.SelectTab = function()
{
	var t = this;
	if (t.lastPanelID)
	{
		if ($("#"+t.lastPanelID,t.divTabs).data("changed"))
		{
			if (t.lastTabID!=N && !confirm("Discard unsaved changes?")) return F;
// 			else t.DiscardChanges($("#"+t.lastPanelID,t.divTabs));
		}
	}
}

/**
 * Handles what happens when tabs are switched
 */
mcst.Dialog.prototype.ShowTab = function()
{
	var t = this,
		curTabID = (t.divTabs?t.divTabs.tabs('option', 'selected'):0),
		panelID = t.panelIDs[curTabID],
		panel = $("#"+panelID,t.divTabs);
	
	// the ui.tabs script currently has a feature that clears any previously set style properties, so we'll have to
	// reset these each time a tab is displayed...just to be sure.
	t.divTabs.children(".ui-tabs-panel").css({height:t.panelHeight});
	
	t.lastPanelID = panelID;
	t.lastTabID = curTabID;
	
	t.curTab = panel.trigger("tabResize").trigger("show");
	
	switch (t.curTab.data("mode"))
	{
		case "select":
			t.mode = "select";
			tb = t.map.toolGroups["nav"].itemsNamed[t.tbItems[t.mode]];
			if (tb != t.map.toolGroups["nav"].activeItem) tb.el.click();
			break;
		case "draw":
			t.mode = "draw";
			tb = t.map.toolGroups["nav"].itemsNamed[t.tbItems[t.mode]];
			if (tb != t.map.toolGroups["nav"].activeItem) tb.el.click();
			break;
	}
	
	mcqe(3,"SpatialTools.Dialog",["ShowTab",(t.curTab?t.curTab.data("name"):'')]);
	
}

/**
 * Updates the contents of the select_history div
 */
mcst.Dialog.prototype.UpdateSelection = function()
{
	var t = this,
		divSH = $(".select_history",t.selectTab),
		deleteIDs = [];
	
	if (divSH.text()=="No features currently selected.") divSH.text("");
	
	// Remove any items that are not part of the selection:
	divSH.children("div.selected_item").each(function(){
		var si = $(this);
		if (!t.SelectLib.features[si.data("layer_name")] || 
			!t.SelectLib.features[si.data("layer_name")][si.data("select_id")])
		{
			si.data("geom").drop();
			if (si.data("simple")) si.data("simple").drop();
			si.remove();
		}
	});
	
	// Add any items that are new to the selection:
	$.each(t.SelectLib.features,function(layer,ft){
		$.each(ft,function(id,fdata){
			if (divSH.children("div.select_id-"+id).length==0)
			{
				var oLI = t.mapLayers.getLayerInfo(layer);
				
				if (!oLI) //the layer must have been deleted, so add it to a list of selection IDs that should be sent back to the server for removal:
				{
					deleteIDs.push(id);
					return;
				}
				
				var newSI = $('<div class="selected_item select_id-'+id+'" style="cursor:pointer;"><p class="select_title" style="float:left">Feature #'+
					fdata[0].si+'</p><p class="select_opts alignright">(<span class="select_remove">remove</span>)</p>'+
					'<p class="select_info"> selected - from <em>'+layer+'</em><br /><ul class="select_attrs hidden" style="margin-left:20px;" /></p>'+
					/*'(<span class="select_attr">attributes</span>|'+*/
					'</div>'),
					siUL = $(".select_attrs",newSI);
				
				switch (fdata[0].layertype)
				{
					case 2:
						$('<li><em>Area: '+fdata[0].area+'</em></li>').appendTo(siUL);
						$('<li><em>Perimeter: '+fdata[0].length+'</em></li>').appendTo(siUL);
					break;
					
					case 1:
						$('<li><em>Length: '+fdata[0].length+'</em></li>').appendTo(siUL);
					break;
					
					case 0:
						$('<li><em>Coordinate: '+fdata[0].extent[0][0]+','+fdata[0].extent[0][1]+'</em></li>').appendTo(siUL);
					break;
				}
				
				$.each(fdata[1],function(n,v){
					if (!oLI.field_aliases) $('<li>'+n+': '+v+'</li>').appendTo(siUL);
					else if (oLI.field_aliases[n]) $('<li>'+oLI.field_aliases[n]+': '+v+'</li>').appendTo(siUL);
				});
				
				divSH.append(newSI);
				newSI.data("layer_name",layer).data("select_id",id).data("shpinfo",fdata[0]).data("attr",fdata[1])
					.data("simple",(fdata[0].simple?t.displayCanvas.addGeom(fdata[0].simple,t.simpleFill):F))
					.data("geom",t.displayCanvas.addGeom(fdata[0].shape,(fdata[0].type!=1?t.displayFill:t.displayOpts)))
					.hover(function(){
						var si = $(this);
						si.css('background-color','#CCF');
						si.data("geom").setStyle((si.data("shpinfo").type!=1?t.hoverFill:t.hoverOpts));
						if (si.data("simple")) si.data("simple").setStyle({display:'visible'});
					},function(){
						var si = $(this);
						si.css('background-color','#FFF');
						si.data("geom").setStyle((si.data("shpinfo").type!=1?t.displayFill:t.displayOpts));
						if (si.data("simple")) si.data("simple").setStyle({display:'none'});
					});
				$(newSI.data("geom").o).mouseenter(function(){
						newSI.css('background-color','#CCF');
						newSI.data("geom").setStyle((newSI.data("shpinfo").type!=1?t.hoverFill:t.hoverOpts));
						if (newSI.data("simple")) newSI.data("simple").setStyle({display:'visible'});
						divSH.scrollTo(newSI,300);
					}).mouseleave(function(){
						newSI.css('background-color','#FFF');
						newSI.data("geom").setStyle((newSI.data("shpinfo").type!=1?t.displayFill:t.displayOpts));
						if (newSI.data("simple")) newSI.data("simple").setStyle({display:'none'});
					});
				$(".select_remove",newSI).click(function(){
					t.SelectLib.GetSelection({mode:"remove",style:"by_id",ids:[id]});
				});
				
				$(".select_info,.select_title",newSI).click(function(){
					siUL.toggle();
					mcqe(8,"SpatialTools",["ToggleSelectedItemAttributes",newSI.data("select_id")]);
				});
			}
		});
	});
	
	if (deleteIDs.length>0)
		t.SelectLib.GetSelection({mode:"remove",style:"by_id",ids:deleteIDs});
	
	if (t.SelectLib.count==0)
		divSH.text("No features currently selected.");
	
}

/**
 * Updates the contents of the drawings list
 */
mcst.Dialog.prototype.DrawListUpdate = function()
{
	var t = this;
	t.drawings = [];
	t.drawingIDs = {};
	
	if (t.DrawLib.drawList) $.each(t.DrawLib.drawList,function(i,d){
		t.drawings.push(d.displayText);
		t.drawingIDs[d.displayText] = d.id;
	});
	
	t.drawings.sort();
	if ($.inArray(t.drawSelect.val(),t.drawings)==-1);
	
	t.drawSelect.unautocomplete();
	t.drawSelect.autocomplete(t.drawings, {
		minChars: 0,
		selectFirst: false,
		max: 999,
		highlight:false,
		mustMatch: true,
		matchContains: true,
		scrollHeight: 220,
		// some quirk in the autocomplete causes troubles if the default multipleSperator of ', ' is encountered in an item's text:
		multipleSeparator: "||multisep||"
	}).result(function(){
		var li = $(this);
		if (li.val()!="") t.DrawLib.GetDrawing({mode:"open",id:t.drawingIDs[li.val()]||-1});
		else t.DrawingCleared();
	}).blur(function(){
		var li = $(this);
		
		if (li.val()=="")
		{
			// To accommodate limitations of the autocomplete:
			setTimeout(function(){
				if (li.val()!="") return;
				else t.DrawingCleared();
			},333);
		}
	});
}

mcst.Dialog.prototype.DrawUpdate = function()
{
	var t = this, 
		dopts = $('div.drawing_options',t.tabDraw),
		curDrawing = t.DrawLib.drawList[t.DrawLib.geomID];
	
	if (t.displayGeom) t.displayGeom.drop();
	switch (t.DrawLib.geomType)
	{
		case "polygon":
			t.displayGeom = t.displayCanvas.addGeom(t.DrawLib.curGeom,t.drawFill);
			dopts.children('p.drawing_measurement')
				.html('<br /><em>Area: '+curDrawing.area+'</em><br /><em>Perimeter: '+curDrawing.len+'</em>');
			if (t.drawTool.style!="polygon" && t.drawTool.style!="circle")
			{
				$("img.draw_polygon",t.tabDraw).click();
			}
		break;
		
		case "line":
			t.displayGeom = t.displayCanvas.addGeom(t.DrawLib.curGeom,t.drawOpts);
			dopts.children('p.drawing_measurement')
				.html('<br /><em>Length '+curDrawing.len+'</em>');
			if (t.drawTool.style!="line" && !(t.drawTool.style!="point" && t.drawTool.mode=="cut"))
			{
				$("img.draw_line",t.tabDraw).trigger("click",[T]);
			}
		break;
		
		case "point":
			t.displayGeom = t.displayCanvas.addGeom(t.DrawLib.curGeom,t.pointFill);
			dopts.children('p.drawing_measurement').html('');
			if (t.drawTool.style!="point" && !(t.drawTool.style!="line" && t.drawTool.mode=="cut"))
			{
				$("img.draw_point",t.tabDraw).trigger("click",[T]);
			}
		break;
	}
	
	t.drawSelect.val(curDrawing?curDrawing.displayText:"");
	$('span.no_drawing',t.tabDraw).hide();
	dopts.show();
	$('input[name="drawing_label"]',dopts).val(curDrawing?(curDrawing.label||""):"");
	
	(t.DrawLib.geomHistory.length>0?ebtn:dbtn)($('button[name="undo_last_draw"]',dopts));
	
	$('input[name="drawing_visible"]',dopts).attr("checked",curDrawing?(curDrawing.visible||F):F);
	dbtn($('button[name="apply_settings"]',dopts));
	
}

mcst.Dialog.prototype.DrawingCleared = function()
{
	var t = this, dopts = $('div.drawing_options',t.tabDraw);
	if (t.displayGeom) t.displayGeom.drop();
	dopts.hide();
	t.drawSelect.val("");
	$('span.no_drawing',t.tabDraw).show();
	$('input[name="drawing_label"]',dopts).val("");
	dbtn($('button[name="undo_last_draw"]',dopts));
	$('input[name="drawing_visible"]',dopts).attr("checked",F);
}

mcst.Dialog.prototype.DrawError = function()
{
	var t = this;
	t.drawTool.Clear();
	t.DrawLib.GetDrawing({mode:"open",id:t.drawingIDs[t.layerInput.val()]||-1});
}