/**
 * The Draw tool - used by the SpatialTools.Dialog
 *
 * @class MapChat.SpatialTools.Draw
 *
 * Depends:
 *	kaMap.js
 *	MapChat.js
 *	MapChat/application.js
 */

// Get an alias for the the MapChat.SpatialTools namespace (to make the code shorter).
var mcst = MapChat.UseNS("MapChat.SpatialTools"),

	// Aliases for the disable/enable button functions and the QueueEvent function:
	dbtn = MapChat.DisableButton, ebtn = MapChat.EnableButton, mcqe = MapChat.QueueEvent;

mcst.Draw = function(options)
{
	var t = this,
		name = (t.name = "MapChat.SpatialTools.Draw"),
		options = (t.options = options),
		map = (t.map = options.map),
		cursor = (t.cursor = "crosshair"),
		style = (t.style = "polygon"),
		mode = (t.mode = "add"),
		freehand = (t.freehand = options.freehand||F),
		drawing = (t.drawing = F),
		bMouseDown = (t.bMouseDown = false),
		lastPt = (t.lastPt = N),
		listener = (t.listener = $('<div><div class="draw-listener" /></div>').children()),
		
		curCoords = (t.curCoords = ""),
		
		startx = (t.startx = N),
		starty = (t.starty = N),
		endx = (t.endx = N),
		endy = (t.endy = N),
		
		drawOpts = (t.drawOpts = options.drawOpts||{stroke:{stroke:"#0F0",'stroke-width':2,'stroke-opacity':0.60},r:0}),
		drawFill = (t.drawFill = $.extend(options.drawFill||{fill:{fill:"#0F0",'fill-opacity':0.35}},t.drawOpts)),
		pointFill = (t.pointFill = $.extend($.extend(options.pointFill||{fill:{fill:"#0F0",'fill-opacity':0.60}},t.drawOpts),{r:5})),
		closeOpts = (t.closeOpts  = options.closeOpts||{stroke:{stroke:"#C73",'stroke-width':2,'stroke-opacity':0.80}}),
		
		DrawLib = (t.DrawLib = MapChat.MapSingleton("MapChat.DrawLib",map)),
		
		displayCanvas = (t.displayCanvas = options.canvas||MapChat.Class("MapChat.Canvas",{map:t.map,zIndex:500})),
		closeGeom = (t.closeGeom = N),
		curGeom = (t.curGeom = N);
	
	kaTool.apply( t, [map] );
	
	MapChat.AddContext(t.map,t.listener.parent());
	
	t.listener.bind("DrawUpdate",function(){t.Clear();});
	
	/** get all the prototype functions from the kaTool object */
	for (var p in kaTool.prototype) {
		if (!mcst.Draw.prototype[p])
			mcst.Draw.prototype[p] = kaTool.prototype[p];
	}
	
	return t;
}

/**
 * MapChat.SpatialTools.Draw.onmousedown( e )
 *
 * called when a mouse button is pressed over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
mcst.Draw.prototype.onmousedown = function(e)
{
	var t = this;
	
	e = $.event.fix(e?e:event);
	
	// Make sure the map has focus:
	$(window).focus();
	
	if (e.button==2||e.metaKey||e.shiftKey) return t.cancelEvent(e);
	
	else
	{
		if (t.map.isIE4) document.onkeydown = kaTool_redirect_onkeypress;
		document.onkeypress = kaTool_redirect_onkeypress;
		
		t.bMouseDown=T;
		
		if (!t.drawing && t.style!="point")
		{
			var x = e.pageX, y = e.pageY, curPt = t.adjustPixPosition( x,y );
			
			t.startx=t.endx=-curPt[0];
			t.starty=t.endy=-curPt[1];
			
			if (t.curGeom) t.curGeom.drop();
			if (t.closeGeom) t.closeGeom.drop();
			
			switch (t.style)
			{
				case "circle":
					t.curGeom = t.displayCanvas.newGeom("point",t.drawFill);
					t.curGeom.addPoint(t.map.pixToGeo( -t.startx, -t.starty ));
				break;
				
				case "line":
					t.curGeom = t.displayCanvas.newGeom("path",t.drawOpts);
				break;
				
				case "polygon":
					t.curCoords = "[]";
					t.curGeom = t.displayCanvas.newGeom("path",t.drawFill,T);
					t.closeGeom = t.displayCanvas.newGeom("path",t.closeOpts);
				break;
			}
			
			if (t.style!="point") t.drawing = T;
		}
		
		e.cancelBubble = T;
		e.returnValue = F;
		if (e.stopPropogation) e.stopPropogation();
		if (e.preventDefault) e.preventDefault();
		return F;
	}
};


/**
 * MapChat.SpatialTools.Draw.onmouseup( e )
 *
 * called when a mouse button is released over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
mcst.Draw.prototype.onmouseup = function(e)
{
	var t = this;
	e = $.event.fix(e?e:event);
	
	$(window).focus();
	
	t.bMouseDown = F;
	
	if (e.button==2||e.metaKey||e.shiftKey) return t.cancelEvent(e);
	
	// Only do stuff on mouseup for non-point features:
	else if (t.drawing && t.style!="point")
	{
		
		if (t.map.isIE4) document.onkeydown = kaTool_redirect_onkeypress;
			document.onkeypress = kaTool_redirect_onkeypress;
		
		// If we are currently drawing, then update the shape (or finish with a circle)
		if (t.drawing)
		{
			// Generally, there should only be one geometry in the curGeom object, but just in case, we'll work on the 'last' one:
			var numGeoms = t.curGeom.xy.length,
				numPoints = (numGeoms>0?t.curGeom.xy[numGeoms-1].length:0),
				x = e.pageX, y = e.pageY, curPt = t.adjustPixPosition( e.pageX, e.pageY ),
				geoPt = MapChat.Round(t.map.pixToGeo(curPt[0],curPt[1]));
			
			t.endx=-curPt[0];
			t.endy=-curPt[1];
			
			switch (t.style)
			{
				case "circle":
					if (t.endx==t.startx && t.endy==t.starty)
					{
						// The user didn't move the mouse while drawing a circle
						t.drawing = F;
						t.curGeom.drop();
						t.curGeom = N;
					}
					else
					{
						// Submit the circle to the drawlib
						$.blockUI();
						
						var c = t.curGeom.xy[t.curGeom.xy.length-1];
						
						// Add the radius as a third value in the circle's point coordinate:
						t.curGeom.xy[t.curGeom.xy.length-1].push(Math.sqrt(Math.pow(c[0]-geoPt[0],2)+Math.pow(c[1]-geoPt[1],2)));
						t.DrawLib.GetDrawing({mode:t.mode,style:t.style,coords:t.curGeom.xy});
						t.drawing = F;
					}
				break;
				
				case "line":
				case "polygon":
					// Ignore duplicate points.
					if (!(numPoints>0 && geoPt[0] == t.curGeom.lastPoint()[0] && geoPt[1] == t.curGeom.lastPoint()[1]))
						t.curGeom.addPoint(geoPt);
					
					var lp = t.curGeom.lastPoint();
					mcqe(8,"SpatialTools",["DrawNewVertex",lp[0],lp[1]]);
					
					if (!t.freehand)
					{
						if (t.style=="polygon")
						{
							// Validate the polygon...if it's invalid, drop the point.
							if (numPoints>1 && !t.displayCanvas.validateGeom(t.curGeom,T))
							{
								t.curGeom.dropPoint();
								mcqe(8,"SpatialTools",["InvalidPolygonThwarted"]);
							}
						}
					}
					else if (t.style=="polygon")
					{
						// Validate the polygon...if it's valid, save a copy of the coordinates.  If not, restore it to the previous copy.
						if (numPoints<3 || t.displayCanvas.validateGeom(t.curGeom,T))
						{
							mcqe(8,"SpatialTools",["FreeHandPolySectionAdded"]);
							t.curCoords = $.toJSON(t.curGeom.xy);
						}
						else
						{
							t.curGeom.xy = $.parseJSON(t.curCoords);
							
							mcqe(8,"SpatialTools",["InvalidFreehandPolygonSectionThwarted"]);
							
							// re-add the current point, if not duplicate and draw:
							if (!(t.curGeom.lastPoint() && geoPt[0] == t.curGeom.lastPoint()[0] && geoPt[1] == t.curGeom.lastPoint()[1]))
							{
								t.curGeom.addPoint(geoPt);
								t.curGeom.draw();
								t.curGeom.dropPoint();
							}
							else t.curGeom.draw();
							
							if (t.closeGeom && t.curGeom.lastPoint())
							{
								t.closeGeom.xy[0] = [geoPt,t.curGeom.lastPoint()];
								t.closeGeom.draw();
							}
							else if (t.closeGeom) t.closeGeom.drop();
						}
					}
				break;
			}
		}
	}
	e.cancelBubble = T;
	e.returnValue = F;
	if (e.stopPropogation) e.stopPropogation();
	if (e.preventDefault) e.preventDefault();
	return F;
};


/**
 * MapChat.SpatialTools.Draw.onmousemove( e )
 *
 * called when the mouse moves over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
mcst.Draw.prototype.onmousemove = function(e)
{
	var t = this;
	
	// Only add points for line/polygon shapes - points get
	if (t.drawing)
	{
		e = $.event.fix(e?e:event);
		
		var numGeoms = t.curGeom.xy.length,
			numPoints = (numGeoms>0?t.curGeom.xy[numGeoms-1].length:0),
			x = e.pageX, y = e.pageY, curPt = t.adjustPixPosition( e.pageX, e.pageY ),
			geoPt = MapChat.Round(t.map.pixToGeo(curPt[0],curPt[1]));
		
		t.lastPt = geoPt;
		
		t.endx=-curPt[0];
		t.endy=-curPt[1];
		
		switch (t.style)
		{
			case "circle":
				t.curGeom.setStyle({r:(Math.sqrt(Math.pow(t.endx-t.startx,2)+Math.pow(t.endy-t.starty,2)))});
				t.curGeom.draw();
			break;
			
			case "line":
			case "polygon":
				// Ignore points equal to the last one (e.g., on a double-click event):
				if (numPoints>0 && geoPt[0] == t.curGeom.lastPoint()[0] && geoPt[1] == t.curGeom.lastPoint()[1]) return;
				
				// Add the new point and draw the geometry...
				t.curGeom.addPoint(geoPt);
				if (numGeoms>0 && t.curGeom.xy[numGeoms-1].length>1) t.curGeom.draw();
				t.curGeom.draw();
				
				// Don't keep the last point if we're not drawing freehand, or if the mouse is not 'down':
				if (!t.freehand||!t.bMouseDown) t.curGeom.dropPoint();
				
				if (t.style=="polygon" && numGeoms>0 && t.curGeom.xy[numGeoms-1].length>0)
				{
					t.closeGeom.xy[0] = [t.curGeom.xy[numGeoms-1][0],geoPt];
					t.closeGeom.draw();
				}
				else if (t.style=="polygon")
				{
					t.closeGeom.xy = [];
					t.closeGeom.draw();
				}
			break;
		}
	}
	e.cancelBubble = T;
	e.returnValue = F;
	if (e.stopPropogation) e.stopPropogation();
	if (e.preventDefault) e.preventDefault();
	return F;
};


/**
 * MapChat.SpatialTools.Draw.ondblclick( e )
 *
 * called when a double-click event is triggered on theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 * skip - boolean, if true a new point isn't added (e.g., if this was called from a keypress event).
 */
mcst.Draw.prototype.ondblclick = function(e,skip)
{
	var t = this;
	
	if (t.drawing)
	{
		e = $.event.fix(e?e:event);
		
		var numGeoms = t.curGeom.xy.length,
			numPoints = (numGeoms>0?t.curGeom.xy[numGeoms-1].length:0);
		
		if (!skip && t.drawing && t.curGeom && t.lastPt 
			&& t.lastPt[0]!=t.curGeom.lastPoint()[0] && t.lastPt[1]!=t.curGeom.lastPoint()[1]) t.curGeom.addPoint(t.lastPt);
		
		if (t.style=="line"||t.style=="polygon")
		{
			
			// If it's a polygon, make sure it's valid before calling this done.
			if (t.style=="polygon")
			{
				if (t.displayCanvas.validateGeom(t.curGeom)) t.drawing = F;
				else if (t.curGeom.xy[numGeoms-1].length>numPoints)
				{
					t.curGeom.dropPoint();
					mcqe(8,"SpatialTools",["InvalidPolygonThwarted"]);
				}
			}
			else t.drawing = F;
			
			// If we're done drawing, make sure there's no closing line geometry:
			if (!t.drawing && t.closeGeom) t.closeGeom.drop();
		}
		
		if (!t.drawing)
		{
			// If we're done, send the shape to the drawlib
			$.blockUI();
			t.DrawLib.GetDrawing({mode:t.mode,style:t.style,coords:t.curGeom.xy});
		}
	}
	else if (t.style=="point")
	{
		if (!t.curGeom) t.curGeom = t.displayCanvas.newGeom("point",t.pointFill);
		
		e = $.event.fix(e?e:event);
		
		var x = e.pageX, y = e.pageY, curPt = t.adjustPixPosition( e.pageX, e.pageY ),
			geoPt = MapChat.Round(t.map.pixToGeo(curPt[0],curPt[1]));
		
		t.curGeom.addPoint(geoPt);
		t.curGeom.draw();
		
		mcqe(8,"SpatialTools",["DrawNewPoint",geoPt[0],geoPt[1]]);
		
		// Points are always finished - send them on:
		t.DrawLib.GetDrawing({mode:t.mode,style:t.style,coords:t.curGeom.xy});
	}
	e.cancelBubble = T;
	e.returnValue = F;
	if (e.stopPropogation) e.stopPropogation();
	if (e.preventDefault) e.preventDefault();
	return F;
};

/**
 * Clears the current drawing.
 */
mcst.Draw.prototype.Clear = function()
{
	var t = this;
	mcqe(8,"SpatialTools",["DrawToolClear"]);
	t.drawing = F;
	t.bMouseDown = F;
	if (t.curGeom) t.curGeom.drop();
	t.curGeom = N;
	if (t.closeGeom) t.closeGeom.drop();
	t.closeGeom = N;
	t.curCoords = "[]";
	$.unblockUI();
}


/**
 * Allow users to press Esc (cancel current drawing), Backspace (undo last coordinate), or Enter (alias for double-click) keys.
 */
mcst.Draw.prototype.onkeypress = function(e)
{
	var t = this;
	e = $.event.fix(e?e:event);
	switch (e.keyCode)
	{
		case 27:
			t.Clear();
		break;
		
		case 13:
			if (t.curGeom.xy)
			{
				var numGeoms = t.curGeom.xy.length,
					numPoints = (numGeoms>0?t.curGeom.xy[numGeoms-1].length:0);
				if (numPoints>0 && (t.lastPt[0] != t.curGeom.lastPoint()[0] || t.lastPt[1] != t.curGeom.lastPoint()[1]))
					t.curGeom.addPoint(t.lastPt);
				t.ondblclick(e,T);
			}
		break;
		
		case 8:
		case 46:
			t.UndoPoint();
		break;
	}
	e.cancelBubble = T;
	e.returnValue = F;
	if (e.stopPropogation) e.stopPropogation();
	if (e.preventDefault) e.preventDefault();
}

mcst.Draw.prototype.UndoPoint = function()
{
	var t = this;
	if (t.drawing)
	{
		var lp = t.curGeom.lastPoint();
		mcqe(8,"SpatialTools",["UndoLastPoint",lp[0],lp[1]]);
		if (t.curGeom)
		{
			t.curGeom.dropPoint();
			t.curCoords = $.toJSON(t.curGeom.xy);
			t.curGeom.draw();
		}
		if (t.closeGeom && t.curGeom.xy.length>0 && t.curGeom.xy[t.curGeom.xy.length-1].length>1)
		{
			t.closeGeom.xy[0] = [t.curGeom.xy[t.curGeom.xy.length-1][0],t.curGeom.lastPoint()];
			t.closeGeom.draw();
		}
		else if (t.closeGeom)
		{
			t.closeGeom.xy = [];
			t.closeGeom.draw();
		}
	}
}

mcst.Draw.prototype.UndoLastDraw = function()
{
	var t = this;
	mcqe(8,"SpatialTools",["UndoLastDraw"]);
	if (t.DrawLib.geomHistory.length>0)
	{
		var undoGeom = t.displayCanvas.addGeom(t.DrawLib.geomHistory.pop(),{display:"none"});
		t.DrawLib.GetDrawing({mode:"save",style:t.DrawLib.geomType,coords:undoGeom.xy});
		undoGeom.drop();
	}
}