/**
 * The Selector tool - used by the SpatialTools.Dialog
 *
 * @class MapChat.SpatialTools.Selector
 *
 * Depends:
 *	kaMap.js
 *	MapChat.js
 *	MapChat/application.js
 */

// Get an alias for the the MapChat.SpatialTools namespace (to make the code shorter).
var mcst = MapChat.UseNS("MapChat.SpatialTools"),

	// Aliases for the disable/enable button functions and the QueueEvent function:
	dbtn = MapChat.DisableButton, ebtn = MapChat.EnableButton, mcqe = MapChat.QueueEvent;

mcst.Selector = function(options)
{
	var t = this,
		name = (t.name = "MapChat.SpatialTools.Selector"),
		options = (t.options = options),
		map = (t.map = options.map),
		cursor = (t.cursor = "crosshair"),
		startx = (t.startx = N),
		starty = (t.starty = N),
		endx = (t.endx = N),
		endy = (t.endy = N),
		bMouseDown = (t.bMouseDown = F),
		style = (t.style = options.style||"rectangle"),
		mode = (t.mode = options.mode||"add"),
		
		distDenom = (t.distDenom = options.distDenom||1),
		dispUnits = (t.dispUnits = options.dispUnits||""),
		
		SelectLib = (t.SelectLib = MapChat.MapSingleton("MapChat.SelectLib",map)),
		
		displayCanvas = (t.displayCanvas = options.canvas||MapChat.Class("MapChat.Canvas",{map:t.map})),
		label = (t.label = $('<div class="tooltip" />').appendTo(document.body)),
		div = (t.div = $("<div>&nbsp;</div>").css({
			position:"absolute",
			top:0,
			left:0,
			width:1,
			height:1,
			zIndex:501,
			visibility:"hidden",
			border:"2px solid cyan",
			"background-color": "#FFF",
			opacity:.35,
			"font-size":1,
			"line-height":0
		})),
		circleGeom = (t.circleGeom = displayCanvas.newGeom("point",{
			stroke:{stroke:"#0FF",'stroke-width':2,'stroke-opacity':0.35},
			fill:{fill:"#FFF",'fill-opacity':0.35},
			display:"none",
			r:1
		}));
	
	kaTool.apply( t, [map] );
	
	$(map.theInsideLayer).append(div);
	
	/** get all the prototype functions from the kaTool object */
	for (var p in kaTool.prototype) {
		if (!mcst.Selector.prototype[p])
			mcst.Selector.prototype[p] = kaTool.prototype[p];
	}
	
	return t;
}

/*
 * draw a box representing the query region.
 *
 * MapChat.SpatialTools.Selector maintains the query region in four variables.  The variables are
 * assumed to be in pixel coordinates and are used to position the box.  If
 * any of the coordinates are null, clear the query box.
 */
mcst.Selector.prototype.drawSelectBox = function() {
	var t = this;
	if (t.startx == null || t.starty == null ||
		t.endx == null || t.endy == null ) {
		t.div.css({
			visibility:'hidden',
			top:0,
			left:0,
			width:1,
			height:1
		});
		t.circleGeom.setStyle({display:"none"});
		t.label.hide().html("");
		return;
	}
	else
	{
		switch (t.style)
		{
			case "rectangle":
				t.div.css({visibility:'visible'});
				if (t.endx < t.startx) {
					t.div.css({left:(t.endx - t.map.xOrigin),width:(t.startx - t.endx)});
				} else {
					t.div.css({left:(t.startx - t.map.xOrigin),width:(t.endx - t.startx)});
				}
				
				if (t.endy < t.starty) {
					t.div.css({top:(t.endy - t.map.yOrigin),height:(t.starty - t.endy)});
				} else {
					t.div.css({top:(t.starty - t.map.yOrigin),height:(t.endy - t.starty)});
				}
			break;
				
			case "circle":
				t.circleGeom.setStyle({display:"visible",r:(Math.sqrt(Math.pow(t.endx-t.startx,2)+Math.pow(t.endy-t.starty,2)))});
				t.circleGeom.draw();
			break;
		}
	}
};

/**
 * MapChat.SpatialTools.Selector.onmouseout( e )
 *
 * called when the mouse leaves theInsideLayer.  Terminate the query
 *
 * e - object, the event object or null (in ie)
 */
mcst.Selector.prototype.onmouseout = function(e) {
	e = $.event.fix(e?e:event);
	var t = this;
	if (e.target.id == t.map.domObj.id) {
		t.bMouseDown = F;
		t.startx = t.endx = t.starty = t.endy = null;
		t.drawSelectBox();
		return kaTool.prototype.onmouseout.apply(t, [e]);
	}
};

/**
 * MapChat.SpatialTools.Selector.onmousemove( e )
 *
 * called when the mouse moves over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
mcst.Selector.prototype.onmousemove = function(e) {
	e = $.event.fix(e?e:event);
	var t = this;
	if (!t.bMouseDown)
	{
		return t.cancelEvent(e);
	}
	var x = e.pageX, y = e.pageY, aPixPos = t.adjustPixPosition( x, y );
	t.endx=-aPixPos[0];
	t.endy=-aPixPos[1];
	t.drawSelectBox();
	if (t.style=="circle" && t.bMouseDown)
	{
		var rad = t.circleGeom.opts.r*t.map.cellSize/t.distDenom;
		
		// make nicer looking numbers:
		var rad = (rad<0?rad.toPrecision(6):(rad<1000?rad.toPrecision(4):Math.round(rad)));
		
		t.label.html("Radius: "+rad+t.dispUnits).css({left:e.pageX+15,top:e.pageY+15}).show();
	}
	return F;
};

/**
 * MapChat.SpatialTools.Selector.onmousedown( e )
 *
 * called when a mouse button is pressed over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
mcst.Selector.prototype.onmousedown = function(e)
{
	var t = this;
	e = $.event.fix(e?e:event);
	
	$(window).focus();
	
	if (e.button==2||e.metaKey||e.shiftKey) return t.cancelEvent(e);
	
	else
	{
		if (t.map.isIE4) document.onkeydown = kaTool_redirect_onkeypress;
		document.onkeypress = kaTool_redirect_onkeypress;
		
		t.bMouseDown=T;
		var x = e.pageX, y = e.pageY, aPixPos = t.adjustPixPosition( x,y );
		t.startx=t.endx=-aPixPos[0];
		t.starty=t.endy=-aPixPos[1];
		
		if (t.style=="circle")
		{
			t.circleGeom.dropPoint();
			t.circleGeom.addPoint(t.map.pixToGeo( -t.startx, -t.starty ));
		}
		
		t.drawSelectBox();
		
		e.cancelBubble = T;
		e.returnValue = F;
		if (e.stopPropogation) e.stopPropogation();
		if (e.preventDefault) e.preventDefault();
		return F;
	}
};

/**
 * MapChat.SpatialTools.Selector.onmouseup( e )
 *
 * called when a mouse button is clicked over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
mcst.Selector.prototype.onmouseup = function(e) {
	e = $.event.fix(e?e:event);
	$(window).focus();
	var t = this, start = t.map.pixToGeo( -t.startx, -t.starty ), coords = start,
		x = e.pageX, y = e.pageY, aPixPos = t.adjustPixPosition( x, y );
	
	if (!t.bMouseDown) return t.cancelEvent(e);;
	
	t.endx=-aPixPos[0];
	t.endy=-aPixPos[1];
	
	t.bMouseDown = F;
	
	if (t.startx!=t.endx&&t.starty!=t.endy)
	{
		coords = start.concat(t.map.pixToGeo( -t.endx, -t.endy ));
		if (coords[2] < coords[0])
		{
			//minx gt maxx then invert values
			var minx = coords[2], maxx = coords[0];
			coords[0] = minx;
			coords[2] = maxx;
		}
		if (coords[1] < coords[3])
		{
			//miny gt maxy then invert values
			var miny = coords[1], maxy = coords[3];
			coords[3] = miny;
			coords[1] = maxy;
		}
	}
	
	t.label.hide().html("");
	
	t.startx = t.endx = t.starty = t.endy = null;
	t.drawSelectBox();
	
	if(coords[2] && coords[0] && coords[3] && coords[1])
	{
		//rubber select
		var selOpts = {mode:t.mode,style:t.style,layer:t.layer,coords:coords};
	}
	else if (coords[0] && coords[1])
	{
		//single click
		var selOpts = {mode:t.mode,style:"point",layer:t.layer,coords:[coords[0],coords[1]]};
	}
	
	t.SelectLib.GetSelection(selOpts);
	
	return F;
};
