/**
 * A tool for identifying feature attributes or raster cell values
 *
 * @class MapChat.mcTools.Identify
 *
 * Depends:
 *	kamap/kaMap.js
 *	MapChat.js
 *	MapChat/application.js
 *	jquery/ui/ui.dialog.js
 *	jquery/jquery.autocomplete.js
 */

$.extend(MapChat.mcTools,{
	Identify: function(options)
	{
		var t = this,
			options = (t.options = options),
			map = (t.map = options.map),
			cursor = (t.cursor = "help"),
			name = (t.name = "Identify"),
			geo = (t.geo = null),
			content = (t.content = $('<div class="autoscroll" style="font-size:.9em;line-height:16px;width:auto;" />')),
			layerInput = (t.layerInput = $('<input class="dropDown" type="text" style="width:250px;" />')),
			ddButton = (t.ddButton = $('<div class="dropDownButton" />')),
			results = (t.results = $('<div style="clear:left;" />')),
			resultsArray = (t.resultsArray = []),
			displayCanvas = (t.displayCanvas = MapChat.Class("MapChat.Canvas",{map:options.map})),
			displayGeom = (t.displayGeom = null),
			mapLayers = (t.mapLayers = MapChat.getMapClassObject("MapChat.MapLayers",map)),
			tempLayers = [];
		
		kaTool.apply( t, [map] );
		
		content.dialog({title:"Identify Features",width:300,height:200,modal:false,autoOpen:false,closeOnEscape:false,
			open:function()
			{
				layerInput.focus(function(e){
					if (layerInput.val().length>0 && layerInput.val()!=layerInput.data("curVal"))
					{
						layerInput.data("curVal",layerInput.val());
						displayCanvas.clear();
						results.html("Click on the map to identify features.");
					}
					else if (layerInput.val()!=layerInput.data("curVal"))
					{
						results.html("");
					}
				});
				MapChat.QueueEvent(3,"Identify",["Open"]);
			},
			close:function()
			{
				// Just to make sure things are cleaned up if a user closes the dialog while the
				// autocomplete list is open:
				layerInput.acHide();
				displayCanvas.deactivate();
				MapChat.QueueEvent(3,"Identify",["Close"]);
			},
			resize:function()
			{
				var dlg = content.parents(".ui-dialog:first");
				content.css({height:($(".ui-resizable-s", dlg).offset().topF|
					$(".ui-dialog-buttonpane", dlg).offset().top)-content.offset().top});
			}
		});
		
		//content.data("dialog").uiDialog.data("draggable").setData("containment","document");
		
		// Build a straight list of map layers with visibility determined by their on/off states:
		$.each(mapLayers.getLayerNames(),function(i,v){
			tempLayers.push(v);
		});
		
		t.layers = tempLayers.sort();
		
		content.append("Select a layer to query:<br />").append(layerInput)
			.append(ddButton).append("<br />").append(results);
		
		
		// The easiest way I could find to force the autocomplete pugin into showing the list:
		ddButton.click(function(){
			// Clear the value so we can see the whole list:
			//layerInput.val("");
			// Open the autocomplete list (hide it first in case it was already open with some results):
			layerInput.trigger("ddToggle");
		});
		
		/** get all the prototype functions from the kaTool object */
		for (var p in kaTool.prototype)
		{
			if (!MapChat.mcTools.Identify.prototype[p])
				MapChat.mcTools.Identify.prototype[p]= kaTool.prototype[p];
		}
		
		MapChat.AddEventCallback(mapLayers.getMapItem(),
			"LayerListUpdate",function(mf){
				t.layerInput.unautocomplete();
				var tempLayers = [];
				$.each(t.mapLayers.getLayerNames(),function(i,v){
					tempLayers.push(v);
				});
				t.layers = tempLayers.sort();
				t.layerInput.autocomplete(t.layers, {
					minChars: 0,
					selectFirst: false,
					max: 999,
					highlight:false,
					mustMatch: true,
					matchContains: true,
					scrollHeight: 220,
					// some quirk in the autocomplete causes troubles if the default multipleSperator of ', ' is encountered in an item's text:
					multipleSeparator: "||multisep||"
				});
			}
		);
		
		return t;
	}
});


/**
 * activate this tool.  Activating the tool causes any existing tools to be
 * deactivated.
 *
 * @param boolean quiet - if true, the dialog is not displayed.
 */
MapChat.mcTools.Identify.prototype.activate = function(quiet)
{
	var t = this;
	
	t.displayCanvas.activate();
	
	if (!quiet)
	{
		t.content.dialog("open").dialog("moveToTop");
		var dlg = t.content.parents(".ui-dialog:first");
		t.content.css({height:($(".ui-resizable-s", dlg).offset().top||
			$(".ui-dialog-buttonpane", dlg).offset().top)-t.content.offset().top});
	}
	t.layerInput.removeAttr("disabled");
	t.layerInput.autocomplete(t.layers, {
		minChars: 0,
		selectFirst: false,
		max: 999,
		highlight:false,
		mustMatch: true,
		matchContains: true,
		scrollHeight: 220,
		// some quirk in the autocomplete causes troubles if the default multipleSperator of ', ' is encountered in an item's text:
		multipleSeparator: "||multisep||"
	});
	
	// Pevent keypress events from moving the map around while this object has focus.
	MapChat.KeyProtect(t.layerInput);
	
	kaTool.prototype.activate.apply(t);
};

/**
 * deactivate this tool.
 */
MapChat.mcTools.Identify.prototype.deactivate = function()
{
		var t = this;
		t.layerInput.attr("disabled",T);
		t.layerInput.unautocomplete();
		kaTool.prototype.deactivate.apply(t);
};

/**
 * MapChat.mcTools.Identify.onmouseup( e )
 *
 * called when a mouse button is clicked over theInsideLayer.
 */
MapChat.mcTools.Identify.prototype.onmouseup = function(e) {
	e = $.event.fix(e?e:event);
	var t = this, layer = t.layerInput.val();
	
	if (e.button==2||e.metaKey||e.shiftKey)
	{
		return t.cancelEvent(e);
	}
	else
	{
		t.content.dialog("open").dialog("moveToTop");
		if (layer!="")
		{
			t.displayCanvas.clear();
			
			// Make sure the dialog is open and canvas is visible.
			t.displayCanvas.activate();
			
			var p = t.adjustPixPosition( e.pageX, e.pageY );
			t.geo = t.map.pixToGeo( p[0], p[1] );
			t.results.html("Retrieving results...");
			t.displayCanvas.clear();
			t.resultsArray = [];
			$.blockUI();
			MapChat.RunFunction("Map_Identify",{"args[map]":t.map.currentMap,
				"args[layer]":layer,"args[x]":t.geo[0],"args[y]":t.geo[1],
				"args[cellSize]":t.map.cellSize},function(xhr){t.parseResults(xhr);});
		}
	}
	return false;
};

/**
 * MapChat.mcTools.Identify.parseResults( xhr )
 *
 * called when a mouse button is clicked over theInsideLayer.
 */
MapChat.mcTools.Identify.prototype.parseResults = function(xhr) {
	var t = this, oLI = t.mapLayers.getLayerInfo(t.layerInput.val());
	
	t.resultsArray = $.parseJSON($(xhr.responseXML).find("results:first").text());
	t.results.html('<p>Clicked coordinates:</p><p style="margin-left:16px;font-style:italic;color:#555;">'+
		t.geo[0]+" "+t.geo[1]+'</p>');
	
	if (t.resultsArray.length==0)
	{
		$('<p style="margin-top:16px;text-indent:16px;">No Results</p>').appendTo(t.results);
	}
	else
	{
		$.each(t.resultsArray,function(i,v){
			var p = $('<p class="identify-result" style="cursor:pointer;background:#FFF url(images/icons/bullet_arrow_right.gif) no-repeat scroll;text-indent:16px;">Feature # '+v[0].si+':</p>').appendTo(t.results);
			var p2 = $('<p style="display:none;" />').appendTo(t.results);
			var ul = $('<ul style="list-style:none;margin-left:16px;"></ul>').appendTo(p2);
			var svg = v[0].shape;
			
			if (v[0].area>0 && v[0].length>0)
			{
				$('<li style="font-style:italic;color:#555;">Area: '+v[0].area+'</li>').appendTo(ul);
				$('<li style="font-style:italic;color:#555;">Perimeter: '+v[0].length+'</li>').appendTo(ul);
			}
			else if (v[0].length>0)
			{
				$('<li style="font-style:italic;color:#555;">Length: '+v[0].length+'</li>').appendTo(ul);
			}
			
			$.each(v[1],function(n,v){
				if (!oLI.field_aliases) $('<li>'+n+': '+v+'</li>').appendTo(ul);
				else if (oLI.field_aliases[n]) $('<li>'+oLI.field_aliases[n]+': '+v+'</li>').appendTo(ul);
			});
			
			p.click(function(e){
				if (!p[0].current)
				{
					t.results.find("p.identify-result").each(function(){
						if (this!=p[0])
						{
							this.current = false;
							$(this).css("background-color","#FFF");
						}
					});
					p[0].current = true;
					p.css("background-color","#CCCCFF");
					p.css("background-image","url(images/icons/bullet_arrow_down.gif)");
					p2.show();
					t.displayCanvas.clear();
					displayOpts = {stroke:{stroke:"#F00",'stroke-width':2,'stroke-opacity':0.8},r:5};
					if (v[0].type!=1) $.extend(displayOpts, {fill:{fill:"#F00",'fill-opacity':0.4}}); // Only fill points/polygons
					t.displayGeom = t.displayCanvas.addGeom(svg,displayOpts);
				}
				else if (t.resultsArray.length>1)
				{
					p2.toggle();
					p.css("background-image","url(images/icons/bullet_arrow_right.gif)");
				}
			});
			
			if (t.resultsArray.length==1)
			{
				p.click();
			}
			
		});
	}
	$.unblockUI();
};


