/**
 * A tool for measuring distance/area - based on ka-Map's kaArea.js
 *
 * @class MapChat.mcTools.Measure
 *
 * Depends:
 *	kamap/kaMap.js
 *	MapChat.js
 *	MapChat/application.js
 *	MapChat/Canvas.js
 */

$.extend(MapChat.mcTools,{
	Measure: function(options)
	{
		var self = this,
			options = (this.options = options),
			map = (this.map = options.map),
			cursor = (this.cursor = "crosshair"),
			name = (this.name = "Measure"),
			label = (this.label = $("<div />")),
			xy = (this.xy = []),
			geoxy = (this.geoxy = []),
			zIndex = (this.zIndex = options.zIndex||300),
			labelClass = (this.labelClass = options.labelClass||"tooltip"),
			timeout = (this.timeout = false);
			
			lineStyle = (this.lineStyle = {stroke:
				{stroke:options.lineColor||"#00F",'stroke-width':options.stroke||2,'stroke-opacity':options.opacity||.65}}),
			closeStyle = (this.closeStyle = {stroke:
				{stroke:options.closeColor||"#C73",'stroke-width':options.stroke||2,'stroke-opacity':options.opacity||.35}}),
			fillStyle = (this.fillStyle = {fill:
				{fill:options.fillColor||"#00F",'fill-opacity':options.opacity||.35}}),
			
			displayCanvas = (this.displayCanvas = MapChat.Class("MapChat.Canvas",{map:options.map,zIndex:zIndex})),
			
			measuring = (this.measuring = false),
			validPolygon = (this.validPolygon = true),
			lineDenom = (this.lineDenom = options.lineDenom||1),
			areaDenom = (this.areaDenom = options.areaDenom||1),
			
			// draw polygons/calculate areas if areaMode:true is passed as an option:
			areaMode = (this.areaMode = (options.areaMode?true:false)),
			
			lineLength = (this.lineLength = 0.0),
			fillArea = (this.fillArea = 0.0),
			
			unitAbrevations = (this.unitAbrevations = new Array("ft","in","km","m","mi","km")),
			lineUnits = (this.lineUnits = options.lineUnits||this.unitAbrevations[this.map.getCurrentMap().units]),
			areaUnits = (this.areaUnits = options.areaUnits||(this.unitAbrevations[this.map.getCurrentMap().units]+"2"));
			
		kaTool.apply( this, [map] );
		
		$.extend(fillStyle,lineStyle);
		
		label.css({
			position:"absolute",
			top:-300,
			left:-300,
			display:"none",
			"z-index": this.zIndex-1
		});
		label.addClass(this.labelClass);
		
		$(document.body).append(label);
		
		/** get all the prototype functions from the kaTool object */
		for (var p in kaTool.prototype) {
			if (!MapChat.mcTools.Measure.prototype[p])
				MapChat.mcTools.Measure.prototype[p]= kaTool.prototype[p];
		}
		
		return this;
	}
});

/**
 * activate this tool.  Activating the tool causes any existing tools to be
 * deactivated.
 */
MapChat.mcTools.Measure.prototype.activate = function()
{
	this.initGeom();
	this.displayCanvas.activate();
	MapChat.QueueEvent(3,"Measure",["Activate",(this.areaMode?"area":"distance")]);
	kaTool.prototype.activate.apply(this);
};

/**
 * deactivate this tool.
 */
MapChat.mcTools.Measure.prototype.deactivate = function()
{
	this.displayCanvas.deactivate();
	this.displayCanvas.clear();
	this.label.hide();
	this.label.html("");
	
	this.measuring = false;
	this.xy = [], this.geoxy = [];
	this.lineLength = this.fillArea = 0.0;
	
	kaTool.prototype.deactivate.apply(this);
};

/**
 * hide the tooltip label on mouse out:
 */
MapChat.mcTools.Measure.prototype.onmouseout = function(e)
{
	if (this.measuring) this.label.hide();
};

/**
 * hide the tooltip label on mouse out:
 */
MapChat.mcTools.Measure.prototype.onmouseover = function(e)
{
	if (this.label.html().length) this.label.show();
};

/**
 * MapChat.mcTools.Measure.onmousemove( e )
 *
 * called when the mouse moves over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
MapChat.mcTools.Measure.prototype.onmousemove = function(e)
{
	if (this.timeout) clearTimeout(this.timeout);
	e = $.event.fix(e?e:event);
	if (this.measuring)
	{
		if (this.geoxy.length>0)
		{
			this.label.css({left:e.pageX+15,top:e.pageY+15});
		}
		if (this.measuring)
		{
			var t = this;
			this.timeout = setTimeout(function(){
				var curPt = t.adjustPixPosition( e.pageX, e.pageY ),
					geoPt = t.map.pixToGeo(curPt[0],curPt[1]),
					curX = -curPt[0] - t.map.xOrigin,
					curY = -curPt[1] - t.map.yOrigin;
				
				if (!t.areaMode || t.geoxy.length>1)
				{
					t.displayGeom.xy[0] = t.geoxy.concat([[geoPt[0],geoPt[1]]]);
					t.displayGeom.draw();
				}
				
				if (t.areaMode)
				{
					t.closeGeom.xy[0] = [t.geoxy[0],[geoPt[0],geoPt[1]]];
					t.closeGeom.draw();
				}
				
				t.calculate(t.geoxy.concat([[geoPt[0],geoPt[1]]]));
			},33);
		}
	}
	
	return false;
};

/**
 * MapChat.mcTools.Measure.onmouseup( e )
 *
 * called when a mouse button is pressed over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
MapChat.mcTools.Measure.prototype.onmouseup = function(e)
{
	e = $.event.fix(e?e:event);
	if (e.button==2||e.metaKey||e.shiftKey)
	{
		return this.cancelEvent(e);
	}
	else
	{
		var pts = this.xy.length, curPt = this.adjustPixPosition( e.pageX, e.pageY ),
			newPt = [-curPt[0] - this.map.xOrigin,-curPt[1] - this.map.yOrigin];
		
		// Ignore points equal to the last one (e.g., on a double-click event):
		if (pts>0 && newPt[0] == this.xy[pts-1][0] && newPt[1] == this.xy[pts-1][1]) return;
		
		var geoPt = this.map.pixToGeo(curPt[0],curPt[1]);
		if (this.measuring)
		{
			this.xy.push([newPt[0],newPt[1]]);
			this.geoxy.push([geoPt[0],geoPt[1]]);
			this.validatePolygon(this.xy,true);
			this.calculate();
			MapChat.QueueEvent(3,"Measure",["Add",geoPt[0],geoPt[1]]);
		}
		else
		{
			this.label.hide();
			this.measuring = true;
			this.xy = [[newPt[0],newPt[1]]];
			this.geoxy = [[geoPt[0],geoPt[1]]];
			this.displayGeom.xy = [[[geoPt[0],geoPt[1]]]];
			this.displayGeom.closePath = false;
			this.displayGeom.draw();
			if (this.areaMode)
			{
				this.closeGeom.xy = [[[geoPt[0],geoPt[1]],[geoPt[0],geoPt[1]]]];
				this.closeGeom.draw();
			}
			MapChat.QueueEvent(3,"Measure",["Start",geoPt[0],geoPt[1]]);
		}
	}
};

/**
 * MapChat.mcTools.Measure.ondblclick( e )
 *
 * called when a double-click event is triggered on theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
MapChat.mcTools.Measure.prototype.ondblclick = function(e)
{
	this.measuring = false;
	if (this.areaMode)
	{
		this.displayGeom.closePath = true;
		this.closeGeom.xy = [[[]]];
		this.closeGeom.draw();
	}
	this.displayGeom.draw();
	this.calculate();
	MapChat.QueueEvent(3,"Measure",["End"]);
}

/**
 * MapChat.mcTools.Measure.scaleChanged( e )
 *
 * When the scale changes, the measurement drawing needs to be redrawn at the correct scale.
 *
 */
MapChat.mcTools.Measure.prototype.scaleChanged = function(e)
{
	var self = this;
	self.label.hide();
	self.xy = [];
	$.each(self.geoxy,function(i){
		var pixPt = self.map.geoToPix(self.geoxy[i][0],self.geoxy[i][1]);
		self.xy.push([pixPt[0]-self.map.xOrigin,pixPt[1]-self.map.yOrigin]);
	});
	var uaPx = self.unadjustPixPosition(self.xy[self.xy.length-1][0],self.xy[self.xy.length-1][1]);
	self.label.css({left:uaPx[0]+15,top:uaPx[1]+15});
	self.label.show();
}

/**
 * MapChat.mcTools.Measure.extentsChanged( e )
 *
 * move the label if the extents change.
 *
 * e - object, the event object or null (in ie)
 */
MapChat.mcTools.Measure.prototype.extentsChanged = function()
{
	if (!this.measuring&&this.xy.length>0)
	{
		var uaPx = this.unadjustPixPosition(this.xy[this.xy.length-1][0],this.xy[this.xy.length-1][1]);
		this.label.css({left:uaPx[0]+15,top:uaPx[1]+15});
	}
};

MapChat.mcTools.Measure.prototype.initGeom = function()
{
	if (this.areaMode)
	{
		this.displayGeom = this.displayCanvas.newGeom("path",this.fillStyle);
		this.closeGeom = this.displayCanvas.newGeom("path",this.closeStyle);
		this.closeGeom.xy = [[]];
	}
	else
	{
		this.displayGeom = this.displayCanvas.newGeom("path",this.lineStyle);
	}
	
	this.displayGeom.xy = [[]];
}

/**
 * MapChat.mcTools.Measure.validatePolygon(x,y)
 *
 * check for self-intersecting polygons before calculating the area - only
 * check to see if the last (or closing) line segments intersect any previous
 * line segments.
 *
 * x,y - arrays that optionally override the currently recorded points
 */
MapChat.mcTools.Measure.prototype.validatePolygon = function(xy,permanent)
{
	// if x/y arrays are not supplied, default to the
	// stored x/y arrays for this object:
	if (!(xy && xy.length>0))
	{
		xy = this.xy;
	}
	var pts = xy.length;
	
	if (pts<4)
	{
		return this.validPolygon = true;
	}
	
	if (!this.validPolygon)
	{
		return false;
	}
	
	var last = [xy[pts-2][0],xy[pts-2][1],xy[pts-1][0],xy[pts-1][1]],
		close = (this.areaMode?[xy[pts-1][0],xy[pts-1][1],xy[0][0],xy[0][1]]:false);
	
	for (var p0=0;p0<(pts-2);p0++)
	{
		var p1 = p0+1;
		if (p0<(pts-3)&&this.lineIntersect(xy[p0][0],xy[p0][1],xy[p1][0],xy[p1][1],last[0],last[1],last[2],last[3]))
		{
			if (permanent) this.validPolygon = false;
			return false;
		}
		if (p0>0&&close&&this.lineIntersect(xy[p0][0],xy[p0][1],xy[p1][0],xy[p1][1],close[0],close[1],close[2],close[3]))
		{
			return false;
		}
	}
	
	return true;
};

MapChat.mcTools.Measure.prototype.lineIntersect = function(x0,y0,x1,y1,x2,y2,x3,y3)
{
	//codi extret de http://www.whisqu.se/per/docs/math28.htm
	var a1,b1,c1,a2,b2,c2,det_inv,m1,m2,x,y;
	
	if ((x1-x0)!=0)
		m1 = (y1-y0)/(x1-x0);
	else
		m1 = 999999999.999999;   // close enough to infinity
	
	if ((x3-x2)!=0)
		m2 = (y3-y2)/(x3-x2);
	else
		m2 = 999999999.999999;   // close enough to infinity
	
	a1 = m1;
	a2 = m2;
	b1 = -1;
	b2 = -1;
	c1 = (y0-m1*x0);
	c2 = (y2-m2*x2);
	
	// inverse of the determinate
	det_inv = 1/(a1*b2 - a2*b1);
	
	// use Kramers rule to compute xi and yi
	x=((b1*c2 - b2*c1)*det_inv);
	y=((a2*c1 - a1*c2)*det_inv);

	if ( (x>=Math.min(x0,x1) && x<=Math.max(x0,x1) 
		&& y>=Math.min(y0,y1) && y<=Math.max(y0,y1)) 
		&& (x>=Math.min(x2,x3) && x<=Math.max(x2,x3) 
		&& y>=Math.min(y2,y3) && y<=Math.max(y2,y3)) ) return true;
	
	else return false;
}


// Implementaci� del c�lcul de l'�rea basada en http://local.wasp.uwa.edu.au/~pbourke/geometry/polyarea/
// Funciona amb pol�gons simples (sense encreuaments), tant convexos com c�ncavs
MapChat.mcTools.Measure.prototype.calculate = function(xy)
{
	// if x/y arrays are not supplied, default to the
	// stored geox/geoy arrays for this object:
	if (!(xy && xy.length>0))
	{
		xy = this.geoxy;
	}
	var pts = xy.length, len = 0.0, area = 0.0, validPoly = true;
	
	if (pts<2)
	{
		this.lineLength = this.fillArea = 0.0;
	}
	
	if (pts<3)
	{
		var dx = xy[0][0]-xy[1][0], dy = xy[0][1]-xy[1][1];
		this.lineLength = Math.sqrt(dx*dx+dy*dy)/this.lineDenom;
		this.fillArea = 0.0;
	}
	
	else
	{
		validPoly = (pts<4||this.validatePolygon(xy));
		for (var i=0;i<pts;i++)
		{
			var j = (i + 1) % pts;
			
			var dx = xy[i][0]-xy[j][0], dy = xy[i][1]-xy[j][1];
			
			if (j!=0||this.areaMode)
			{
				len += Math.sqrt(dx*dx+dy*dy);
			}
			
			if (this.areaMode && validPoly)
			{
				area += Math.abs(xy[i][0]) * Math.abs(xy[j][1]);
				area -= Math.abs(xy[i][1]) * Math.abs(xy[j][0]);
			}
		}
		
		area /= 2;
		
		this.lineLength = len/this.lineDenom;
		this.fillArea = (validPoly?Math.abs(area)/(this.areaDenom*this.areaDenom):"Invalid Polygon");
	}
	
	// make nicer looking numbers:
	var dispLen = (this.lineLength<0?
		this.lineLength.toPrecision(6):
			(this.lineLength<1000?this.lineLength.toPrecision(4):
				Math.round(this.lineLength)));
	var dispArea = (this.fillArea<0?
		this.fillArea.toPrecision(6):
			(this.fillArea<1000?this.fillArea.toPrecision(4):
				Math.round(this.fillArea)));
	
	if (this.areaMode)
	{
		if (validPoly)
		{
			this.label.html("Area: "+dispArea+" "+this.areaUnits+
				"<br />Perimeter: "+dispLen+" "+this.lineUnits);
		}
		else
		{
			this.label.html("n/a");
		}
	}
	else
	{
		this.label.html("Distance: "+dispLen+" "+this.lineUnits);
	}
	
	this.label.show();
};

/**
 * adjust a page-relative pixel position into a kaMap relative
 * pixel position
 *
 * @param {Integer} x the x page coordinate to convert
 * @param {Integer} y the y page coordinate to convert
 * @return {Array} return an array containing the converted coordinates
 */
MapChat.mcTools.Measure.prototype.unadjustPixPosition = function( pX, pY )
{
	var x = pX+this.mapCanvas.offset().left;
	var y = pY+this.mapCanvas.offset().top;
	return [x,y];
};