/**
 * A rubber-zoom box - adapted from ka-Map's kaRubberZoom.js.
 *
 * @class MapChat.mcTools.ZoomIn
 *
 * Depends:
 *	kaMap.js
 *	MapChat.js
 *	MapChat/application.js
 */

$.extend(MapChat.mcTools,{
	ZoomIn: function(options)
	{
		var self = this,
			options = (this.options = options),
			map = (this.map = options.map),
			cursor = (this.cursor = "crosshair"),
			div = (this.div = $("<div>&nbsp;</div>")),
			startx = (this.startx = null),
			starty = (this.starty = null),
			endx = (this.endx = null),
			endy = (this.endy = null),
			bMouseDown = (this.bMouseDown = false),
			name = (this.name = "ZoomIn");
		
		kaTool.apply( this, [map] );
		
		div.css({
			position:"absolute",
			top:0,
			left:0,
			width:1,
			height:1,
			zIndex:500,
			visibility:"hidden",
			border:"1px solid blue",
			"background-color": "#FFF",
			opacity:.35,
			"font-size":1,
			"line-height":0
		});
		
		$(map.theInsideLayer).append(div);
		
		/** get all the prototype functions from the kaTool object */
		for (var p in kaTool.prototype) {
			if (!MapChat.mcTools.ZoomIn.prototype[p])
				MapChat.mcTools.ZoomIn.prototype[p]= kaTool.prototype[p];
		}
		
		return this;
	}
});

/*
 * draw a box representing the query region.
 *
 * MapChat.mcTools.ZoomIn maintains the query region in four variables.  The variables are
 * assumed to be in pixel coordinates and are used to position the box.  If
 * any of the coordinates are null, clear the query box.
 */
MapChat.mcTools.ZoomIn.prototype.drawZoomBox = function() {
	if (this.startx == null || this.starty == null ||
		this.endx == null || this.endy == null ) {
		this.div.css({
			visibility:'hidden',
			top:0,
			left:0,
			width:1,
			height:1
		});
		return;
	}
	
	this.div.css({visibility:'visible'});
	if (this.endx < this.startx) {
		this.div.css({left:(this.endx - this.map.xOrigin),width:(this.startx - this.endx)});
	} else {
		this.div.css({left:(this.startx - this.map.xOrigin),width:(this.endx - this.startx)});
	}
	
	if (this.endy < this.starty) {
		this.div.css({top:(this.endy - this.map.yOrigin),height:(this.starty - this.endy)});
	} else {
		this.div.css({top:(this.starty - this.map.yOrigin),height:(this.endy - this.starty)});
	}
};

/**
 * MapChat.mcTools.ZoomIn.onmouseout( e )
 *
 * called when the mouse leaves theInsideLayer.  Terminate the query
 *
 * e - object, the event object or null (in ie)
 */
MapChat.mcTools.ZoomIn.prototype.onmouseout = function(e) {
	e = $.event.fix(e?e:event);
	if (e.target.id == this.map.domObj.id) {
		this.bMouseDown = false;
		this.startx = this.endx = this.starty = this.endy = null;
		this.drawZoomBox();
		return kaTool.prototype.onmouseout.apply(this, [e]);
	}
};

/**
 * MapChat.mcTools.ZoomIn.onmousemove( e )
 *
 * called when the mouse moves over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
MapChat.mcTools.ZoomIn.prototype.onmousemove = function(e) {
	e = $.event.fix(e?e:event);
	if (!this.bMouseDown)
	{
		return false;
	}
	var x = e.pageX, y = e.pageY, aPixPos = this.adjustPixPosition( x, y );
	this.endx=-aPixPos[0];
	this.endy=-aPixPos[1];
	this.drawZoomBox();
	return false;
}
;
/**
 * MapChat.mcTools.ZoomIn.onmousedown( e )
 *
 * called when a mouse button is pressed over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
MapChat.mcTools.ZoomIn.prototype.onmousedown = function(e)
{
	e = $.event.fix(e?e:event);
	
	if (e.button==2||e.metaKey||e.shiftKey)
	{
		return this.cancelEvent(e);
	}
	else
	{
		if (this.map.isIE4) document.onkeydown = kaTool_redirect_onkeypress;
		document.onkeypress = kaTool_redirect_onkeypress;
		
		this.bMouseDown=true;
		var x = e.pageX, y = e.pageY, aPixPos = this.adjustPixPosition( x,y );
		this.startx=this.endx=-aPixPos[0];
		this.starty=this.endy=-aPixPos[1];
		this.drawZoomBox();
		
		e.cancelBubble = true;
		e.returnValue = false;
		if (e.stopPropogation) e.stopPropogation();
		if (e.preventDefault) e.preventDefault();
		return false;
	}
};

/**
 * MapChat.mcTools.ZoomIn.onmouseup( e )
 *
 * called when a mouse button is clicked over theInsideLayer.
 *
 * e - object, the event object or null (in ie)
 */
MapChat.mcTools.ZoomIn.prototype.onmouseup = function(e) {
	e = $.event.fix(e?e:event);
	var start = this.map.pixToGeo( -this.startx, -this.starty ), coords = start;
	
	if (this.startx!=this.endx&&this.starty!=this.endy)
	{
		coords = start.concat(this.map.pixToGeo( -this.endx, -this.endy ));
		if (coords[2] < coords[0])
		{
			//minx gt maxx then invert values
			var minx = coords[2], maxx = coords[0];
			coords[0] = minx;
			coords[2] = maxx;
		}
		if (coords[1] < coords[3])
		{
			//miny gt maxy then invert values
			var miny = coords[1], maxy = coords[3];
			coords[3] = miny;
			coords[1] = maxy;
		}
	}
	
	this.startx = this.endx = this.starty = this.endy = null;
	this.drawZoomBox();
	
	if(coords[2] && coords[0] && coords[3] && coords[1])
	{
		//Rubber Zoom
		MapChat.QueueEvent(3,"ZoomIn",["box",coords[0],coords[1],coords[2],coords[3]]);
		this.map.zoomToExtents(coords[0],coords[1],coords[2],coords[3]);
	}
	else if (coords[0] && coords[1])
	{
		//single click
		this.map.zoomIn();
		this.map.zoomTo(coords[0],coords[1]);
		MapChat.QueueEvent(3,"ZoomIn",["point",coords[0],coords[1]]);
		
	}
	
	return false;
};
