<?php

class MapChat_API_Chat_Get
{
	public function __construct()
	{
		// Load the Response Object from the Api Controller
		$this->xml = Zend_Registry::get("XMLResponse");
		$this->config = Zend_Registry::get("config");
		$this->discussion = Zend_Registry::get("discussion");
		$this->me = Zend_Registry::get("me");
	}

	/**
	 * Gets messages from the database.
	 * 
	 * @param string $opts - only messages that match the options specified in this JSON-encoded array will be returned.
	 *
	 * Possible values that can be included in the opts parameter:
	 *  maps: REQUIRED - an array of map names that should be checked for user-visibility states.
	 *  sinceTime: if specified, only messages recorded after this number of milliseconds since the UNIX 'epoch' time will be returned.
	 *  sinceID: if specified, -2 gets the most recent batch of messages; greater than or equal to -1 gets the next batch
	 *           of in sequential order messages with IDs greater than sinceID.
	 *  limit: defaults to 25; 0 will retrieve all messages; values greater than zero will limit the number of messages returned to
	 *         the specified value.
	 *  from: if set, only return messages from the specified username
	 *  reverse: if true, then get messages *before* sinceID
	 *  depth: the thread-depth to get messages at; -1 is any depth, 0 is the top-level message that starts thread, 1, etc...
	 *  flags: an array of text flags - if defined, only messages that satisfy these flags will be returned.
	 *  selection: an integer that references the id of a set of features selected by the user - only messages explicitly linked
	 *             to these features will be returned.
	 *  intersects: the same as 'selection', but will return any messages that are referenced to geographic features that intersect
	 *              the selection set identified.
	 *  extent: an array of xmin,ymin,xmax,ymax coordinates to explicitly define a geographical extent for this message - if
	 *          the min/max coordinates are the same (or max is missing), the extent will be treated as a point.
	 *  distance: the distance to search for nearby messages (works with intersects or extent).
	 *  list: if true, only IDs will be returned for matching messages
	 *  describe: if an array of IDs is supplied, details will be returned for the matching messages - NOTE: all above parameters
	 *            will be ignored in the request if this is used
	 *  children: if true, then the messages returned will be replies to sinceID or IDs listed in 'describe' (ignored if sinceID or 
	 *            describe are not set)
	 */
	public function Get($opts=NULL)
	{
		$opts = Zend_Json::decode($opts);
		$chat_messages = $this->discussion->Chat_Get($opts);
		$this->xml->chat_messages = Zend_Json::encode($chat_messages);
		$this->xml->chat_messages->addAttribute("length",count($chat_messages));
		
		// The assumption here is that requesting the latest batch happens at the start.
		if (isset($opts["sinceID"]) && $opts["sinceID"]==-2) $this->xml->chat_messages->addAttribute("count",$this->discussion->Chat_MessageCount());
		
		return $this->xml;
	}
}

?>