<?php

class MapChat_API_Chat_Send
{
	public function __construct()
	{
		// Load the Response Object from the Api Controller
		$this->xml = Zend_Registry::get("XMLResponse");
		$this->config = Zend_Registry::get("config");
		$this->discussion = Zend_Registry::get("discussion");
		$this->me = Zend_Registry::get("me");
	}

	/**
	 * Send messages to the server.
	 * 
	 * @param string $opts - only messages that match the options specified in this JSON-encoded array will be returned.
	 *
	 * Possible values that can be included in the opts parameter:
	 *  clientTime: the time reported by the client, adjusted for the time delta relative to the server (should be pretty much equal).
	 *  parentID: if specified, this message is in reply to a previous message.
	 *  flags: an array of text flags - if defined, these will be saved in a space-separated string in the chat_log
	 *  selection: a list of integers that references a set of features selected by the user - if specified, these will be included
	 *             in the spatial links for the chat message.
	 *  drawing: an integer that references the id of a user-drawn feature - if specified, this will be included
	 *           in the spatial links for the chat message.
	 *  extent: an array of xmin,ymin,xmax,ymax coordinates to explicitly define a geographical extent for this message - if
	 *          the min/max coordinates are the same (or max is missing), the extent will be treated as a point.  This will be set as the
	 *          the extent for this message.
	 *  distance: (not yet implemented) if set, the geometry for the chat_log will be a buffer around the extent and/or selected
	 *            features included above
	 *  type: if set, this will determine the 'type' of message (e.g., if some level of argumentation structure is to be used in a discussion)
	 *  text: the text of the message
	 */
	public function Send($opts=NULL)
	{
		$this->xml->chat_send = Zend_Json::encode($this->discussion->Chat_Send(Zend_Json::decode($opts)));
		return $this->xml;
	}
}

?>