<?php 

class MapChat_API_Discussion_Edit
{
	private $_xml = null;

	public function __construct()
	{
		// Load the Response Object from the Api Controller
		$this->_xml = Zend_Registry::get("XMLResponse");
		$this->me = Zend_Registry::get("me");
	}
	
	public function Edit($discussion,$title,$description,$moderator,$mapfile,$config,$srid,$extent,$disabled,$members)
	{
		// only let logged-in users get this:
		if (Zend_Registry::get("auth")->hasIdentity() && $this->me->canManageDiscussions())
		{
			$d = new MapChat_Discussion($discussion);
			if ($srid!=$d->srid)
			{
				throw new MapChat_Exception("Changing the spatial reference for the discussion is not currently supported.");
			}
			
			$opts = array("title"=>$title,"description"=>$description,"moderator"=>$moderator,
			"mapfile"=>$mapfile,"config"=>$config,"disabled"=>$disabled);
			if (strlen($extent))
			{
				$opts["extent"]=$extent;
			}
			$d->setInfo($opts);
			
			// Run through the list of users/groups and add them as members:
			if ($members!="false")
			{
				$aMembers = array("user"=>array(),"group"=>array());
				$oMembers = Zend_Json::decode($members);
				foreach ($oMembers["user"] as $member)
				{
					$user = new MapChat_User($member["id"]);
					$d->addMember($user,$member["ro"]);
					$aMembers["user"][$member["id"]] = $member["ro"];
				}
				
				foreach ($oMembers["group"] as $member)
				{
					$group = new MapChat_Group($member["id"]);
					$d->addMember($group,$member["ro"]);
					$aMembers["group"][$member["id"]] = $member["ro"];
				}
				
				
				// Drop an existing members that weren't in the original user list, and reset
				// the readonly status for those that are (in case previously existing user
				// readonly states were changed):
				$aCurMembers = array("user"=>$d->listUsers(false),"group"=>$d->listGroups());
				foreach ($aCurMembers["user"] as $member)
				{
					$u = new MapChat_User($member);
					if (!isset($aMembers["user"][$member]))
					{
						$d->dropMember($u);
					}
					else
					{
						$d->setReadonly($u,$aMembers["user"][$member]);
					}
				}
				
				foreach ($aCurMembers["group"] as $member)
				{
					$g = new MapChat_Group($member);
					if (!isset($aMembers["group"][$member]))
					{
						$d->dropMember($g);
					}
					else
					{
						$d->setReadonly($g,$aMembers["group"][$member]);
					}
				}
			}
			
			$this->_xml->result = $d->exists();
			return $this->_xml;
		}
		else
		{
			throw new MapChat_Exception("Unauthorized Access.");
		}
	}
	
}

?>