<?php 
/**
 * Run startup functions when a user enters a MapChat discussion.
 */
class MapChat_API_Discussion_Start
{
	public function __construct()
	{
		// Load the Response Object from the Api Controller
		$this->xml = Zend_Registry::get("XMLResponse");
		$this->me = Zend_Registry::get("me");
	}
	
	/*
	 * Make sure the user is a member of the requested discussion, log the start of
	 * the discussion, load maps, etc...
	 * 
	 * @param string $d the name of the discussion to load:
	 */
	public function Start($d)
	{
		// only let logged-in users get this:
		if (Zend_Registry::get("auth")->hasIdentity())
		{
			$oDiscussion = new MapChat_Discussion($d);
			
			// If there is any data cached for this discussion, delete it (this means it will be bad to have
			// the same session open in multiple browsers).
			if (is_array(MapChat_Session::ns()->discussions) && isset(MapChat_Session::ns()->discussions[$d]))
				unset(MapChat_Session::ns()->discussions[$d]);
			
			if (!$oDiscussion->hasMember($this->me))
			{
				throw new MapChat_Exception("Unauthorized Access.");
			}
			
			// calling getReadonly will throw an exception if the user is not a
			// member of the discussion:
			$ro = $oDiscussion->getReadonly($this->me);
			
			$this->xml->discussion->name = $d;
			$this->xml->discussion->ro = ($ro?"true":"false");
			$this->xml->discussion->time = date($_SERVER['REQUEST_TIME']);
			
			// Save the discussion object to the registry:
			MapChat_Session::ns()->discussion = $oDiscussion;
			
			// Append the discussion's config to the responseXML:
			MapChat_XML::append($this->xml->discussion,$oDiscussion->discussion_config);
			
			// Make sure there's no flag to redirect the browser any more (allows the page to be
			// reloaded to get back to the main page).
			unset(MapChat_Session::ns()->load_discussion);
			
// 			// setup the logger here:
// 			$event = new MapChat_Discussion_Event($oDiscussion);
// 			$event->logEntry();
			$oDiscussion->Event_logEntry();
			
			// Set the user's online status:
			$oDiscussion->Event_userOnline();
			
			return $this->xml;
		}
		else
		{
			throw new MapChat_Exception("Unauthorized Access.");
		}
	}
	
}

?>