<?php 
/**
 * Drops all layers that belong to a given display group from a mapfile
 *
 * @class MapChat_API_Plan_GetFeature
 *
 */
class MapChat_API_Map_DropDisplayGroup
{
	public function __construct()
	{
		// Load the Response Object from the Api Controller
		$this->xml = Zend_Registry::get("XMLResponse");
		$this->discussion = Zend_Registry::get("discussion");
		$this->me = Zend_Registry::get("me");
	}
	
	/**
	 * Return a JSON-encoded array of information describing the affected layers in the map (those with an index greater than any dropped indexes);
	 *
	 * @param string $dg - the name of the display group to drop
	 * @param string $mapfile - the map to drop the display group from
	 */
	public function DropDisplayGroup($dg,$mapfile)
	{
		$oMap = $this->discussion->Map_getMapObj($mapfile);
		$affectedLayers = array();
		$groupLayers = array();
		$affectedGroups = array();
		$subtractIndex = 0;
		for ($i=0;$i<$oMap->numlayers;$i++)
		{
			$oLayer = $oMap->getLayer($i);
			
			if ($oLayer->getMetaData("mapchat_display_group")==$dg)
			{
				$subtractIndex++;
				$affectedGroups[$oLayer->group] = $oLayer->group;
				$oLayer->set("status",MS_DELETE);
			}
			elseif ($subtractIndex>0)
			{
				$affectedLayers[$oLayer->name] = $subtractIndex;
			}
			
			if ($oLayer->status == MS_ON)
			{
				if (isset($groupLayers[$oLayer->group])) $groupLayers[$oLayer->group][] = $oLayer->index-$subtractIndex;
				else $groupLayers[$oLayer->group] = array($oLayer->index-$subtractIndex);
				if ($subtractIndex>0) $affectedGroups[$oLayer->group] = $oLayer->group;
			}
		}
		
		$affectedGroupLayers = array();
		foreach ($affectedGroups as $group)
		{
			$affectedGroupLayers[$group] = (isset($groupLayers[$group])?$groupLayers[$group]:array());
			
			// If this group wasn't flagged as a user layer before, then it should be now, since the indexes have changed.
			$aLayers = $oMap->getLayersIndexByGroup($group);
			foreach ($aLayers as $i)
			{
				$oLayer = $oMap->getLayer($i);
				$oLayer->setMetaData("mapchat_userlayer","true");
			}
		}
		
		$this->discussion->Map_saveMapfile($mapfile,$oMap);
		
		$this->xml->affected = Zend_Json::encode(array("layers"=>$affectedLayers,"groups"=>$affectedGroupLayers));
		$this->discussion->Event_log(array(
			"type"=>5,
			"text"=>"Map_DropDisplayGroup",
			"data"=>array($dg,$mapfile),
			"username"=>$this->me->username,
		));
		return $this->xml;
	}
	
}