<?php 

class MapChat_API_Map_LayerStates
{
	public function __construct()
	{
		// Load the Response Object from the Api Controller
		$this->xml = Zend_Registry::get("XMLResponse");
		$this->config = Zend_Registry::get("config");
		$this->discussion = Zend_Registry::get("discussion");
		$this->me = Zend_Registry::get("me");
	}
	
	/**
	 * Accepts a JSON-encoded array of mapfile/layer names, with
	 * boolean on/off layer states to set in the discussion maps.
	 *
	 * @param string $states JSON-encoded list of layer states.
	 */
	public function LayerStates($states)
	{
		$aMapFileGroups = array();
		$aMapFileDisplayGroups = array();
		
		$aStates = Zend_Json::decode($states);
		foreach ($aStates as $map => $aLayers)
		{
			foreach ($aLayers as $layer => $state)
			{
				$this->discussion->Layer_setProps($map,$layer,array("status"=>$state?MS_ON:MS_OFF));
			}
			
			// Return a listing of group states so the client knows which image layers need to be turned on/off
			$oMap = $this->discussion->Map_getMapObj($map);
			$aGroups = $oMap->getAllGroupNames();
			foreach ($aGroups as $groupIdx => $groupName)
			{
				$aLayersIdx = $oMap->getLayersIndexByGroup($groupName);
				$bGroupStatus = NULL;
				$bPartialStatus = false;
				$aLayers = array();
				
				foreach ($aLayersIdx as $i)
				{
					$oLayer = $oMap->getLayer($i);
					if ($oLayer->status==MS_ON) $aLayers[] = $oLayer->index;
					if (($bGroupStatus && $oLayer->status==MS_OFF) || ($bGroupStatus===false && $oLayer->status==MS_ON))
					{
						$bPartialStatus = true;
					}
					if ($oLayer->status==MS_ON)
					{
						$bGroupStatus = true;
					}
				}
				
				$aGroups[$groupIdx] = array(
					"name"=>$groupName,
					"status"=>$bGroupStatus,
					"partial_status"=>$bPartialStatus,
					"layers"=>$aLayers,
				);
			}
			$aMapFileGroups[$map] = $aGroups;
			
			// Return a listing of display group states so the client is able to update any UI controls:
			$aDisplayGroups = array();
			for ($i=0;$i<$oMap->numlayers;$i++)
			{
				$oLayer = $oMap->getLayer($i);
				$sDG = $oLayer->getMetaData("mapchat_display_group");
				if (!isset($aDisplayGroups[$sDG])) $aDisplayGroups[$sDG] = 
					array("status"=>NULL,"partial_status"=>false);
				
				if (($aDisplayGroups[$sDG]["status"] && $oLayer->status==MS_OFF) ||($aDisplayGroups[$sDG]===false && $oLayer->status==MS_ON))
				{
					$aDisplayGroups[$sDG]["partial_status"] = true;
				}
				if ($oLayer->status==MS_ON)
				{
					$aDisplayGroups[$sDG]["status"] = true;
				}
			}
			$aMapFileDisplayGroups[$map] = $aDisplayGroups;
		}
		
		$this->xml->addChild("success");
		
		$groupStates = Zend_Json::encode($aMapFileGroups);
		$this->xml->group_states = $groupStates;
		$this->xml->display_group_states = Zend_Json::encode($aMapFileDisplayGroups);
		
		$this->discussion->Event_log(array(
			"type"=>5,
			"text"=>"Map_LayerStates",
			"data"=>array($states,$groupStates),
			"username"=>$this->me->username,
		));
		
		return $this->xml;
	}
}

?>