<?php 
/**
 * Copies all content from an existing plan to a new plan with a different title.
 *
 * @class MapChat_API_Plan_Copy
 *
 */
 
class MapChat_API_Plan_Copy
{
	public function __construct()
	{
		// Load the Response Object from the Api Controller
		$this->xml = Zend_Registry::get("XMLResponse");
		$this->discussion = Zend_Registry::get("discussion");
		$this->me = Zend_Registry::get("me");
	}
	
	/**
	 * Returns the a JSON-encoded array of descriptive information.
	 *
	 * @param integer $plan_id - the id of the plan to copy.
	 * @param integer $newTitle - the title for the new copy of the plan.
	 * @param string $mapfile - the name of the mapfile to add the plan's layer to.
	 */
	public function Copy($plan_id,$newTitle,$mapfile=false)
	{
		// Read-only members cannot copy plans.
		if ($this->discussion->getReadOnly($this->me)) throw new MapChat_Exception("Insufficient permissions to create a new plan");
		$new_id = $this->discussion->Plan_Copy($plan_id,$newTitle);
		$this->xml->plan_id = $new_id;
		$this->xml->imageformat = $this->discussion->discussion_config->user_imageformat;
		$this->xml->opacity = $this->discussion->discussion_config->user_opacity;
		if ($mapfile!==false) $this->xml->plan_maplayers = Zend_Json::encode($this->discussion->Plan_Open($new_id,$mapfile));
		$this->xml->plan_info = Zend_Json::encode($this->discussion->Plan_Info($new_id));
		$this->xml->plan_feature_counts = Zend_Json::encode($this->discussion->Plan_FeatureCounts($new_id));
		$this->discussion->Event_log(array(
			"type"=>4,
			"text"=>"Plan_Copy",
			"data"=>array($plan_id,$new_id,$newTitle),
			"username"=>$this->me->username,
		));
		return $this->xml;
	}
}

?>