<?php 
/**
 * Create a new plan for the current user.
 *
 * @class MapChat_API_Plan_Create
 *
 */
 
class MapChat_API_Plan_Create
{
	public function __construct()
	{
		// Load the Response Object from the Api Controller
		$this->xml = Zend_Registry::get("XMLResponse");
		$this->discussion = Zend_Registry::get("discussion");
		$this->me = Zend_Registry::get("me");
	}
	
	/**
	 * Returns the id of the new plan in the database upon successful creation.
	 *
	 * @param mixed $title - the title to be assigned to the plan
	 * @param mixed $type - the type of plan to look for (either an integer value, or the
	 *  description value listed in the plan_types lookup table);
	 * @param mixed $mapfile - if specified, add the layers to the named mapfile.
	 */
	public function Create($title,$type,$mapfile=false)
	{
		// Read-only members cannot create new plans in the map.
		if ($this->discussion->getReadOnly($this->me)) throw new MapChat_Exception("Insufficient permissions to create a new plan");
		
		$plan_id = $this->discussion->Plan_Create($title,$type);
		
		$this->xml->plan_id = $plan_id;
		$this->xml->imageformat = $this->discussion->discussion_config->user_imageformat;
		$this->xml->opacity = $this->discussion->discussion_config->user_opacity;
		$this->xml->plan_maplayers = Zend_Json::encode($this->discussion->Plan_Setup($plan_id,$mapfile));
		$this->xml->plan_info = Zend_Json::encode($this->discussion->Plan_Info($plan_id));
		$this->xml->plan_feature_counts = Zend_Json::encode($this->discussion->Plan_FeatureCounts($plan_id));
		$this->discussion->Event_log(array(
			"type"=>4,
			"text"=>"Plan_Create",
			"data"=>array($plan_id,$title,$type),
			"username"=>$this->me->username,
		));
		return $this->xml;
	}
}

?>