<?php

class MapChat_API_Startup_Options
{
	private $_xml = null;

	public function __construct()
	{
		// Load the Response Object from the Api Controller
		$this->_xml = Zend_Registry::get("XMLResponse");
		$this->me = Zend_Registry::get("me");
	}

	public function Options($browser)
	{
		// Save the browser information:
		MapChat_Session::ns()->browser = Zend_Json::decode($browser);
		
		// HTML will contain an XML representation of the
		// acutal html that will be displayed to the user.
		$html = $this->_xml->addChild("div");
		
		// The bindings will contain the information about
		// what to do for the 'click' event of each option.
		$optBindings = $this->_xml->addChild("binds");
		
		$dOpts = $html->addChild("div");
		$dOpts->addAttribute("id","DiscussionList");
		$dOpts->addAttribute("class","opts_section");
		$dOpts->p[0] = "Available Maps";
		$dOpts->p[0]->addAttribute("class","opts_title");
		$dOpts->addChild("ul");
		$aDiscussions = $this->me->listDiscussions();
		$optBindings->addChild("dOpts");
		
		foreach ($aDiscussions as $sDiscussion)
		{
			$d = new MapChat_Discussion($sDiscussion);
			$li = $dOpts->ul->addChild("li",$d->title);
			$li->addAttribute("class","dOpts");
			$optBindings->dOpts->addChild("opt",json_encode(array("bind"=>"click",
				"run"=>"MapChat.LoadDiscussion(\"".$d->discussion."\");",)));
		}
		
		$uOpts = $html->addChild("div");
		$uOpts->addAttribute("id","UserOptions");
		$uOpts->addAttribute("class","opts_section");
		$uOpts->p[0] = "User Options";
		$uOpts->p[0]->addAttribute("class","opts_title");
		$optBindings->addChild("uOpts");
		
		$uOpts->ul->li[0] = "Edit Profile";
		$uOpts->ul->li[0]->addAttribute("class","uOpts");
		$optBindings->uOpts->opt[0] = json_encode(array("bind"=>"click",
			"run"=>"MapChat.Using('User');MapChat.mcuser.showProfile();",));
		
		$uOpts->ul->li[1] = "Change Password";
		$uOpts->ul->li[1]->addAttribute("class","uOpts");
		$optBindings->uOpts->opt[1] = json_encode(array("bind"=>"click",
			"run"=>"MapChat.Using('User');MapChat.mcuser.showPW();",));
		
		$uOpts->ul->li[2] = "Logout";
		$uOpts->ul->li[2]->addAttribute("class","uOpts");
		$optBindings->uOpts->opt[2] = json_encode(array("bind"=>"click","run"=>"MapChat.logout();",));
		
		
		if ($this->me->hasSomeAdmin())
		{
			$aOpts = $html->addChild("div");
			$aOpts->addAttribute("id","AdminOptions");
			$aOpts->addAttribute("class","opts_section");
			$aOpts->p[0] = "Admin Options";
			$aOpts->p[0]->addAttribute("class","opts_title");
			$aOpts->addChild("ul");
			$optBindings->addChild("aOpts");
		
			if ($this->me->canCreateUsers() || $this->me->canManageUsers())
			{
				$li = $aOpts->ul->addChild("li","Manage User Accounts");
				$li->addAttribute("class","aOpts");
				$optBindings->aOpts->addChild("opt",
					json_encode(array("bind"=>"click",
					"run"=>"MapChat.Using('Admin');MapChat.mcadmin.showManager('User');",)));
			}
			if ($this->me->canCreateGroups() || $this->me->canManageGroups())
			{
				$li = $aOpts->ul->addChild("li","Manage User Groups");
				$li->addAttribute("class","aOpts");
				$optBindings->aOpts->addChild("opt",
					json_encode(array("bind"=>"click",
					"run"=>"MapChat.Using('Admin');MapChat.mcadmin.showManager('Group');",)));
			}
			if ($this->me->canCreateDiscussions() || $this->me->canManageDiscussions())
			{
				$li = $aOpts->ul->addChild("li","Manage Discussions");
				$li->addAttribute("class","aOpts");
				$optBindings->aOpts->addChild("opt",
					json_encode(array("bind"=>"click",
					"run"=>"MapChat.Using('Admin');MapChat.mcadmin.showManager('Discussion');",)));
			}
			
			
		}
		
		return $this->_xml;
	}
}

?>