<?php

// A class for doing stuff with Arrays
class MapChat_Array
{

	/**
	 * Format an array for use in PostgreSQL queries.
	 *
	 * @param array $a - the array to convert
	 * @param boolean $q - if true, each array element is quoted as a string.
	 * @param boolean $z - if true, the result is returned as a new Zend_Db_Expr object
	 *
	 * @return string
	 */
	public function toPG($a,$q=true,$z=false)
	{
	
		if (count($a>0))
		{
			if (!is_array($a)) $a=array($a);
			if ($q) foreach ($a as $i => $v) $a[$i] = Zend_Registry::get("db")->quote($v);
			$s = "ARRAY[".implode(",",$a)."]";
		}
		
		else $s = "'{}'";
		
		if ($z) return new Zend_Db_Expr($s);
		else return $s;
		
	}
	
	/**
	 * Format an array for use in PostgreSQL queries as a list.
	 *
	 * @param array $a - the array to convert
	 * @param boolean $q - if true, each array element is quoted as a string.
	 * @param boolean $z - if true, the result is returned as a new Zend_Db_Expr object
	 *
	 * @return string
	 */
	public function toPGList($a,$q=true,$z=false)
	{
	
		if (count($a>0))
		{
			if (!is_array($a)) $a=array($a);
			if ($q) foreach ($a as $i => $v) $a[$i] = Zend_Registry::get("db")->quote($v);
			$s = "(".implode(",",$a).")";
		}
		
		else $s = "(null)";
		
		if ($z) return new Zend_Db_Expr($s);
		else return $s;
		
	}
	
	/**
	 * Format an array for use in PostgreSQL queries as a list of integers.
	 *
	 * @param array $a - the array to convert
	 * @param boolean $z - if true, the result is returned as a new Zend_Db_Expr object
	 *
	 * @return string
	 */
	public function toIntPG($a,$z=false)
	{
	
		if (count($a>0))
		{
			if (!is_array($a)) $a=array($a);
			foreach ($a as $i => $v) $a[$i] = (integer)$v;
			$s = "(".implode(",",$a).")";
		}
		
		else throw new MapChat_Exception("toIntPG: No input array elements to convert");
		
		if ($z) return new Zend_Db_Expr($s);
		else return $s;
		
	}
}

?>