<?php
/**
 * class MapChat_Discussion_Plan
 * Provides the general utilities for managing users' plans in the discussion:
 *
 * @category MapChat
 * @package MapChat
 */
class MapChat_Discussion_Plan_Irrigation
{
	
	/**
	 * Setup layers representing the specified plan
	 *
	 * @param integer $plan_id - the id for the user plan to setup
	 *
	 * @return array
	 */
	static function Setup($plan_id)
	{
		$d = Zend_Registry::get("discussion");
		$planInfo = $d->Plan_Info($plan_id);
		
		$aUserLayers = array(
		
			// Create a paddocks layer (Paddocks).
			"paddocks" => $d->Layer_Create("paddocks","Paddocks",3,array(
				"Crop Type"=>"text",
				"Annual Crop Yield"=>"text",
				"Size of Irrigated Area (ha)"=>"numeric",
				"Primary Irrigation Type"=>"text",
				"Additional Irrigation Type"=>"text",
				"Max. Rate (litres/sec.)"=>"numeric",
				"Normal Rate (litres/sec.)"=>"numeric",
				"Low Rate (litres/sec.)"=>"numeric",
				"Start of Irrigation"=>"text",
				"End of Irrigation"=>"text",
				"Frequency of Irrigation"=>"text",
				"Soil Type"=>"text",
				"Notes"=>"text",
			)),
			
			// Create a networks layer
			"networks" => $d->Layer_Create("networks","Networks",2,array(
				"Feature Type"=>"text",
				"Capacity (litres/sec.)"=>"numeric",
				"Notes"=>"text",
			)),
			
			// Create an equipment layer of a variable GEOMETRY type
			"equipment" => $d->Layer_Create("equipment","Equipment",4,array(
				"Feature Type"=>"text",
				"Radius of Centre Pivot (metres)"=>"text",
				"Number of Segments (C-Pivot) or Pods (K-Line)"=>"text",
				"Power Input/Output"=>"text",
				"Notes"=>"text",
			)),
			
			// Create a storage layer
			"storage" => $d->Layer_Create("storage","Storage Areas",2,array(
				"Volume (tonnes/cubic metres)"=>"text",
				"Purpose"=>"text",
				"Notes"=>"text",
			)),
			
			// Create a storage layer
			"takes" => $d->Layer_Create("takes","Water Takes",2,array(
				"Type of Permit"=>"text",
				"Maximum Rate (litres/sec.)"=>"numeric",
				"Maximum Daily (cubic metres)"=>"numeric",
				"Maximum Weekly (cubic metres)"=>"numeric",
				"Maximum Monthly (cubic metres)"=>"numeric",
				"Area of use (ha)"=>"numeric",
				"Purpose"=>"text",
				"Notes"=>"text",
			)),
			
			// Create an 'other' layer.
			"other" => $d->Layer_Create("other","Other Features",4,array(
				"Description"=>"text",
				"Notes"=>"text",
			)),
			
		);
		
		$d->Plan_linkToLayer($plan_id,$aUserLayers["paddocks"]);
		$d->Plan_linkToLayer($plan_id,$aUserLayers["networks"]);
		$d->Plan_linkToLayer($plan_id,$aUserLayers["equipment"]);
		$d->Plan_linkToLayer($plan_id,$aUserLayers["storage"]);
		$d->Plan_linkToLayer($plan_id,$aUserLayers["takes"]);
		$d->Plan_linkToLayer($plan_id,$aUserLayers["other"]);
		
		return $aUserLayers;
	}
	
	/**
	 * Add layers in an appropriate format in the mapfile
	 *
	 * @param integer $plan_id - the id for the user plan to setup
	 * @param string $mapfile - the name of the mapfile to add layers to.
	 *
	 * @return array
	 */
	static function AddMapLayers($plan_id,$mapfile)
	{
		$pg_conn=Zend_Registry::get("pg_conn");
		$db = Zend_Registry::get("db");
		$d = Zend_Registry::get("discussion");
		$aPlanInfo = $db->fetchRow($db->select()->from("user_plans","*",$d->name)->where("id = ?",$plan_id));
		
		$aNamedLayers = $d->Plan_LayerInfo($plan_id);
		
		$imageformat = $d->discussion_config->user_imageformat;
		$opacity = $d->discussion_config->user_opacity;
		
		// The returned map layer indexes.
		$aLayerIndexes = array();
		
		if ($imageformat == "GD_GIF")
		{
			$hatchStyle = array(
				array(
					"symbolname"=>"hatch",
					"size"=>6,
					"width"=>1,
					"angle"=>45,
				),
				array(
					"symbolname"=>"hatch",
					"size"=>6,
					"width"=>1,
					"angle"=>135,
				),
			);
		}
		else
		{
			$hatchStyle=array(array(),array());
		}
		
		$aOtherAttr = MapChat_Layer::UserLayerAttrInfo($aNamedLayers["other"]["id"]);
		$aOtherPoly = array(
			"type"=>MS_LAYER_POLYGON,
			"connectiontype"=>MS_POSTGIS,
			"connection"=>$pg_conn,
			"data"=>"geom from (select geom, id".$aOtherAttr["sql"]." from ".$d->name.".user_drawings where not deleted and st_geometrytype(geom) in ('ST_Polygon','ST_MultiPolygon') and layer_id = ".
				$aNamedLayers["other"]["id"]." order by id) as foo using unique id using srid=".$d->srid,
			"name"=>$aNamedLayers["other"]["description"]." - polygon (".$aPlanInfo["title"].")",
			"group"=>"USER_LAYERS",
			"status"=>MS_ON,
			"projection"=>"init=epsg:".$d->srid,
			
			"metadata"=>array(
				"mapchat_description"=>$aNamedLayers["other"]["description"]." - polygon",
				"mapchat_display_group"=>$aPlanInfo["title"],
				"mapchat_option_type"=>"checkbox",
				"mapchat_aliases"=>"Shape ID\t".$aOtherAttr["aliases"],
				"mapchat_fields"=>"id\t".$aOtherAttr["fields"],
				"mapchat_plan_type"=>"Irrigation",
				"mapchat_userlayer"=>"true",
				"mapchat_userlayer_id"=>$aNamedLayers["other"]["id"],
				"imageformat"=>$imageformat,
				"opacity"=>$opacity,
			),
			
			"classes"=>array(
				array(
					"name"=>$aNamedLayers["other"]["description"]." - polygon",
					"status"=>MS_ON,
					"styles"=>array(
						array_merge(array(
							"color"=>array(240,170,150),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(240,170,150),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(190,135,115),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
			),
			
		);
		
		$aLayerIndexes[$aOtherPoly["name"]] = $d->Map_addLayer($mapfile,$aOtherPoly);
		
		$aPaddocksAttr = MapChat_Layer::UserLayerAttrInfo($aNamedLayers["paddocks"]["id"]);
		$aPaddocks = array(
			"type"=>MS_LAYER_POLYGON,
			"connectiontype"=>MS_POSTGIS,
			"connection"=>$pg_conn,
			"data"=>"geom from (select geom, id".$aPaddocksAttr["sql"]." from ".$d->name.".user_drawings where not deleted and layer_id = ".
				$aNamedLayers["paddocks"]["id"]." order by id) as foo using unique id using srid=".$d->srid,
			"name"=>$aNamedLayers["paddocks"]["description"]." (".$aPlanInfo["title"].")",
			"group"=>"USER_LAYERS",
			"status"=>MS_ON,
			"projection"=>"init=epsg:".$d->srid,
			"classitem"=>MapChat_String::normalize("Primary Irrigation Type"),
			
			"metadata"=>array(
				"mapchat_description"=>$aNamedLayers["paddocks"]["description"],
				"mapchat_display_group"=>$aPlanInfo["title"],
				"mapchat_option_type"=>"checkbox",
				"mapchat_aliases"=>"Shape ID\t".$aPaddocksAttr["aliases"],
				"mapchat_fields"=>"id\t".$aPaddocksAttr["fields"],
				"mapchat_plan_type"=>"Irrigation",
				"mapchat_userlayer"=>"true",
				"mapchat_userlayer_id"=>$aNamedLayers["paddocks"]["id"],
				"imageformat"=>$imageformat,
				"opacity"=>$opacity,
			),
			
			"classes"=>array_reverse(array(
				array(
					"name"=>"Other",
					"status"=>MS_ON,
					"styles"=>array(
						array_merge(array(
							"color"=>array(255,125,80),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(255,125,80),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(205,100,60),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
				array(
					"name"=>"Contour Flood",
					"status"=>MS_ON,
					"expression"=>"Contour Flood",
					"styles"=>array(
						array_merge(array(
							"color"=>array(0,180,180),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(0,180,180),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(0,160,160),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
				array(
					"name"=>"Wild Flood",
					"status"=>MS_ON,
					"expression"=>"Wild Flood",
					"styles"=>array(
						array_merge(array(
							"color"=>array(0,0,255),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(0,0,255),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(0,0,200),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
				array(
					"name"=>"Border Dyke",
					"status"=>MS_ON,
					"expression"=>"Border Dyke",
					"styles"=>array(
						array_merge(array(
							"color"=>array(180,100,255),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(180,100,255),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(150,80,225),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
				array(
					"name"=>"Centre Pivot",
					"status"=>MS_ON,
					"expression"=>"Centre Pivot",
					"styles"=>array(
						array_merge(array(
							"color"=>array(0,255,255),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(0,255,255),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(0,200,200),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
				array(
					"name"=>"K-Line",
					"status"=>MS_ON,
					"expression"=>"K-Line",
					"styles"=>array(
						array_merge(array(
							"color"=>array(255,0,255),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(255,0,255),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(200,0,200),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
				array(
					"name"=>"Spray (stationary)",
					"status"=>MS_ON,
					"expression"=>"Spray (stationary)",
					"styles"=>array(
						array_merge(array(
							"color"=>array(0,255,0),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(0,255,0),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(0,200,0),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
				array(
					"name"=>"Spray (travelling)",
					"status"=>MS_ON,
					"expression"=>"Spray (travelling)",
					"styles"=>array(
						array_merge(array(
							"color"=>array(255,175,0),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(255,175,0),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(210,150,0),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
				array(
					"name"=>"None/Undefined",
					"status"=>MS_ON,
					"expression"=>"('[".MapChat_String::normalize("Primary Irrigation Type")."]' eq 'None' or length('[".MapChat_String::normalize("Primary Irrigation Type")."]') = 0 )",
					"styles"=>array(
						array_merge(array(
							"color"=>array(255,0,0),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(255,0,0),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(255,0,0),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
			)),
		);
		
		$aLayerIndexes[$aPaddocks["name"]] = $d->Map_addLayer($mapfile,$aPaddocks);
		
		
		$aStorageAttr = MapChat_Layer::UserLayerAttrInfo($aNamedLayers["storage"]["id"]);
		$aStorage = array(
			"type"=>MS_LAYER_POLYGON,
			"connectiontype"=>MS_POSTGIS,
			"connection"=>$pg_conn,
			"data"=>"geom from (select geom, id".$aStorageAttr["sql"]." from ".$d->name.".user_drawings where not deleted and layer_id = ".
				$aNamedLayers["storage"]["id"]." order by id) as foo using unique id using srid=".$d->srid,
			"name"=>$aNamedLayers["storage"]["description"]." (".$aPlanInfo["title"].")",
			"group"=>"USER_LAYERS",
			"status"=>MS_ON,
			"projection"=>"init=epsg:".$d->srid,
			
			"metadata"=>array(
				"mapchat_description"=>$aNamedLayers["storage"]["description"],
				"mapchat_display_group"=>$aPlanInfo["title"],
				"mapchat_option_type"=>"checkbox",
				"mapchat_aliases"=>"Shape ID\t".$aStorageAttr["aliases"],
				"mapchat_fields"=>"id\t".$aStorageAttr["fields"],
				"mapchat_plan_type"=>"Irrigation",
				"mapchat_userlayer"=>"true",
				"mapchat_userlayer_id"=>$aNamedLayers["storage"]["id"],
				"imageformat"=>$imageformat,
				"opacity"=>$opacity,
			),
			
			"classes"=>array(
				array(
					"name"=>"Storage",
					"status"=>MS_ON,
					"styles"=>array(
						array_merge(array(
							"color"=>array(0,0,200),
							"opacity"=>30,
						),$hatchStyle[0]),
						array_merge(array(
							"color"=>array(0,0,200),
							"opacity"=>30,
						),$hatchStyle[1]),
						array(
							"outlinecolor"=>array(0,0,180),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
			),
			
		);
		
		$aLayerIndexes[$aStorage["name"]] = $d->Map_addLayer($mapfile,$aStorage);
		
		
		$aOtherLine = array(
			"type"=>MS_LAYER_LINE,
			"connectiontype"=>MS_POSTGIS,
			"connection"=>$pg_conn,
			"data"=>"geom from (select geom, id".$aOtherAttr["sql"]." from ".$d->name.".user_drawings where not deleted and st_geometrytype(geom) in ('ST_LineString','ST_MultiLineString') and layer_id = ".
				$aNamedLayers["other"]["id"]." order by id) as foo using unique id using srid=".$d->srid,
			"name"=>$aNamedLayers["other"]["description"]." - line (".$aPlanInfo["title"].")",
			"group"=>"USER_LAYERS",
			"status"=>MS_ON,
			"projection"=>"init=epsg:".$d->srid,
			
			"metadata"=>array(
				"mapchat_description"=>$aNamedLayers["other"]["description"]." - line",
				"mapchat_display_group"=>$aPlanInfo["title"],
				"mapchat_option_type"=>"checkbox",
				"mapchat_aliases"=>"Shape ID\t".$aOtherAttr["aliases"],
				"mapchat_fields"=>"id\t".$aOtherAttr["fields"],
				"mapchat_plan_type"=>"Irrigation",
				"mapchat_userlayer"=>"true",
				"mapchat_userlayer_id"=>$aNamedLayers["other"]["id"],
				"imageformat"=>$imageformat,
				"opacity"=>$opacity,
			),
			
			"classes"=>array(
				array(
					"name"=>$aNamedLayers["other"]["description"]." - line",
					"status"=>MS_ON,
					"styles"=>array(
						array(
							"color"=>array(240,170,150),
							"width"=>2,
							"opacity"=>70,
						),
					),
				),
			),
			
		);
		
		$aLayerIndexes[$aOtherLine["name"]] = $d->Map_addLayer($mapfile,$aOtherLine);
		
		
		$aNetworksAttr = MapChat_Layer::UserLayerAttrInfo($aNamedLayers["networks"]["id"]);
		$aNetworks = array(
			"type"=>MS_LAYER_LINE,
			"connectiontype"=>MS_POSTGIS,
			"connection"=>$pg_conn,
			"data"=>"geom from (select geom, id".$aNetworksAttr["sql"]." from ".$d->name.".user_drawings where not deleted and layer_id = ".
				$aNamedLayers["networks"]["id"]." order by id) as foo using unique id using srid=".$d->srid,
			"name"=>$aNamedLayers["networks"]["description"]." (".$aPlanInfo["title"].")",
			"group"=>"USER_LAYERS",
			"status"=>MS_ON,
			"projection"=>"init=epsg:".$d->srid,
			"classitem"=>MapChat_String::normalize("Feature Type"),
			
			"metadata"=>array(
				"mapchat_description"=>$aNamedLayers["networks"]["description"],
				"mapchat_display_group"=>$aPlanInfo["title"],
				"mapchat_option_type"=>"checkbox",
				"mapchat_aliases"=>"Shape ID\t".$aNetworksAttr["aliases"],
				"mapchat_fields"=>"id\t".$aNetworksAttr["fields"],
				"mapchat_plan_type"=>"Irrigation",
				"mapchat_userlayer"=>"true",
				"mapchat_userlayer_id"=>$aNamedLayers["networks"]["id"],
				"imageformat"=>$d->discussion_config->user_imageformat,
				"opacity"=>$d->discussion_config->user_opacity,
			),
			
			"classes"=>array_reverse(array(
				array(
					"name"=>"Other",
					"status"=>MS_ON,
					"styles"=>array(
						array(
							"color"=>array(255,145,15),
							"width"=>2,
						),
					),
				),
				array(
					"name"=>"Races",
					"status"=>MS_ON,
					"expression"=>"Race",
					"styles"=>array(
						array(
							"color"=>array(85,165,240),
							"width"=>2,
						),
					),
				),
				array(
					"name"=>"Creeks/Streams",
					"status"=>MS_ON,
					"expression"=>"Creek/Stream",
					"styles"=>array(
						array(
							"color"=>array(0,0,255),
							"width"=>2,
						),
					),
				),
				array(
					"name"=>"Pipelines",
					"status"=>MS_ON,
					"expression"=>"Pipeline",
					"styles"=>array(
						array(
							"color"=>array(0,50,200),
							"width"=>2,
						),
					),
				),
				array(
					"name"=>"None/Undefined",
					"status"=>MS_ON,
					"expression"=>"('[".MapChat_String::normalize("Feature Type")."]' eq 'None' or length('[".MapChat_String::normalize("Feature Type")."]') = 0)",
					"styles"=>array(
						array(
							"color"=>array(0,0,0),
							"width"=>2,
						),
					),
				),
			)),
		);
		
		$aLayerIndexes[$aNetworks["name"]] = $d->Map_addLayer($mapfile,$aNetworks);
		
		
		$aOtherPoint = array(
			"type"=>MS_LAYER_POINT,
			"connectiontype"=>MS_POSTGIS,
			"connection"=>$pg_conn,
			"data"=>"geom from (select geom, id".$aOtherAttr["sql"]." from ".$d->name.".user_drawings where not deleted and st_geometrytype(geom) in ('ST_Point','ST_MultiPoint') and layer_id = ".
				$aNamedLayers["other"]["id"]." order by id) as foo using unique id using srid=".$d->srid,
			"name"=>$aNamedLayers["other"]["description"]." - point (".$aPlanInfo["title"].")",
			"group"=>"USER_LAYERS",
			"status"=>MS_ON,
			"projection"=>"init=epsg:".$d->srid,
			
			"metadata"=>array(
				"mapchat_description"=>$aNamedLayers["other"]["description"]." - point",
				"mapchat_display_group"=>$aPlanInfo["title"],
				"mapchat_option_type"=>"checkbox",
				"mapchat_aliases"=>"Shape ID\t".$aOtherAttr["aliases"],
				"mapchat_fields"=>"id\t".$aOtherAttr["fields"],
				"mapchat_plan_type"=>"Irrigation",
				"mapchat_userlayer"=>"true",
				"mapchat_userlayer_id"=>$aNamedLayers["other"]["id"],
				"imageformat"=>$imageformat,
				"opacity"=>$opacity,
			),
			
			"classes"=>array(
				array(
					"name"=>$aNamedLayers["other"]["description"]." - point",
					"status"=>MS_ON,
					"styles"=>array(
						array(
							"symbolname"=>"circle",
							"color"=>array(240,170,150),
							"outlinecolor"=>array(0,0,0),
							"size"=>7,
						),
					),
				),
			),
			
		);
		
		$aLayerIndexes[$aOtherPoint["name"]] = $d->Map_addLayer($mapfile,$aOtherPoint);
		
		
		$aEquipmentAttr = MapChat_Layer::UserLayerAttrInfo($aNamedLayers["equipment"]["id"]);
		$aEquipment = array(
			"type"=>MS_LAYER_POINT,
			"connectiontype"=>MS_POSTGIS,
			"connection"=>$pg_conn,
			"data"=>"geom from (select geom, id".$aEquipmentAttr["sql"]." from ".$d->name.".user_drawings where not deleted and layer_id = ".
				$aNamedLayers["equipment"]["id"]." order by id) as foo using unique id using srid=".$d->srid,
			"name"=>$aNamedLayers["equipment"]["description"]." (".$aPlanInfo["title"].")",
			"group"=>"USER_LAYERS",
			"status"=>MS_ON,
			"projection"=>"init=epsg:".$d->srid,
			"classitem"=>MapChat_String::normalize("Feature Type"),
			
			"metadata"=>array(
				"mapchat_description"=>$aNamedLayers["equipment"]["description"],
				"mapchat_display_group"=>$aPlanInfo["title"],
				"mapchat_option_type"=>"checkbox",
				"mapchat_aliases"=>"Shape ID\t".$aEquipmentAttr["aliases"],
				"mapchat_fields"=>"id\t".$aEquipmentAttr["fields"],
				"mapchat_plan_type"=>"Irrigation",
				"mapchat_userlayer"=>"true",
				"mapchat_userlayer_id"=>$aNamedLayers["equipment"]["id"],
				"imageformat"=>$imageformat,
				"opacity"=>$opacity,
			),
			
			"classes"=>array_reverse(array(
				array(
					"name"=>"Other",
					"status"=>MS_ON,
					"styles"=>array(
						array(
							"symbolname"=>"circle",
							"color"=>array(255,145,15),
							"outlinecolor"=>array(0,0,0),
							"size"=>7,
						),
					),
				),
				array(
					"name"=>"Centre Pivots",
					"status"=>MS_ON,
					"expression"=>"Centre Pivot",
					"styles"=>array(
						array(
							"symbolname"=>"circle",
							"color"=>array(160,70,255),
							"outlinecolor"=>array(0,0,0),
							"size"=>7,
						),
					),
				),
				array(
					"name"=>"K-Lines",
					"status"=>MS_ON,
					"expression"=>"K-Line",
					"styles"=>array(
						array(
							"symbolname"=>"circle",
							"color"=>array(70,255,165),
							"outlinecolor"=>array(0,0,0),
							"size"=>7,
						),
					),
				),
				array(
					"name"=>"Pumps",
					"status"=>MS_ON,
					"expression"=>"Pump",
					"styles"=>array(
						array(
							"symbolname"=>"circle",
							"color"=>array(100,100,100),
							"outlinecolor"=>array(0,0,0),
							"size"=>7,
						),
					),
				),
				array(
					"name"=>"Generators",
					"status"=>MS_ON,
					"expression"=>"Generator",
					"styles"=>array(
						array(
							"symbolname"=>"circle",
							"color"=>array(128,128,0),
							"outlinecolor"=>array(0,0,0),
							"size"=>7,
						),
					),
				),
				array(
					"name"=>"Wiers",
					"status"=>MS_ON,
					"expression"=>"Wier",
					"styles"=>array(
						array(
							"symbolname"=>"circle",
							"color"=>array(192,255,255),
							"outlinecolor"=>array(0,0,0),
							"size"=>7,
						),
					),
				),
				array(
					"name"=>"None/Undefined",
					"status"=>MS_ON,
					"expression"=>"('[".MapChat_String::normalize("Feature Type")."]' eq 'None' or length('[".MapChat_String::normalize("Feature Type")."]') = 0)",
					"styles"=>array(
						array(
							"symbolname"=>"circle",
							"color"=>array(0,0,0),
							"outlinecolor"=>array(0,0,0),
							"size"=>7,
						),
					),
				),
			)),
		);
		
		$aLayerIndexes[$aEquipment["name"]] = $d->Map_addLayer($mapfile,$aEquipment);
		
		$aTakesAttr = MapChat_Layer::UserLayerAttrInfo($aNamedLayers["takes"]["id"]);
		$aTakes = array(
			"type"=>MS_LAYER_POINT,
			"connectiontype"=>MS_POSTGIS,
			"connection"=>$pg_conn,
			"data"=>"geom from (select geom, id".$aTakesAttr["sql"]." from ".$d->name.".user_drawings where not deleted and layer_id = ".
				$aNamedLayers["takes"]["id"]." order by id) as foo using unique id using srid=".$d->srid,
			"name"=>$aNamedLayers["takes"]["description"]." (".$aPlanInfo["title"].")",
			"group"=>"USER_LAYERS",
			"status"=>MS_ON,
			"projection"=>"init=epsg:".$d->srid,
			
			"metadata"=>array(
				"mapchat_description"=>$aNamedLayers["takes"]["description"],
				"mapchat_display_group"=>$aPlanInfo["title"],
				"mapchat_option_type"=>"checkbox",
				"mapchat_aliases"=>"Shape ID\t".$aTakesAttr["aliases"],
				"mapchat_fields"=>"id\t".$aTakesAttr["fields"],
				"mapchat_plan_type"=>"Irrigation",
				"mapchat_userlayer"=>"true",
				"mapchat_userlayer_id"=>$aNamedLayers["takes"]["id"],
				"imageformat"=>$imageformat,
				"opacity"=>$opacity,
			),
			
			"classes"=>array_reverse(array(
				array(
					"name"=>"Water Takes",
					"status"=>MS_ON,
					"styles"=>array(
						array(
							"symbolname"=>"water_drop",
							"size"=>12,
						),
					),
				),
			)),
		);
		
		$aLayerIndexes[$aTakes["name"]] = $d->Map_addLayer($mapfile,$aTakes);
		
		return $aLayerIndexes;
		
	}
}

?>