<?php
/**
 * class MapChat_Discussion_User
 * Provides information about the users in a given discussion.
 *
 * @category MapChat
 * @package MapChat
 */
class MapChat_Discussion_User
{
	/**
	 * Get the MapChat config/session/discussion environment, and
	 * attempt to load the most recent maps from the database.
	 */
	public function __construct($discussion)
	{
		$this->db = Zend_Registry::get("db");
		$this->config = Zend_Registry::get("config");
		$this->discussion = $discussion;
		$this->me = Zend_Registry::get("me");
	}
	
	/**
	 * Return a list users, with their online status, display state and display colour
	 *
	 * @param string $mapfile - the mapfile to use to determine users' drawing display states/colors
	 */
	public function ListAll($mapfile)
	{
		$oMap = $this->discussion->Map_getMapObj($mapfile);
		
		$aUserPoly = array();
		$oUserPoly = $oMap->getLayerByName("User Drawings (polygon)");
		for ($i=0;$i<$oUserPoly->numclasses-1;$i++)
		{
			$oClass = $oUserPoly->getClass($i);
			$aUserPoly[substr($oClass->getExpressionString(),1,-1)] = $oClass;
		}
		
		$aUserLine = array();
		$oUserLine = $oMap->getLayerByName("User Drawings (line)");
		for ($i=0;$i<$oUserPoly->numclasses-1;$i++)
		{
			$oClass = $oUserLine->getClass($i);
			$aUserLine[substr($oClass->getExpressionString(),1,-1)] = $oClass;
		}
		
		$aUserPoint = array();
		$oUserPoint = $oMap->getLayerByName("User Drawings (point)");
		for ($i=0;$i<$oUserPoly->numclasses-1;$i++)
		{
			$oClass = $oUserPoint->getClass($i);
			$aUserPoint[substr($oClass->getExpressionString(),1,-1)] = $oClass;
		}
		
		$aUsers = $this->discussion->listUsers(true,NULL,true,true);
		$aOnlineUsers = $this->discussion->getOnlineUsers(false,true,false,true);
		
		foreach ($aUsers as $i => $u)
		{
			$aUsers[$i]["display_name"] = ((isset($aUsers[$i]["full_name"])&&strlen($aUsers[$i]["full_name"])>0)?$aUsers[$i]["full_name"]:$aUsers[$i]["username"]);
			$aUsers[$i]["online"] = in_array($u["username"],$aOnlineUsers);
			$aUsers[$i]["show_drawings"] = (isset($aUserPoly[$u["username"]]) && $aUserPoly[$u["username"]]->status==MS_ON &&
				isset($aUserLine[$u["username"]]) && $aUserLine[$u["username"]]->status==MS_ON &&
				isset($aUserPoint[$u["username"]]) && $aUserPoint[$u["username"]]->status==MS_ON);
			$aUsers[$i]["draw_color"] = 
				(isset($aUserPoly[$u["username"]])?
					MapChat_Color::rgb2hex($aUserPoly[$u["username"]]->getStyle(0)->color->red,
						$aUserPoly[$u["username"]]->getStyle(0)->color->green,
						$aUserPoly[$u["username"]]->getStyle(0)->color->blue):
					MapChat_Color::makeColor($u["username"]));
		}
		
		// Add 'all users', which should be the last class in any of the users layers:
		$oAUClassPoly = $oUserPoly->getClass($oUserPoly->numclasses-1);
		$oAUClassLine = $oUserLine->getClass($oUserPoly->numclasses-1);
		$oAUClassPoint = $oUserPoint->getClass($oUserPoly->numclasses-1);
		array_unshift($aUsers,array("all_users"=>true,
			"show_drawings"=>($oAUClassPoly->status==MS_ON && $oAUClassPoly->status==MS_ON && $oAUClassLine->status==MS_ON),
			"draw_color"=>MapChat_Color::rgb2hex($oAUClassPoly->getStyle(0)->color->red,
				$oAUClassPoly->getStyle(0)->color->green,
				$oAUClassPoly->getStyle(0)->color->blue)
		));
		
		return $aUsers;
		
	}
	
	
	/**
	 * Return a list of users who have specific visbility state in the current usersonline status, display state and display colour
	 *
	 * @param string $mapfile - the mapfile to use to determine users' drawing display states/colors
	 */
	public function ListVisible($mapfile)
	{
		$oMap = $this->discussion->Map_getMapObj($mapfile);
		
		// Assuming the users' drawing states are the same for all three point/line/polygon 
		// layers, we'll just list whoever is 'on' in the polygon layer.
		$aUsers = array();
		$bAllUsers = false;
		$bMyDrawings = false;
		$oUserPoly = $oMap->getLayerByName("User Drawings (polygon)");
		for ($i=0;$i<$oUserPoly->numclasses;$i++)
		{
			$oClass = $oUserPoly->getClass($i);
			if ($oClass->status==MS_ON)
			{
				$username = substr($oClass->getExpressionString(),1,-1);
				if ($username == $this->me->username) $bMyDrawings = true;
				else if (strlen($username)>0) $aUsers[] = $username;
				else $bAllUsers = true; // All Users has no value for the expression.
			}
		}
		
		return array($bMyDrawings,$aUsers,$bAllUsers);
		
	}
	
	
	/**
	 * Sets the states/colors of classes for user-drawn shapes in the map
	 *
	 * @param string $mapfile - the mapfile to use to determine users' drawing display states/colors
	 * @param array $ustates - the states/colors of specific users in the map
	 */
	public function SetStates($mapfile,$ustates)
	{
		$oMap = $this->discussion->Map_getMapObj($mapfile);
		
		$aUsers = $this->discussion->listUsers(true,NULL,true,true);
		$aUNamed = array();
		foreach ($aUsers as $i => $u)
		{
			if ($u["username"] == $this->me->username) unset($aUsers[$i]);
			else
			{
				$aUsers[$i]["display_name"] = 
					((isset($u["full_name"])&&strlen($u["full_name"])>0)?$u["full_name"]:$u["username"]);
				$aUNamed[$u["username"]] = $aUsers[$i];
			}
		}
		
		// This will save a list of all usernames that are currently displayed:
		//$aNotAll = array($this->me->username);
		
		// Get arrays of classes for the user drawings layer organized by 
		$aUserPoly = array();
		$aUserPolyI = array();
		$oUserPoly = $oMap->getLayerByName("User Drawings (polygon)");
		for ($i=0;$i<$oUserPoly->numclasses;$i++)
		{
			$oClass = $oUserPoly->getClass($i);
			$aUserPoly[substr($oClass->getExpressionString(),1,-1)] = $oClass;
			$aUserPolyI[substr($oClass->getExpressionString(),1,-1)] = $i;
			//if ($oClass->status==MS_ON && $i>0) $aNotAll[] = substr($oClass->getExpressionString(),1,-1);
		}
		
		$aUserLine = array();
		$aUserLineI = array();
		$oUserLine = $oMap->getLayerByName("User Drawings (line)");
		for ($i=0;$i<$oUserPoly->numclasses;$i++)
		{
			$oClass = $oUserLine->getClass($i);
			$aUserLine[substr($oClass->getExpressionString(),1,-1)] = $oClass;
			$aUserLineI[substr($oClass->getExpressionString(),1,-1)] = $i;
		}
		
		$aUserPoint = array();
		$aUserPointI = array();
		$oUserPoint = $oMap->getLayerByName("User Drawings (point)");
		for ($i=0;$i<$oUserPoly->numclasses;$i++)
		{
			$oClass = $oUserPoint->getClass($i);
			$aUserPoint[substr($oClass->getExpressionString(),1,-1)] = $oClass;
			$aUserPointI[substr($oClass->getExpressionString(),1,-1)] = $i;
		}
		
		foreach ($ustates as $utype => $uinfo)
		{
			switch ($utype)
			{
				// The current user ('me') should be the *first* class in the user drawings layers:
				case "me":
					$status = ($uinfo["show"]?MS_ON:MS_OFF);
					$oUserPoly->getClass(0)->set("status",$status);
					$oUserLine->getClass(0)->set("status",$status);
					$oUserPoint->getClass(0)->set("status",$status);
					$color = MapChat_Color::hex2rgb($uinfo["color"]);
					$oUserPoly->getClass(0)->getStyle(0)->color->setRGB($color[0],$color[1],$color[2]);
					$oUserPoly->getClass(0)->getStyle(1)->color->setRGB($color[0],$color[1],$color[2]);
					$oUserPoly->getClass(0)->getStyle(2)->outlinecolor->setRGB($color[0],$color[1],$color[2]);
					$oUserLine->getClass(0)->getStyle(0)->color->setRGB($color[0],$color[1],$color[2]);
					$oUserPoint->getClass(0)->getStyle(0)->color->setRGB($color[0],$color[1],$color[2]);
					
				break;
				
				// All users should be the *last* class in the user drawings layers - since it's the last class, it
				// doesn't need to be deleted when it is turned off:
				case "all":
					$status = ($uinfo["show"]?MS_ON:MS_OFF);
					$oUserPoly->getClass($oUserPoly->numclasses-1)->set("status",$status);
					$oUserLine->getClass($oUserPoly->numclasses-1)->set("status",$status);
					$oUserPoint->getClass($oUserPoly->numclasses-1)->set("status",$status);
					$color = MapChat_Color::hex2rgb($uinfo["color"]);
					$oUserPoly->getClass($oUserPoly->numclasses-1)->getStyle(0)->color->setRGB($color[0],$color[1],$color[2]);
					$oUserPoly->getClass($oUserPoly->numclasses-1)->getStyle(1)->color->setRGB($color[0],$color[1],$color[2]);
					$oUserPoly->getClass($oUserPoly->numclasses-1)->getStyle(2)->outlinecolor->setRGB($color[0],$color[1],$color[2]);
					$oUserLine->getClass($oUserPoly->numclasses-1)->getStyle(0)->color->setRGB($color[0],$color[1],$color[2]);
					$oUserPoint->getClass($oUserPoly->numclasses-1)->getStyle(0)->color->setRGB($color[0],$color[1],$color[2]);
				break;
				
				default:
					
					$imageformat = $this->discussion->discussion_config->user_imageformat;
					$opacity = $this->discussion->discussion_config->user_opacity;
					
					foreach ($uinfo as $u)
					{
						$status = ($u["show"]?MS_ON:MS_OFF);
						$color = MapChat_Color::hex2rgb($u["color"]);
						//if ($u["show"]) $aNotAll[] = $u["username"];
						
						// Remove the user's display classes if not showing them:
						if (!$u["show"])
						{
							if (isset($aUserPoly[$u["username"]])) $aUserPoly[$u["username"]]->set("status",MS_DELETE);
							if (isset($aUserLine[$u["username"]])) $aUserLine[$u["username"]]->set("status",MS_DELETE);
							if (isset($aUserPoint[$u["username"]])) $aUserPoint[$u["username"]]->set("status",MS_DELETE);
						}
						
						// Add the users display classes if they don't exist, right before the last one...
						else if ($u["show"])
						{
							if (isset($aUserPoly[$u["username"]]))
							{
								$aUserPoly[$u["username"]]->set("status",$status);
								$aUserPoly[$u["username"]]->getStyle(0)->color->setRGB($color[0],$color[1],$color[2]);
								$aUserPoly[$u["username"]]->getStyle(1)->color->setRGB($color[0],$color[1],$color[2]);
								$aUserPoly[$u["username"]]->getStyle(2)->outlinecolor->setRGB($color[0],$color[1],$color[2]);
							}
							else
							{
								$oClass = ms_newClassObj($oUserPoly);
								$oClass->set("name","Drawings by ".$aUNamed[$u["username"]]["display_name"]);
								$oClass->setExpression($u["username"]);
								$oClass->set("status",MS_ON);
								
								$oStyle = ms_newStyleObj($oClass);
								$oStyle->color->setRGB($color[0],$color[1],$color[2]);
								$oStyle->set("opacity",15);
								if ($imageformat == "GD_GIF")
								{
									$oStyle->set("symbolname","hatch");
									$oStyle->set("width",1);
									$oStyle->set("size",9);
									$oStyle->set("angle",45);
								}
								
								$oStyle = ms_newStyleObj($oClass);
								$oStyle->color->setRGB($color[0],$color[1],$color[2]);
								$oStyle->set("opacity",15);
								if ($imageformat == "GD_GIF")
								{
									$oStyle->set("symbolname","hatch");
									$oStyle->set("width",1);
									$oStyle->set("size",9);
									$oStyle->set("angle",135);
								}
								
								$oStyle = ms_newStyleObj($oClass);
								$oStyle->outlinecolor->setRGB($color[0],$color[1],$color[2]);
								$oStyle->set("opacity",70);
								$oStyle->set("width",2);
								
								// move the class up so it's listed above all user's drawings:
								$oUserPoly->moveclassup($oUserPoly->numclasses-1);
								
							}
							
							if (isset($aUserLine[$u["username"]]))
							{
								$aUserLine[$u["username"]]->set("status",$status);
								$aUserLine[$u["username"]]->getStyle(0)->color->setRGB($color[0],$color[1],$color[2]);
							}
							else
							{
								$oClass = ms_newClassObj($oUserLine);
								$oClass->set("name","Drawings by ".$aUNamed[$u["username"]]["display_name"]);
								$oClass->setExpression($u["username"]);
								$oClass->set("status",MS_ON);
								
								$oStyle = ms_newStyleObj($oClass);
								$oStyle->set("width",2);
								$oStyle->set("opacity",70);
								$oStyle->color->setRGB($color[0],$color[1],$color[2]);
								
								// move the class up so it's listed above all user's drawings:
								$oUserLine->moveclassup($oUserLine->numclasses-1);
								
							}
							
							
							if (isset($aUserPoint[$u["username"]]))
							{
								$aUserPoint[$u["username"]]->set("status",$status);
								$aUserPoint[$u["username"]]->getStyle(0)->color->setRGB($color[0],$color[1],$color[2]);
							}
							else
							{
								$oClass = ms_newClassObj($oUserPoint);
								$oClass->set("name","Drawings by ".$aUNamed[$u["username"]]["display_name"]);
								$oClass->setExpression($u["username"]);
								$oClass->set("status",MS_ON);
								
								$oStyle = ms_newStyleObj($oClass);
								$oStyle->set("symbolname","circle");
								$oStyle->set("opacity",70);
								$oStyle->color->setRGB($color[0],$color[1],$color[2]);
								$oStyle->outlinecolor->setRGB(0,0,0);
								$oStyle->set("size",9);
								
								// move the class up so it's listed above all user's drawings:
								$oUserPoint->moveclassup($oUserPoint->numclasses-1);
								
							}
						}
					}
				break;
			}
		}
		
		
// 		// Fix the data string to flag shapes that are not part of the 'all_users' class:
// 		if (count($aNotAll)==0) $sNotAll = "true::varchar as all_users, ";
// 		else $sNotAll = "(username not in ".MapChat_Array::toPGList($aNotAll,true).")::varchar as all_users, ";
// 		
// 		$oUserPoly->set("data","geom from (select ".$sNotAll.
// 			"geom, id, username, case when length(full_name)>0 then full_name else username end as display_name, ".
// 			"attributes[1] as label from ".$this->discussion->name.".user_drawings ud join users u using (username) where ".
// 			"not deleted and visible and st_geometrytype(geom) in ('ST_Polygon','ST_MultiPolygon') and ".
// 			"layer_id is null order by id) as foo using unique id using srid=".$this->discussion->srid);
// 		
// 		$oUserLine->set("data","geom from (select ".$sNotAll.
// 			"geom, id, username, case when length(full_name)>0 then full_name else username end as display_name, ".
// 			"attributes[1] as label from ".$this->discussion->name.".user_drawings ud join users u using (username) where ".
// 			"not deleted and visible and st_geometrytype(geom) in ('ST_LineString','ST_MultiLineString') and ".
// 			"layer_id is null order by id) as foo using unique id using srid=".$this->discussion->srid);
// 		
// 		$oUserPoint->set("data","geom from (select ".$sNotAll.
// 			"geom, id, username, case when length(full_name)>0 then full_name else username end as display_name, ".
// 			"attributes[1] as label from ".$this->discussion->name.".user_drawings ud join users u using (username) where ".
// 			"not deleted and visible and st_geometrytype(geom) in ('ST_Point','ST_MultiPoint') and ".
// 			"layer_id is null order by id) as foo using unique id using srid=".$this->discussion->srid);
		
		$this->discussion->Map_saveMapfile($mapfile,$oMap);
		
		return true;
		
	}
	
	
}

?>