<?php

class MapChat_Loader
{
	/* The Full Class Name of the Class We are Gonna Load. */
	protected $_class = NULL;

	/* The Name of the function we will execute */
	protected $_function = NULL;

	/* The Instance of the Class we will be reflecting */
	protected $_object = NULL;

	/* Our Constructor */
	public function __construct()
	{
		// Load our config from the Registry
		$this->config = Zend_Registry::get("config");
	}

	public function LoadClass($class, $args)
	{
		// Generate the Full Class Name
		$this->_class = "MapChat_API_".$class;
		// Strip out the Actual Function Name we want to Invoke
		$this->_function = substr($this->_class, (strrpos($this->_class, "_") + 1));

		// Locate the file we want to load
		$dir = str_replace("_", "/", $this->_class);
		$fullPath = $this->config->LibDir."/".$dir.".php";

		if(!file_exists($fullPath))
		{
			throw new MapChat_Exception('Failed to Load API Module '.$class.', Module Doesnt Exist.');
		}

		try
		{
			// Setup a new reflector of the class.
			$r = new ReflectionClass($this->_class);
		}
		catch(Exception $e)
		{
			// We couldnt reflect this class, throw a sensible error
			throw new MapChat_Exception('Failed to Load API Module '.$class.'.');
		}

		try
		{
			// Reflect the Construct and Function Methods in the Class.
			$classConstructor = $r->getMethod("__construct");
			$classFunction = $r->getMethod($this->_function);
		}
		catch(Exception $e)
		{
			throw new MapChat_Exception('Failed to Load Module '.$class.', Invalid Module, Doest not Fit Schema.');
		}

		// Make sure the constructor and function we want to call are public and callable
		if($classConstructor->isPublic() === false || $classFunction->isPublic() === false)
		{
			throw new MapChat_Exception('Failed to Load Module '.$class.', Invalid Module, Methods not Callable');
		}

		try
		{
			$this->_object = $r->newInstance();
		}
		catch(Exception $e)
		{
			throw new Exception("Failed to Invoke Method");
		}
		
		$params = $classFunction->getParameters();
		
		// If the first argument has a numeric key, we'll assume the whole list of
		// args in the correct order for calling the function:
		if (is_numeric(key($args)))
		{
			$argsAssoc = false;
			// Count the number of mandatory parameters in the function:
			$pcount = 0;
			foreach ($params as $param)
			{
				if (!$param->isOptional())
				{
					$pcount++;
				}
			}
		}
		else
		{
			$argsAssoc = true;
			$argsSorted = array();
			// For an associative array of args, lets verify that they are in the right
			// order, and no mandatory parameters are missing:
			foreach ($params as $i => $param)
			{
				if (isset($args[$param->getName()]))
				{
					$argsSorted[$i]=$args[$param->getName()];
				}
				else
				{
					if (!$param->isOptional())
					{
						throw new MapChat_Exception("Missing parameter '".$param->getName()."' in the submitted arguments.");
					}
					else
					{
						$argsSorted[$i] = $param->getDefaultValue();
					}
				}
			}
			$args = $argsSorted;
		}
		
		
		// If the number of arguments supplied is enough to satisfy
		// any non-optional arguments, and is not greater than the total
		// number of optional+non-optional arguments, it should be fine
		// to run the function:
		if($argsAssoc || (count($args) >= $pcount && count($args) <= count($params)))
		{
			if(count($params) > 0)
			{
				return $classFunction->invokeArgs($this->_object, $args);
			}
			else
			{
				return $classFunction->invoke($this->_object);
			}
		}
		else
		{
			// Invoke method with no args.
			throw new Exception("Invalid Number of Params Specified");
		}
	}
}

?>