<?php
/**
 * Just a class for doing common time-related stuff - the Zend_Date
 * class doesn't seem to grab milliseconds on its own, so we'll
 * feed it an accurate timestamp ourselves, and use that as needed.
 *
 * @category MapChat
 * @package MapChat
 */
class MapChat_Time
{
	/**
	 * Return a Zend_Date object with accurate timestamp information:
	 *
	 * @param string $time if specified, this will be treated as
	 *   the output from the php microtime() function without
	 *   the optional argument to round microseconds and add them
	 *   as an decimals to the timestamp.
	 */
	public function time($time=false)
	{
		if ($time===false)
		{
			$time = microtime();
		}
		$aTime = explode(" ",$time);
		$date = new Zend_Date();
		$date->setTimestamp($aTime[1]);
		$date->setMillisecond($aTime[0]);
		return $date;
	}
	
	/**
	 * Return a real numerical UNIX timestamp (with decimals for milliseconds);
	 *
	 * @param Zend_Date $date if provided, the $date argument will
	 *   be used to create the timestamp - otherwise, now() is used
	 *   to get a Zend_Date with the current timestamp.  If $date
	 *   is provided, it is assumed to have microseconds set, and
	 *   a fractional precision of 6 decimal places.
	 */
	public function stamp(Zend_Date $date=NULL)
	{
		if (is_null($date))
		{
			$date = MapChat_Time::time();
		}
		return sprintf("%.6F",$date->getTimestamp() + $date->getMillisecond());
	}
}

?>