<?php

class MapChat_User
{

	private $_aUserInfo = NULL;
	private $_bIsAdmin = NULL;
	private $_bExists = false;
	private $_aPermissions = NULL;
	private $_db = NULL;
	private $_config = NULL;
	
	/*
	 * Identify the properties required to restore this object
	 * (i.e., the properties that must be saved to the session).
	 */
	public function __sleep()
	{
		return array("_aUserInfo","_bIsAdmin","_bExists","_aPermissions");
	}
	
	/*
	 * Identify the properties that must be reinitialized when this
	 * object is restored from a saved session:
	 */
	public function __wakeup()
	{
		$this->_db = Zend_Registry::get("db");
		$this->_config = Zend_Registry::get("config");
	}
	
	/*
	 * Allow this object to be cast as a string containing the username.
	 */
	public function __toString() 
	{
		if (isset($this->_aUserInfo["username"]))
		{
			return $this->_aUserInfo["username"];
		}
		else
		{
			return "";
		}
	}
	
	/*
	 * Allow the user properties accessed as object members.
	 */
	public function __get($n) 
	{
		if (isset($this->_aUserInfo[$n]))
		{
			return $this->_aUserInfo[$n];
		}
		else
		{
			return NULL;
		}
	}
	
	
//   // Disabled for now - this might make it too easy to mess up the database
//   // while developing/testing code.
//   /*
//    * Allow the values of user properties to be set as object members.
//    */
// 	public function __set($n,$v) 
// 	{
// 		$this->setInfo(array($n => $v));
// 	}
// 	
	
  /*
	 * Construct the object by querying for the record in the
	 * users table that matches the username supplied to the 
	 * object constructor:
	 */
	public function __construct($sUsername=false)
	{
		$this->_db = Zend_Registry::get("db");
		$this->_config = Zend_Registry::get("config");
		
		if ($sUsername!==false && strlen($sUsername)>0)
		{
			$user = $this->_db->fetchRow('SELECT * FROM users WHERE username = ? LIMIT 1',$sUsername);
			if ($user)
			{
				$this->_aUserInfo = $user;
				$this->_bExists = true;
			}
			
			// Even if the user does not exist, we should at least
			// store the username, so we know what name the object
			// was initialized with:
			else
			{
				$this->_aUserInfo = array("username"=>$sUsername);
			}
		}
	}
	
	/*
	 * Returns true if there is an array saved in the aUserInfo
	 * property (i.e., either the user existed when the object
	 * was instantiated, or it has since been created).
	 */
	public function exists()
	{
		return $this->_bExists;
	}
	
	/*
	 * Checks if this user has administrative rights. Since this
	 * is likely to be checked frequently, an effort is made to
	 * only query the database once.
	 */
	public function isAdmin()
	{
		// If we've already checked once before, then don't check again...
		if (!is_null($this->_bIsAdmin))
			return $this->_bIsAdmin;
		
		// If the user doesn't exist, then this is a definite false...
		elseif (!$this->exists())
		{
			return ($this->_bIsAdmin = false);
			
		}
		
		// Check to see if the user is a member of a group with is_admin = true
		else
		{
			
			// If this query returns any rows, then the user is an admin.
			$isAdmin = $this->_db->fetchRow("select groupname from groups left join group_user using (groupname) left join users using (username) where is_admin and username = ? limit 1",$this->_aUserInfo["username"]);
			 
			if ($isAdmin)
				$this->_bIsAdmin = true;
			else
				$this->_bIsAdmin = false;
			
			return $this->_bIsAdmin;
		}
	}
	
	/*
	 *	Checks if the user has a permission:
	 */
	public function hasPermission($sPermission)
	{
		if (!isset($this->_aPermissions[$sPermission]))
		{
			$pgCheck = $this->_db->fetchRow("select groupname from groups left join group_user using (groupname) left join users using (username) where ".$sPermission." and username = ? limit 1",$this->_aUserInfo["username"]);
			$this->_aPermissions[$sPermission] = ($this->isAdmin()||(is_array($pgCheck) && count($pgCheck)>0));
		}
		return	$this->_aPermissions[$sPermission];
	}
	
	/*
	 *	Checks if the user has permissions to create new groups.
	 */
	public function canCreateGroups()
	{
		return $this->hasPermission("can_create_groups");
	}
	
	/*
	 *	Checks if the user has permissions to create new users.
	 */
	public function canCreateUsers()
	{	
		return $this->hasPermission("can_create_users");
	}
	
	/*
	 *	Checks if the user has permissions to create new discussions.
	 */
	public function canCreateDiscussions()
	{
		return $this->hasPermission("can_create_discussions");
	}
	
	
	/*
	 *	Checks if the user has permissions to manage existing groups.
	 */
	public function canManageGroups()
	{
		return $this->hasPermission("can_manage_groups");
	}
	
	/*
	 *	Checks if the user has permissions to manage existing users
	 */
	public function canManageUsers()
	{
		return $this->hasPermission("can_manage_users");
	}
	
	/*
	 *	Checks if the user has permissions to manage existing discussions
	 */
	public function canManageDiscussions()
	{
		return $this->hasPermission("can_manage_discussions");
	}
	
	/*
	 * Checks if the user has any admin-related permissions:
	 */
	function hasSomeAdmin()
	{
		return ($this->isAdmin()||$this->canCreateDiscussions()||$this->canCreateGroups()||$this->canCreateUsers()||
			$this->canManageDiscussions()||$this->canManageGroups()||$this->canManageUsers());
	}
	
	/*
	 * This function returns info from the user's record in users.
	 */
	public function getInfo($sID=false)
	{
		// If an ID is provided that references a column in the users
		// table, then return only that particular attribute...
		if ($sID!=false)
		{
			if (isset($this->_aUserInfo[$sID]))
				return $this->_aUserInfo[$sID];
			
			// If an invalid column ID was supplied:
			else
				return false;
		}
		
		// If nothing is supplied for a column ID, then return the full
		// user's record as an array:
		else
		{
			return $this->_aUserInfo;
		}
	}
	
	/*
	 * This function updates attributes of the user in the database.
	 * the $aInfo must be an array with values referenced by keys that
	 * correspond to the columns in users.
	 */
	public function setInfo($aInfo)
	{
		if (!$this->exists())
		{
			throw new MapChat_Exception("Cannot update non-existent user."); 
		}
		
		if (is_array($aInfo)&&count($aInfo)>0)
		{
			// Get the users table metatada:
			$aTblUsers = $this->_db->describeTable("users","public");
			
			// Loop through the supplied values...
			foreach ($aInfo as $sKey => $sValue)
			{
				// If one of the variables doesn't match, throw an exception...
				if (!isset($aTblUsers[$sKey])) 
				{
					throw new MapChat_Exception("Invalid column specified: ".$sKey);
				}
				elseif ($aTblUsers[$sKey]["PRIMARY"]) 
				{
					throw new MapChat_Exception("Altering the column '".$sKey."' is not allowed, as it is a primary key.");
				}
			}
			
			// If one of the items is the password, make sure it gets encrypted:
			if (isset($aInfo["password"]))
			{
				if (strlen($aInfo["password"])>5)
				{
					$aInfo["password"] = new Zend_Db_Expr($this->_db->quoteInto('md5(?)',$aInfo["password"]));
				}
				else
				{
					throw new MapChat_Exception("The password must be at least 6 characters in length.");
				}
			}
			
			// Execute the update:
			$this->_db->update("users",$aInfo,$this->_db->quoteInto("username = ?",$this->_aUserInfo["username"]));
			
			$this->_aUserInfo = $this->_db->fetchRow('SELECT * FROM users WHERE username = ? LIMIT 1',$this->_aUserInfo["username"]);
			
		}
		
		// The input wasn't an array, or was empty:
		else
		{
			throw new MapChat_Exception("Invalid input for MapChat_User::setInfo() - must be an array.");
		}
		
	}
	
	/*
	 * This function returns either full_name or username (if full_name is not
	 * available) - useful for getting an appropriate display name.
	 */
	public function displayName()
	{
		return @((isset($this->_aUserInfo["full_name"]) && strlen($this->_aUserInfo["full_name"]))>0?$this->_aUserInfo["full_name"]:$this->_aUserInfo["username"]);
	}
	
	/*
	 * This function can be used to create new user accounts.  If
	 * $bDoRegistration is false, then the user will simply be created,
	 * othewise, the usual registration process is initiated (i.e., using
	 * email confirmation, etc.). The $aUserInfo parameter should be an
	 * array containing values with keys that match the columns of the
	 * user table in the MapChat2 database.
	 */
	public function create($aUserInfo,$bDoRegistration=true)
	{
		// Before doing anything, cleanup any old registration attempts that 
		// are incomplete and are older than 24 hours. This should leave existing
		// (i.e., verified) users and users that have attempted registration
		// within the past 24 hours untouched:
		$this->cleanupRegistration();
		
		// The username is optional, provided that one was supplied when 
		// the current instance of the MapChat_User object was created...
		if (!isset($aUserInfo["username"]))
		{
			if ($this->_aUserInfo["username"])
			{
				$aUserInfo["username"] = $this->_aUserInfo["username"];
			}
			else
			{
				throw new MapChat_Exception("A username must be specified in order to create a new user");
			}
		}
		
		$tmpUser = new MapChat_User($aUserInfo["username"]);
		if ($tmpUser->exists())
		{
			throw new MapChat_Exception("The user '".$aUserInfo["username"]."' already exists.");
		}
		
		// If the registration process is being skipped, then the user
		// account will automatically be set as 'confirmed':
		$aUserInfo["confirmed"] = (integer)!$bDoRegistration;
		
		// Verify that the required parameters username, password, and email are all OK:
		if (strlen($aUserInfo["username"])<3)
		{
			throw new MapChat_Exception("The username '".$aUserInfo["username"]."' is too short.  Please use three or more characters.");
		}
		if (!isset($aUserInfo["password"])||strlen($aUserInfo["password"])<6)
		{
			throw new MapChat_Exception("A password of six or more characters must be provided.");
		}
		if ($bDoRegistration&&(!isset($aUserInfo["email"])||strlen($aUserInfo["email"])<0))
		{
			// TODO: add a check to make sure the email is valid?
			throw new MapChat_Exception("Please supply a valid email address.");
		}
		
		$aUserInfo["password"] = new Zend_Db_Expr($this->_db->quoteInto('md5(?)',$aUserInfo["password"]));
		
		// Get the users table metatada:
		$aTblUsers = $this->_db->describeTable("users","public");
		
		// Loop through the supplied values...
		foreach ($aUserInfo as $sKey => $sValue)
		{
			// If one of the variables doesn't match, throw an exception...
			if (!isset($aTblUsers[$sKey])) 
			{
				throw new MapChat_Exception("Invalid column specified: ".$sKey);
			}
		}
		
		$this->_db->insert('users', $aUserInfo);
		
		$this->_bExists = true;
		
		$this->_aUserInfo = $this->_db->fetchRow('SELECT * FROM users WHERE username = ? LIMIT 1',$aUserInfo["username"]);
		
		if ($bDoRegistration)
		{
			// Make sure an email was supplied - might also be worthwhile to validate the email:
			if (!isset($aUserInfo["email"]))
			{
				throw new MapChat_Exception("An email is required in order to complete the registration process.");
			}
			
			// Determine the approriate display name for the user:
			$sDisplayName = (strlen($aUserInfo["full_name"])>0?$aUserInfo["full_name"]:$aUserInfo["username"]);
			
			// Get a unique string for a conformation identifier...
			$sConfCode = md5(uniqid(rand()));
			
			// Add the registration record...
			$this->_db->insert("registration",array("username" => $aUserInfo["username"], "code" => $sConfCode));
			
			$mail = new Zend_Mail();
			
			// Form a URI that can be given to link back to the registration controller
			// on this server (assumes the MapChat web service is configured to rewrite
			// all URLs to point to /web_path/index.php):
			$sURI = ((isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"]=="on")?"https://":"http://")
				.$_SERVER["HTTP_HOST"].dirname($_SERVER["SCRIPT_NAME"]).
				"/registration/confirm?c=".$sConfCode;
				
			// Put together the content of the email to be sent to the user:
			$mail->setSubject("MapChat account registration");
			$mail->setBodyText("Hello ".$sDisplayName.",

Welcome to MapChat.

This email is being sent to you because a new account has been
registered using your email address.

In order to validate your account, please navigate to the link
below in a web browser, where you will be prompted to re-enter
the password that you chose for the account in order to verify
your identity:

".$sURI."

===================

If you did not register for this account and/or this email is
being sent to you in error, you may disregard this message. If
any problems persist, please contact the website maintainer at
".$this->_config->admin_email.".");
				
			$mail->setFrom($this->_config->admin_email, 'MapChat Registration');
			$mail->addTo($aUserInfo["email"], $sDisplayName);
			$mail->send();
		}
		else
		{
			// Add the user to everyone:
			$everyone = new MapChat_Group("everyone");
			$everyone->addUser($this);
		}
	}
	
	/*
	 *	This function validates the confirmation code is valid for a
	 * newly registered user.
	 */
	public function confirmRegistration($sCode)
	{
		// First, cleanup any unconfirmed users or registration records that
		// are older than 24 hours:
		$this->cleanupRegistration();
		
		$reg = $this->_db->fetchRow("select * from users where username in (select username from registration where code = ?) limit 1",$sCode);
		
		if ($reg)
		{
			$this->_aUserInfo = $reg;
			$this->_bExists = true;
			return ($this->_bExists && $this->_aUserInfo["confirmed"]!="t");
		}
		else
		{
			return false;
		}
	}
	
	
	/*
	 *	Cleans up old registrations
	 */
	public function cleanupRegistration()
	{
		// First, cleanup any unconfirmed users or registration records that
		// are older than 24 hours:
		$this->_db->beginTransaction();
		try {
			$this->_db->delete("users","username in (select username from registration where age(now(),creation_date)>='24 hours')");
			$this->_db->delete("registration","age(now(),creation_date)>='24 hours'");
		} catch (Exception $e) {
			$this->_db->rollBack();
			throw new MapChat_Exception($e->getMessage(),$e->getCode());
		}
		$this->_db->commit();
	}
	
	
	/*
	 *	This function finalizes the user registration process
	 */
	public function completeRegistration($sCode,$sPassword)
	{
		// Verify the code again...return -1 if it is no longer valid:
		if (!$this->confirmRegistration($sCode))
		{
			throw new MapChat_Exception("Invalid registration code '".$sCode."'.");
		}
		
		// Attempt authentication to verify the user's identity:
		$auth = new MapChat_Auth();
		$result = $auth->authenticate($this->_aUserInfo["username"],$sPassword,false);

		if ($result->isValid())
		{
			// Update the confirmed status of the user:
			$this->setInfo(array("confirmed"=>true));
			$this->_aUserInfo["confirmed"] = true;
			
			// Delete the registration record:
			$this->_db->delete("registration",array(
				$this->_db->quoteInto("username = ?",$this->_aUserInfo["username"]),
				$this->_db->quoteInto("code = ?",$sCode)
			));
			
			// Add the user to 'everyone':
			$everyone = new MapChat_Group("everyone");
			$everyone->addUser($this);
		}
		
		// Must have been an invalid password:
		else
		{
			throw new MapChat_Exception("The password you entered was incorrect.");
		}
	}
	
	/*
	 * This function begins the process for resetting passwords.
	 */
	public function forgotPassword()
	{
		// Make sure an email was supplied - might also be worthwhile to validate the email:
		if (!isset($this->_aUserInfo["email"])||strlen($this->_aUserInfo["email"])==0)
		{
			throw new MapChat_Exception("An email is required in order to complete the registration process.");
		}
		
		// Get a unique string for a conformation identifier...
		$sConfCode = md5(uniqid(rand()));
		
		// Add the registration record...
		$this->_db->insert("forgotpassword",array("username" => $this->_aUserInfo["username"], "code" => $sConfCode));
			
		$mail = new Zend_Mail();
			
		// Form a URI that can be given to link back to the registration controller
		// on this server (assumes the MapChat web service is configured to rewrite
		// all URLs to point to /web_path/index.php):
		$sURI = ($_SERVER["HTTPS"]=="on"?"https://":"http://")
			.$_SERVER["HTTP_HOST"].dirname($_SERVER["SCRIPT_NAME"]).
			"/forgot?c=".$sConfCode;
			
		// Put together the content of the email to be sent to the user:
		$mail->setSubject("MapChat account registration");
		$mail->setBodyText("Hello ".$this->displayName().",

This email is being sent to you because a request was made to
reset the password for your account.  In order to complete
this process, please visit this link and follow the
instructions:

".$sURI."

===================

If you did not request a new password for this account,
please you may disregard this message. If any problems 
persist, please contact the website maintainer at
".$this->_config->admin_email.".");
				
		$mail->setFrom($this->_config->admin_email, 'MapChat Password Reset');
		$mail->addTo($this->_aUserInfo["email"], $this->displayName());
		$mail->send();
	}
	
	/*
	 * This function verifies the confirmation code for a password reset request:
	 */
	public function confirmForgot($sCode)
	{
		// First, cleanup any unconfirmed password reset request records that
		// are older than 24 hours:
		$this->_db->delete("forgotpassword","age(now(),creation_date)>='24 hours'");
		
		$forgot = $this->_db->fetchRow("select * from users where username in (select username from forgotpassword where code = ?) limit 1",$sCode);
		
		if ($forgot )
		{
			$this->_aUserInfo = $forgot;
			$this->_bExists = true;
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/*
	 * This function completes the password reset process
	 */
	public function completeForgot($sCode,$sPassword)
	{
		// Verify the code again...return -1 if it is no longer valid:
		if (!$this->confirmForgot($sCode))
		{
			throw new MapChat_Exception("Invalid forgotpassword code '".$sCode."'.");
		}
		
		// Save the supplied password to the user's account:
		$this->setInfo(array("password"=>$sPassword));
		
		// Delete the password reset request record:
		$this->_db->delete("forgotpassword",array(
			$this->_db->quoteInto("username = ?",$this->_aUserInfo["username"]),
			$this->_db->quoteInto("code = ?",$sCode)
		));
	}
	
	/*
	 * This function checks whether a string matches the user's current
	 * password (e.g., for verifying actions such as editing a user's
	 * profile)
	 */
	public function checkPassword($sPassword)
	{
		return $this->_aUserInfo["password"]==md5($sPassword);
	}
	
	/*
	 * List all discussions that this user is a member of, with recursion into
	 * group memberships optional, and filtering for readonly/non-readonly status
	 * also an option:
	 *
	 */
	public function listDiscussions($recurse=true,$readonly=NULL)
	{
		$ld_select = $this->_db->select()->from("user_discussion",array("discussion"))->joinUsing("discussions","discussion",array())->where("username = ?",$this->username)->where('not disabled');
		if (!is_null($readonly))
		{
			$ld_select->where("readonly = ?::boolean",(integer)$readonly);
		}
		$ld = $this->_db->fetchCol($ld_select);
		
		if ($recurse)
		{
			$lgd_select = $this->_db->select()->from("group_discussion",array("discussion"))->joinUsing("group_user","groupname",array())->joinUsing("discussions","discussion",array())->where("username = ?",$this->username)->where('not disabled');
			if (!is_null($readonly))
			{
				$lgd_select->where("readonly = ?::boolean",(integer)$readonly);
			}
			$lgd = $this->_db->fetchCol($lgd_select);
			$ld = array_unique(array_merge($ld,$lgd));
			
					
			// If we're looking for readonly users, then we want to exclude anyone that
			// is also a member via groups that are not readonly:
			if ($recurse && $readonly)
			{
				$ld = array_diff($ld,$this->listUsers($recurse,false,false));
			}
		}
		
		return $ld;
	}
	
	/*
	 * List all groups that this user is a member of:
	 *
	 */
	public function listGroups()
	{
		$lu_select = $this->_db->select()->distinct()->from("group_user",array("groupname"))->joinUsing("groups","groupname",array())->where("username = ?",$this->username)->where('not disabled');
		
		$lu = $this->_db->fetchCol($lu_select);
		
		return $lu;
	}
	
	/*
	 * List all users in the current database:
	 */
	public function listAllUsers()
	{
		return Zend_Registry::get("db")->fetchAll("select * from users order by username;");
	}
}

?>
