<?php

// Temp folder (for storing php sessions, cached js files, etc.).
$tmpDir = "/var/www/tmp";

// Ram folder (if set, this can be used instead of tmpDir for temporary files).
$ramDir = "/media/rd";

// Turning off cookies for now - sid has to be passed via POST/GET, but allows multiple sessions to be tested simultaneously within a single browser session (very useful for testing MapChat).  Any GET/POST requests should omit the sid variable if this is set to true (i.e., to take advantage of better security using browser cookies for session/user identification):
$useCookies = false;

return array(
	// Debug Mode? (if this is true and minify is false, the index view will load all scripts in the header, rather than on-demand)
	"debug"		=> true,

	// Redirect non-HTTPS connections (for the main page ONLY)?
	"useHTTPS" => false,

	// Minify JS/CSS? (if this is false and debug is true, the index view will load all scripts in the header, rather than on-demand)
	"minify"		=> true,

	// Base Dir
	"BaseDir"	=> str_replace("\\","/",dirname(dirname(__FILE__))), //"/var/www/mapchat2",

	// Lib Dir
	"LibDir"		=> str_replace("\\","/",dirname(dirname(__FILE__)))."/library", //"/var/www/mapchat2/library",

	// MVC Dir
	"MVCDir"		=> str_replace("\\","/",dirname(dirname(__FILE__)))."/MapChat", //"/var/www/mapchat2/MapChat",

	// Writeable temp folder
	"tmpDir"		=> ((file_exists($ramDir)&&is_writable($ramDir))?$ramDir:$tmpDir),
	
	// Writeable & web-accessible temp folder (e.g., where MapServer images will be saved).
	"tmpWeb"		=> $tmpDir."/ms_tmp",

	// URL to reach tmpWeb:
	"tmpWebURL" => "/ms_tmp",

	// The sub-folder name for containing Ka-Map cached tiles (must have slashes on both ends).
	"cacheName" => "/mccache/",

	// DB Parameters
	"database"	=> array(
		'host' => 'localhost',
		'port' => 5432,
		'username' => 'postgres',
		'password' => '',
		'dbname' => 'mapchat2',
		'options' => array(Zend_Db::AUTO_QUOTE_IDENTIFIERS => false),
	),
	
	// Locale-related settings for the database:
	"db_encoding" => "UNICODE",
	"db_tsconfig" => "english",
	
	"useCookies" => $useCookies,
	
	// Session-related INI parameters:
	"session" => array(
		'use_cookies' => ($useCookies?'1':'0'),
		'use_only_cookies' => ($useCookies?'1':'0'),
		'name' => 'sid',
		'save_path' => ((file_exists($ramDir)&&is_writable($ramDir))?$ramDir:$tmpDir),
	),
	
	// Mail-related settings:
	"admin_email" => "mgleahy@alumni.uwaterloo.ca",
	
	// Filenames of PHP modules needed:
	"modules" => array(
		"mapscript" => "php_mapscript.".PHP_SHLIB_SUFFIX,
		"gd" => "php_gd.".PHP_SHLIB_SUFFIX,
	),
	
	// Ka-Map Configuration:
	"KaMapConfig" => array(
		"tileWidth" => 400,
		"tileHeight" => 300,
		"metaWidth" => 3,
		"metaHeight" => 3,
		"metaBuffer" => 200,
		"debug" => false,
	),
	
	// Image-returning API functions:
	"imgAPIs" => array(
		"Map_ClassIcon" => true,
	),

	// Captcha parameters
	"CaptchaConfig" => array(
		"timeout" => 300,
		"wordLen" => 6,
		"font" => str_replace("\\","/",dirname(dirname(__FILE__)))."/etc/VeraMoBd.ttf", //"/var/www/mapchat2/etc/VeraSe.ttf",
		"expiration" => 600,
		"fontSize" => 18,
		"height" => 40,
		"width" => 150,
		"imgDir" => $tmpDir."/ms_tmp",
		"imgUrl" => "/ms_tmp",
	),
);

?>
